@@ -14,7 +14,7 @@
  * for more details.
  */
 
-char ipsec_init_c_version[] = "RCSID $Id: ipsec_init.c,v 1.41 1999/11/23 23:04:03 rgb Exp $";
+char ipsec_init_c_version[] = "RCSID $Id: ipsec_init.c,v 1.48 2000/05/11 18:26:10 rgb Exp $";
 
 #include <linux/config.h>
 #include <linux/version.h>
@@ -67,10 +67,10 @@ char ipsec_init_c_version[] = "RCSID $Id: ipsec_init.c,v 1.41 1999/11/23 23:04:0
 #include "ipsec_ah.h"
 #include "ipsec_esp.h"
 
-#ifdef CONFIG_IPSEC_PFKEYv2
+/* #ifdef CONFIG_IPSEC_PFKEYv2 */
 #include <pfkeyv2.h>
 #include <pfkey.h>
-#endif
+/* #endif */
 
 #ifdef DEBUG_IPSEC
 int debug_eroute = 0;
@@ -84,7 +84,11 @@ unsigned int ipsec_spi_next = 0x1000;
 static
 #endif /* PROC_FS_2325 */
 int
-ipsec_eroute_get_info(char *buffer, char **start, off_t offset, int length, int dummy)
+ipsec_eroute_get_info(char *buffer, char **start, off_t offset, int length
+#ifndef PROC_NO_DUMMY
+, int dummy
+#endif /* !PROC_NO_DUMMY */
+)
 {
 	struct wsbuf w = {buffer, length, offset, 0, 0, 0, 0};
 	unsigned long flags; /* save irq state for spinlock */
@@ -117,7 +121,11 @@ ipsec_eroute_get_info(char *buffer, char **start, off_t offset, int length, int
 static
 #endif /* PROC_FS_2325 */
 int
-ipsec_spi_get_info(char *buffer, char **start, off_t offset, int length, int dummy)
+ipsec_spi_get_info(char *buffer, char **start, off_t offset, int length
+#ifndef  PROC_NO_DUMMY
+, int dummy
+#endif /* !PROC_NO_DUMMY */
+)
 {
 	int len = 0;
 	off_t pos = 0, begin = 0;
@@ -139,6 +147,9 @@ ipsec_spi_get_info(char *buffer, char **start, off_t offset, int length, int dum
 		for (tdbp = tdbh[i]; tdbp; tdbp = tdbp->tdb_hnext) {
 			sa_len = satoa(tdbp->tdb_said, 0, sa, SATOA_BUF);
 			len += sprintf(buffer + len, "%s ", sa);
+#if 1
+			len += sprintf(buffer + len, "%s%s%s", TDB_XFORM_NAME(tdbp));
+#else
 			switch(tdbp->tdb_proto) {
 			case IPPROTO_AH:
 				len += sprintf(buffer + len, "AH");
@@ -149,11 +160,11 @@ ipsec_spi_get_info(char *buffer, char **start, off_t offset, int length, int dum
 			case IPPROTO_IPIP:
 				len += sprintf(buffer + len, "IPIP");
 				break;
-#if 0 /* don't get your hopes up just yet... */
+#if CONFIG_IPCOMP /* don't get your hopes up just yet... */
 			case IPPROTO_COMP:
 				len += sprintf(buffer + len, "COMP");
 				break;
-#endif
+#endif /* CONFIG_IPCOMP */
 			default:
 				len += sprintf(buffer + len, "UNKNOWN_proto");
 			}
@@ -201,9 +212,9 @@ ipsec_spi_get_info(char *buffer, char **start, off_t offset, int length, int dum
 				len += sprintf(buffer + len, "_auth");
 #endif
 			}
+#endif
 
-			len += sprintf(buffer + len, ":");
-			len += sprintf(buffer + len, " dir=%s",
+			len += sprintf(buffer + len, ": dir=%s",
 				       (tdbp->tdb_flags & EMT_INBOUND) ?
 				       "in" : "out");
 			if(tdbp->tdb_iv_bits) {
@@ -216,17 +227,19 @@ ipsec_spi_get_info(char *buffer, char **start, off_t offset, int length, int dum
 				}
 			}
 			if(tdbp->tdb_encalg || tdbp->tdb_authalg) {
-				len += sprintf(buffer + len, " ooowin=%d",
-					       tdbp->tdb_replaywin);
+				if(tdbp->tdb_replaywin) {
+					len += sprintf(buffer + len, " ooowin=%d",
+						       tdbp->tdb_replaywin);
+				}
 				if(tdbp->tdb_replaywin_errs) {
 					len += sprintf(buffer + len, " ooo_errs=%d",
 						       tdbp->tdb_replaywin_errs);
 				}
-				if(/* tdbp->tdb_replaywin */ 1) {
+				if(tdbp->tdb_replaywin_lastseq) {
                                        len += sprintf(buffer + len, " seq=%d",
 						      tdbp->tdb_replaywin_lastseq);
 				}
-				if(tdbp->tdb_flags & EMT_INBOUND) {
+				if(tdbp->tdb_replaywin_bitmap) {
 					len += sprintf(buffer + len, " bit=0x%Lx",
 						       tdbp->tdb_replaywin_bitmap);
 				}
@@ -235,19 +248,19 @@ ipsec_spi_get_info(char *buffer, char **start, off_t offset, int length, int dum
 						       tdbp->tdb_replaywin_maxdiff);
 				}
 			}
-			len += sprintf(buffer + len, " flags=0x%x",
-				       tdbp->tdb_flags & ~EMT_INBOUND);
 			if(tdbp->tdb_flags & ~EMT_INBOUND) {
+				len += sprintf(buffer + len, " flags=0x%x",
+					       tdbp->tdb_flags & ~EMT_INBOUND);
 				len += sprintf(buffer + len, "<");
 				/* flag printing goes here */
 				len += sprintf(buffer + len, ">");
 			}
 			if(tdbp->tdb_auth_bits) {
-				len += sprintf(buffer + len, " alen=%dbits",
+				len += sprintf(buffer + len, " alen=%d",
 					       tdbp->tdb_auth_bits);
 			}
 			if(tdbp->tdb_key_bits_a) {
-				len += sprintf(buffer + len, " aklen=%dbits",
+				len += sprintf(buffer + len, " aklen=%d",
 					       tdbp->tdb_key_bits_a);
 			}
 			if(tdbp->tdb_auth_errs) {
@@ -255,7 +268,7 @@ ipsec_spi_get_info(char *buffer, char **start, off_t offset, int length, int dum
 					       tdbp->tdb_auth_errs);
 			}
 			if(tdbp->tdb_key_bits_e) {
-				len += sprintf(buffer + len, " eklen=%dbits",
+				len += sprintf(buffer + len, " eklen=%d",
 					       tdbp->tdb_key_bits_e);
 			}
 			if(tdbp->tdb_encsize_errs) {
@@ -276,30 +289,54 @@ ipsec_spi_get_info(char *buffer, char **start, off_t offset, int length, int dum
 					       buf_s, buf_d);
 			}
 
+#if 0
 			len += sprintf(buffer + len, " lifetimes(current,soft,hard)=");
-			len += sprintf(buffer + len, "alloc(%d,%d,%d)",
+#else
+			len += sprintf(buffer + len, " life(c,s,h)=");
+#endif
+			if(tdbp->tdb_lifetime_allocations_c > 1 || 
+			   tdbp->tdb_lifetime_allocations_s ||
+			   tdbp->tdb_lifetime_allocations_h) {
+				len += sprintf(buffer + len, "alloc(%d,%d,%d)",
 					       tdbp->tdb_lifetime_allocations_c,
 					       tdbp->tdb_lifetime_allocations_s,
 					       tdbp->tdb_lifetime_allocations_h);
-			len += sprintf(buffer + len, ",bytes(%Ld,%Ld,%Ld)",
+			}
+			if(tdbp->tdb_lifetime_bytes_c ||
+			   tdbp->tdb_lifetime_bytes_s ||
+			   tdbp->tdb_lifetime_bytes_h) {
+				len += sprintf(buffer + len, "bytes(%Ld,%Ld,%Ld)",
 					       tdbp->tdb_lifetime_bytes_c,
 					       tdbp->tdb_lifetime_bytes_s,
 					       tdbp->tdb_lifetime_bytes_h);
-			len += sprintf(buffer + len, ",add(%Ld,%Ld,%Ld)",
+			}
+			if(tdbp->tdb_lifetime_addtime_c ||
+			   tdbp->tdb_lifetime_addtime_s ||
+			   tdbp->tdb_lifetime_addtime_h) {
+				len += sprintf(buffer + len, "add(%Ld,%Ld,%Ld)",
 					       tdbp->tdb_lifetime_addtime_c,
 					       tdbp->tdb_lifetime_addtime_s,
 					       tdbp->tdb_lifetime_addtime_h);
-			len += sprintf(buffer + len, ",use(%Ld,%Ld,%Ld)",
+			}
+			if(tdbp->tdb_lifetime_usetime_c ||
+			   tdbp->tdb_lifetime_usetime_s ||
+			   tdbp->tdb_lifetime_usetime_h) {
+				len += sprintf(buffer + len, "use(%Ld,%Ld,%Ld)",
 					       tdbp->tdb_lifetime_usetime_c,
 					       tdbp->tdb_lifetime_usetime_s,
 					       tdbp->tdb_lifetime_usetime_h);
-			len += sprintf(buffer + len, ",packets(%Ld,%Ld,%Ld)",
+			}
+			if(tdbp->tdb_lifetime_packets_c ||
+			   tdbp->tdb_lifetime_packets_s ||
+			   tdbp->tdb_lifetime_packets_h) {
+				len += sprintf(buffer + len, "packets(%Ld,%Ld,%Ld)",
 					       tdbp->tdb_lifetime_packets_c,
 					       tdbp->tdb_lifetime_packets_s,
 					       tdbp->tdb_lifetime_packets_h);
+			}
 
 			if(tdbp->tdb_lifetime_usetime_c) {
-				len += sprintf(buffer + len, " last_used=%Ld",
+				len += sprintf(buffer + len, " idle=%Ld",
 					       jiffies / HZ - tdbp->tdb_lifetime_usetime_l);
 			}
 
@@ -330,7 +367,11 @@ ipsec_spi_get_info(char *buffer, char **start, off_t offset, int length, int dum
 static
 #endif /* PROC_FS_2325 */
 int
-ipsec_spigrp_get_info(char *buffer, char **start, off_t offset, int length, int dummy)
+ipsec_spigrp_get_info(char *buffer, char **start, off_t offset, int length
+#ifndef PROC_NO_DUMMY
+, int dummy
+#endif /* !PROC_NO_DUMMY */
+)
 {
 	int len = 0;
 	off_t pos = 0, begin = 0;
@@ -387,7 +428,11 @@ ipsec_spigrp_get_info(char *buffer, char **start, off_t offset, int length, int
 static
 #endif /* PROC_FS_2325 */
 int
-ipsec_tncfg_get_info(char *buffer, char **start, off_t offset, int length, int dummy)
+ipsec_tncfg_get_info(char *buffer, char **start, off_t offset, int length
+#ifndef PROC_NO_DUMMY
+, int dummy
+#endif /* !PROC_NO_DUMMY */
+)
 {
 	int len = 0;
 	off_t pos = 0, begin = 0;
@@ -403,7 +448,7 @@ ipsec_tncfg_get_info(char *buffer, char **start, off_t offset, int length, int d
 
 	for(i = 0; i < IPSEC_NUM_IF; i++) {
 		sprintf(name, "ipsec%d", i);
-		dev = dev_get(name);
+		dev = ipsec_dev_get(name);
 		if(dev) {
 			priv = (struct ipsecpriv *)(dev->priv);
 			len += sprintf(buffer + len, "%s",
@@ -442,7 +487,11 @@ ipsec_tncfg_get_info(char *buffer, char **start, off_t offset, int length, int d
 static
 #endif /* PROC_FS_2325 */
 int
-ipsec_spi_get_new(char *buffer, char **start, off_t offset, int length, int dummy)
+ipsec_spi_get_new(char *buffer, char **start, off_t offset, int length
+#ifndef PROC_NO_DUMMY
+, int dummy
+#endif /* !PROC_NO_DUMMY */
+)
 {
 
 	int len = 0;
@@ -477,7 +526,11 @@ ipsec_spi_get_new(char *buffer, char **start, off_t offset, int length, int dumm
 static
 #endif /* PROC_FS_2325 */
 int
-ipsec_version_get_info(char *buffer, char **start, off_t offset, int length, int dummy)
+ipsec_version_get_info(char *buffer, char **start, off_t offset, int length
+#ifndef PROC_NO_DUMMY
+, int dummy
+#endif /* !PROC_NO_DUMMY */
+)
 {
 
 	int len = 0;
@@ -506,7 +559,11 @@ ipsec_version_get_info(char *buffer, char **start, off_t offset, int length, int
 static
 #endif /* PROC_FS_2325 */
 int
-ipsec_klipsdebug_get_info(char *buffer, char **start, off_t offset, int length, int dummy)
+ipsec_klipsdebug_get_info(char *buffer, char **start, off_t offset, int length
+#ifndef PROC_NO_DUMMY
+, int dummy
+#endif /* !PROC_NO_DUMMY */
+)
 {
 
 	int len = 0;
@@ -526,9 +583,9 @@ ipsec_klipsdebug_get_info(char *buffer, char **start, off_t offset, int length,
 	len += sprintf(buffer + len, "debug_esp=%08x.\n", debug_esp);
 	len += sprintf(buffer + len, "debug_ah=%08x.\n", debug_ah);
 	len += sprintf(buffer + len, "debug_rcv=%08x.\n", debug_rcv);
-#ifdef CONFIG_IPSEC_PFKEYv2
+	/* #ifdef CONFIG_IPSEC_PFKEYv2 */
 	len += sprintf(buffer + len, "debug_pfkey=%08x.\n", debug_pfkey);
-#endif
+	/* #endif */
 
 	*start = buffer + (offset - begin);	/* Start of wanted data */
 	len -= (offset - begin);			/* Start slop */
@@ -670,12 +727,14 @@ ipsec_init(void)
 
 	printk("klips_debug:ipsec_init: ipsec module loading. freeswan version: %s\n",
 	       freeswan_version);
+#if 0
 	KLIPS_PRINT(1, "klips_debug:ipsec_init: ipsec_init version: %s\n",
 		    ipsec_init_c_version);
 	KLIPS_PRINT(1, "klips_debug:ipsec_init: ipsec_tunnel version: %s\n",
 		    ipsec_tunnel_c_version);
 	KLIPS_PRINT(1, "klips_debug:ipsec_init: ipsec_netlink version: %s\n",
 		    ipsec_netlink_c_version);
+#endif
 
 #ifndef SPINLOCK
 	/* spin_lock_init( */ tdb_lock.lock = 0; /* ); */
@@ -685,11 +744,13 @@ ipsec_init(void)
 	error |= ipsec_tdbinit();
 	error |= ipsec_radijinit();
 
-#ifdef CONFIG_IPSEC_PFKEYv2
+	/* #ifdef CONFIG_IPSEC_PFKEYv2 */
 	error |= pfkey_init();
-#endif
+	/* #endif */
 
+#if 0
 	(void)netlink_attach(NETLINK_IPSEC, ipsec_callback);
+#endif
 	register_netdevice_notifier(&ipsec_dev_notifier);
 
 #ifdef CONFIG_IPSEC_ESP
@@ -727,7 +788,9 @@ ipsec_cleanup(void)
 
 	unregister_netdevice_notifier(&ipsec_dev_notifier);
 
+#if 0
 	netlink_detach(NETLINK_IPSEC);
+#endif
 
 	KLIPS_PRINT(debug_netlink, /* debug_tunnel & DB_TN_INIT, */
 		    "klips_debug:ipsec_cleanup: calling ipsec_tdbcleanup.\n");
@@ -736,11 +799,11 @@ ipsec_cleanup(void)
 		    "klips_debug:ipsec_cleanup: calling ipsec_radijcleanup.\n");
 	error |= ipsec_radijcleanup();
 	
-#ifdef CONFIG_IPSEC_PFKEYv2
+	/* #ifdef CONFIG_IPSEC_PFKEYv2 */
 	KLIPS_PRINT(debug_pfkey, /* debug_tunnel & DB_TN_INIT, */
 		    "klips_debug:ipsec_cleanup: calling pfkey_cleanup.\n");
 	error |= pfkey_cleanup();
-#endif
+	/* #endif */
 
 #ifdef CONFIG_PROC_FS
 #  ifndef PROC_FS_2325
@@ -805,6 +868,31 @@ cleanup_module(void)
 
 /*
  * $Log: ipsec_init.c,v $
+ * Revision 1.48  2000/05/11 18:26:10  rgb
+ * Commented out calls to netlink_attach/detach to avoid activating netlink
+ * in the kenrel config.
+ *
+ * Revision 1.47  2000/05/10 22:35:26  rgb
+ * Comment out most of the startup version information.
+ *
+ * Revision 1.46  2000/03/22 16:15:36  rgb
+ * Fixed renaming of dev_get (MB).
+ *
+ * Revision 1.45  2000/03/16 06:40:48  rgb
+ * Hardcode PF_KEYv2 support.
+ *
+ * Revision 1.44  2000/01/22 23:19:20  rgb
+ * Simplified code to use existing macro TDB_XFORM_NAME().
+ *
+ * Revision 1.43  2000/01/21 06:14:04  rgb
+ * Print individual stats only if non-zero.
+ * Removed 'bits' from each keylength for brevity.
+ * Shortened lifetimes legend for brevity.
+ * Changed wording from 'last_used' to the clearer 'idle'.
+ *
+ * Revision 1.42  1999/12/31 14:57:19  rgb
+ * MB fix for new dummy-less proc_get_info in 2.3.35.
+ *
  * Revision 1.41  1999/11/23 23:04:03  rgb
  * Use provided macro ADDRTOA_BUF instead of hardcoded value.
  * Sort out pfkey and freeswan headers, putting them in a library path.
