@@ -14,7 +14,7 @@
  * for more details.
  */
 
-char ipsec_rcv_c_version[] = "RCSID $Id: ipsec_rcv.c,v 1.37 1999/12/14 20:07:59 rgb Exp $";
+char ipsec_rcv_c_version[] = "RCSID $Id: ipsec_rcv.c,v 1.48 2000/05/10 19:14:40 rgb Exp $";
 
 #include <linux/config.h>
 #include <linux/version.h>
@@ -62,9 +62,13 @@ char ipsec_rcv_c_version[] = "RCSID $Id: ipsec_rcv.c,v 1.37 1999/12/14 20:07:59
 #include "ipsec_esp.h"
 #endif /* !CONFIG_IPSEC_ESP */
 
-#ifdef CONFIG_IPSEC_PFKEYv2
+/* #ifdef CONFIG_IPSEC_PFKEYv2 */
 #include <pfkeyv2.h>
 #include <pfkey.h>
+/* #endif */
+
+#if 0
+#define INBOUND_POLICY_CHECK
 #endif
 
 #ifdef DEBUG_IPSEC
@@ -94,6 +98,32 @@ ipsec_checkreplaywindow(struct tdb*tdbp, __u32 seq)
 {
 	__u32 diff;
 	
+	if (tdbp->tdb_replaywin == 0)	/* replay shut off */
+		return 1;
+	if (seq == 0) 
+		return 0;		/* first == 0 or wrapped */
+
+	/* new larger sequence number */
+	if (seq > tdbp->tdb_replaywin_lastseq) {
+		return 1;		/* larger is good */
+	}
+	diff = tdbp->tdb_replaywin_lastseq - seq;
+
+	/* too old or wrapped */ /* if wrapped, kill off SA? */
+	if (diff >= tdbp->tdb_replaywin) {
+		return 0;
+	}
+	/* this packet already seen */
+	if (tdbp->tdb_replaywin_bitmap & (1 << diff))
+		return 0;
+	return 1;			/* out of order but good */
+}
+
+static int
+ipsec_updatereplaywindow(struct tdb*tdbp, __u32 seq)
+{
+	__u32 diff;
+	
 	if (tdbp->tdb_replaywin == 0)	/* replay shut off */
 		return 1;
 	if (seq == 0) 
@@ -189,11 +219,32 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 	
 	int pad = 0, ilen, padlen, len, replay = 0;
 	__u8 *idat;
+	unsigned long tdb_flags; /* save irq state for tdb spinlock */
+#ifdef INBOUND_POLICY_CHECK
+	struct tdb* tdbprev = NULL;
+	struct tdb* tdbnext = NULL;
+#ifdef INBOUND_POLICY_CHECK_
+	unsigned long eroute_flags; /* save irq state for eroute spinlock */
+	struct sockaddr_encap matcher;	/* eroute search key */
+	struct eroute *er;
+	struct tdb* policy_tdb = NULL;
+	struct sa_id policy_said;
+	struct sockaddr_encap policy_eaddr;
+	struct sockaddr_encap policy_emask;
+#endif /* INBOUND_POLICY_CHECK */
+#endif /* INBOUND_POLICY_CHECK */
 
 	/* Don't unlink in the middle of a turnaround */
 	MOD_INC_USE_COUNT;
 	
 	if (skb == NULL) {
+		KLIPS_PRINT(debug_rcv, "klips_debug:ipsec_rcv: NULL skb passed in.\n");
+		goto rcvleave;
+	}
+		
+	if (skb->data == NULL) {
+		KLIPS_PRINT(debug_rcv, "klips_debug:ipsec_rcv: "
+			    "NULL skb->data passed in, packet is bogus, dropping.\n");
 		goto rcvleave;
 	}
 		
@@ -208,30 +259,54 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 	   a copy of our own to modify */
 	if(skb_cloned(skb)) {
 		/* include any mac header while copying.. */
+		if(skb_headroom(skb) < hard_header_len) {
+			printk(KERN_WARNING "klips_error:ipsec_rcv: "
+			       "tried to skb_push hhlen=%d, %d available.  "
+			       "This should never happen, please report.\n",
+			       hard_header_len, skb_headroom(skb));
+			goto rcvleave;
+		}
 		skb_push(skb, hard_header_len);
 		if ((skb = skb_cow(skb, skb_headroom(skb))) == NULL) {
 			goto rcvleave;
 		}
+		if(skb->len < hard_header_len) {
+			printk(KERN_WARNING "klips_error:ipsec_rcv: "
+			       "tried to skb_pull hhlen=%d, %d available.  "
+			       "This should never happen, please report.\n",
+			       hard_header_len, skb->len);
+			goto rcvleave;
+		}
 		skb_pull(skb, hard_header_len);
 	}
 	
 #endif /* NET_21 */
 		
 	KLIPS_PRINT(debug_rcv, "klips_debug:ipsec_rcv: Info -- ");
-	KLIPS_PRINT(debug_rcv && skb->dev, "sdb->dev=%s ", skb->dev->name);
-	KLIPS_PRINT(debug_rcv && dev, "dev=%s ", dev->name);
+	KLIPS_PRINT(debug_rcv && skb->dev, "sdb->dev=%s ",
+		    skb->dev->name ? skb->dev->name : "NULL");
+	KLIPS_PRINT(debug_rcv && dev, "dev=%s ",
+		    dev->name ? dev->name : "NULL");
 	KLIPS_PRINT(debug_rcv, "\n");
 
 	KLIPS_PRINT(debug_rcv && !(skb->dev && dev && (skb->dev == dev)),
 		    "klips_debug:ipsec_rcv: Informational -- "
 		    "**if this happens, find out why** "
 		    "skb->dev:%s is not equal to dev:%s\n",
-		    skb->dev->name, dev->name);
+		    skb->dev ? (skb->dev->name ? skb->dev->name : "NULL") : "NULL",
+		    dev ? (dev->name ? dev->name : "NULL") : "NULL");
 
 #ifdef NET_21
 	protoc = ((struct iphdr *)skb->data)->protocol;
 #else /* NET_21 */
-	protoc = protocol->protocol;
+	if(protocol) {
+		protoc = protocol->protocol;
+	} else {
+		KLIPS_PRINT(debug_rcv & DB_RX_TDB,
+			    "klips_debug:ipsec_rcv: "
+			    "protocol arg is NULL, this is odd, using value in packet.\n");
+		protoc = ((struct iphdr *)skb->data)->protocol;
+	}
 #endif /* NET_21 */
 
 	if( (protoc != IPPROTO_AH) &&
@@ -242,41 +317,51 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 		goto rcvleave;
 	}
 
-	for(i = 0; i < IPSEC_NUM_IF; i++) {
-		sprintf(name, "ipsec%d", i);
-		if(!strcmp(name, skb->dev->name)) {
-			prv = (struct ipsecpriv *)(skb->dev->priv);
-			stats = (struct enet_statistics *) &(prv->mystats);
-			ipsecdev = skb->dev;
-			KLIPS_PRINT(debug_rcv,
-				    "klips_debug:ipsec_rcv: Info -- pkt "
-				    "already proc'ed an ipsec header, "
-				    "proc'ing next ipsec header.\n");
-			break;
+	if(skb->dev) {
+		for(i = 0; i < IPSEC_NUM_IF; i++) {
+			sprintf(name, "ipsec%d", i);
+			if(!strcmp(name, skb->dev->name)) {
+				prv = (struct ipsecpriv *)(skb->dev->priv);
+				if(prv) {
+					stats = (struct enet_statistics *) &(prv->mystats);
+				}
+				ipsecdev = skb->dev;
+				KLIPS_PRINT(debug_rcv,
+					    "klips_debug:ipsec_rcv: Info -- pkt "
+					    "already proc'ed a group of ipsec headers, "
+					    "processing next group of ipsec headers.\n");
+				break;
+			}
+			if((ipsecdev = ipsec_dev_get(name)) == NULL) {
+				KLIPS_PRINT(debug_rcv,
+					    "klips_error:ipsec_rcv: device %s does "
+					    "not exist\n", name);
+			}
+			prv = ipsecdev ? (struct ipsecpriv *)(ipsecdev->priv) : NULL;
+			prvdev = prv ? (struct device *)(prv->dev) : NULL;
+			
+			KLIPS_PRINT(debug_rcv && prvdev, 
+				    "klips_debug:ipsec_rcv: physical device"
+				    " for device %s is %s\n",
+				    name, prvdev->name);
+			if(prvdev && skb->dev &&
+			   !strcmp(prvdev->name, skb->dev->name)) {
+				stats = prv ? ((struct enet_statistics *) &(prv->mystats)) : NULL;
+				skb->dev = ipsecdev;
+				if(stats) {
+					stats->rx_packets++;
+				}
+				break;
+			}
 		}
-		if((ipsecdev = dev_get(name)) == NULL) {
-			KLIPS_PRINT(debug_rcv,
-				    "klips_error:ipsec_rcv: device %s does "
-				    "not exist\n", name);
-		}
-		prv = ipsecdev ? (struct ipsecpriv *)(ipsecdev->priv) : NULL;
-		prvdev = prv ? (struct device *)(prv->dev) : NULL;
-
-		KLIPS_PRINT(debug_rcv && prvdev, 
-			    "klips_debug:ipsec_rcv: physical device"
-			    " for device %s is %s\n",
-			    name, prvdev->name);
-		KLIPS_PRINT(debug_rcv && !skb->dev, 
+	} else {
+		KLIPS_PRINT(debug_rcv, 
 			    "klips_debug:ipsec_rcv: device supplied"
 			    " with skb is NULL\n");
-
-		if(prvdev && skb->dev &&
-		   !strcmp(prvdev->name, skb->dev->name)) {
-			stats = (struct enet_statistics *) &(prv->mystats);
-			skb->dev = ipsecdev;
-			stats->rx_packets++;
-			break;
-		}
+	}
+			
+	if(!stats) {
+		ipsecdev = NULL;
 	}
 	KLIPS_PRINT((debug_rcv && !stats),
 		    "klips_error:ipsec_rcv: packet received from physical "
@@ -285,7 +370,7 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 		    "klips_error          May not have SA for decoding.  "
 		    "Is IPSEC traffic expected on this I/F?  "
 		    "Check routing.\n",
-		    skb->dev->name);
+		    skb->dev ? (skb->dev->name ? skb->dev->name : "NULL") : "NULL");
 
 #ifdef DEBUG_IPSEC
 	if (debug_rcv) {
@@ -302,8 +387,6 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 
 	/* begin decapsulating loop here */
 	do {
-		unsigned long flags; /* save irq state for spinlock */
-
 #ifdef CONFIG_IPSEC_ESP
 		espp = NULL;
 		esphlen = 0;
@@ -349,11 +432,9 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 		switch(proto) {
 #ifdef CONFIG_IPSEC_ESP
 		case IPPROTO_ESP:
-/*		if(proto == IPPROTO_ESP) { */
 			espp = (struct esp *)(skb->data + iphlen);
 			said.spi = espp->esp_spi;
 			break;
-/*		} else { */
 #endif /* !CONFIG_IPSEC_ESP */
 #ifdef CONFIG_IPSEC_AH
 		case IPPROTO_AH:
@@ -398,12 +479,12 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 		 * The spinlock is to prevent any other process from accessing or deleting
 		 * the structure while we are using and updating it.
 		 */
-		spin_lock_irqsave(&tdb_lock, flags);
+		spin_lock_irqsave(&tdb_lock, tdb_flags);
 
 		tdbp = gettdb(&said);
 		sa_len = satoa(said, 0, sa, SATOA_BUF);
 		if (tdbp == NULL) {
-			spin_unlock_irqrestore(&tdb_lock, flags);
+			spin_unlock_irqrestore(&tdb_lock, tdb_flags);
 			KLIPS_PRINT(debug_rcv,
 				    "klips_debug:ipsec_rcv: no Tunnel Descriptor "
 				    "Block for SA:%s: incoming packet with no SA "
@@ -413,12 +494,59 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 			}
 			goto rcvleave;
 		}
+#ifdef INBOUND_POLICY_CHECK
+		if(tdbnext) {
+			if(tdbnext != tdbp) {
+				spin_unlock_irqrestore(&tdb_lock, tdb_flags);
+				KLIPS_PRINT(debug_rcv,
+					    "klips_debug:ipsec_rcv: "
+					    "unexpected SA:%s: does not agree with tdb->inext policy, "
+					    "dropped\n", sa);
+				if(stats) {
+					stats->rx_dropped++;
+				}
+				goto rcvleave;
+			} else {
+				KLIPS_PRINT(debug_rcv,
+					    "klips_debug:ipsec_rcv: "
+					    "SA:%s grouping from previous SA is OK.\n", sa);
+			}
+		} else {
+			KLIPS_PRINT(debug_rcv,
+				    "klips_debug:ipsec_rcv: "
+				    "SA:%s First SA in group.\n", sa);
+		}
+
+#if 0
+		if(tdbp->tdb_onext) {
+			if(tdbprev != tdbp->tdb_onext) {
+				spin_unlock_irqrestore(&tdb_lock, tdb_flags);
+				KLIPS_PRINT(debug_rcv,
+					    "klips_debug:ipsec_rcv: "
+					    "unexpected SA:%s: does not agree with tdb->onext policy, "
+					    "dropped\n", sa);
+				if(stats) {
+					stats->rx_dropped++;
+				}
+				goto rcvleave;
+			} else {
+				KLIPS_PRINT(debug_rcv,
+					    "klips_debug:ipsec_rcv: "
+					    "SA:%s grouping to previous SA is OK.\n", sa);
+			}
+		} else {
+			KLIPS_PRINT(debug_rcv,
+				    "klips_debug:ipsec_rcv: "
+				    "SA:%s No previous backlink in group.\n", sa);
+		}
+#endif
+#endif /* INBOUND_POLICY_CHECK */
 		
-#ifdef CONFIG_IPSEC_PFKEYv2
+		/* #ifdef CONFIG_IPSEC_PFKEYv2 */
 		/* If it is in larval state, drop the packet, we cannot process yet. */
 		if(tdbp->tdb_state == SADB_SASTATE_LARVAL) {
-			spin_unlock_irqrestore(&tdb_lock, flags);
-			KLIPS_PRINT(debug_tunnel & DB_TN_CROUT,
+			spin_unlock_irqrestore(&tdb_lock, tdb_flags);
+			KLIPS_PRINT(debug_rcv,
 				    "klips_debug:ipsec_rcv: "
 				    "TDB in larval state, cannot be used yet, "
 				    "dropping packet.\n");
@@ -429,8 +557,8 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 		}
 		
 		if(tdbp->tdb_state == SADB_SASTATE_DEAD) {
-			spin_unlock_irqrestore(&tdb_lock, flags);
-			KLIPS_PRINT(debug_tunnel & DB_TN_CROUT,
+			spin_unlock_irqrestore(&tdb_lock, tdb_flags);
+			KLIPS_PRINT(debug_rcv,
 				    "klips_debug:ipsec_rcv: "
 				    "TDB in dead state, cannot be used any more, "
 				    "dropping packet.\n");
@@ -443,7 +571,7 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 		if(tdbp->tdb_lifetime_bytes_s &&
 		   (tdbp->tdb_lifetime_bytes_c > tdbp->tdb_lifetime_bytes_s)) {
 			tdbp->tdb_state = SADB_SASTATE_DYING;
-			KLIPS_PRINT(debug_tunnel & DB_TN_CROUT,
+			KLIPS_PRINT(debug_rcv,
 				    "klips_debug:ipsec_rcv: "
 				    "soft bytes lifetime of SA:%s has been reached, "
 				    "SA expiring, soft expire message sent up, "
@@ -454,7 +582,7 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 		   (tdbp->tdb_lifetime_bytes_c > tdbp->tdb_lifetime_bytes_h)) {
 			/* pfkey_expire(tdbp) */;
 			deltdbchain(tdbp);
-			spin_unlock_irqrestore(&tdb_lock, flags);
+			spin_unlock_irqrestore(&tdb_lock, tdb_flags);
 			KLIPS_PRINT(debug_rcv,
 				    "klips_debug:ipsec_rcv: "
 				    "hard bytes lifetime of SA:%s has been reached, "
@@ -469,7 +597,7 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 		   ((jiffies / HZ) - tdbp->tdb_lifetime_addtime_c >
 		    tdbp->tdb_lifetime_addtime_s)) {
 			tdbp->tdb_state = SADB_SASTATE_DYING;
-			KLIPS_PRINT(debug_tunnel & DB_TN_CROUT,
+			KLIPS_PRINT(debug_rcv,
 				    "klips_debug:ipsec_rcv: "
 				    "soft addtime lifetime of SA:%s has been reached, "
 				    "SA expiring, soft expire message sent up, "
@@ -481,7 +609,7 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 		    tdbp->tdb_lifetime_addtime_h)) {
 			/* pfkey_expire(tdbp) */;
 			deltdbchain(tdbp);
-			spin_unlock_irqrestore(&tdb_lock, flags);
+			spin_unlock_irqrestore(&tdb_lock, tdb_flags);
 			KLIPS_PRINT(debug_rcv,
 				    "klips_debug:ipsec_rcv: "
 				    "hard addtime lifetime of SA:%s has been reached, "
@@ -492,37 +620,39 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 			goto rcvleave;
 		}
 		
-		if(tdbp->tdb_lifetime_usetime_s &&
-		   ((jiffies / HZ) - tdbp->tdb_lifetime_usetime_c >
-		    tdbp->tdb_lifetime_usetime_s)) {
-			tdbp->tdb_state = SADB_SASTATE_DYING;
-			KLIPS_PRINT(debug_tunnel & DB_TN_CROUT,
-				    "klips_debug:ipsec_rcv: "
-				    "soft usetime lifetime of SA:%s has been reached, "
-				    "SA expiring, soft expire message sent up, "
-				    "incoming packet still processed.\n", sa);
-			/* pfkey_expire(tdbp) */;
-		}
-		if(tdbp->tdb_lifetime_usetime_h &&
-		   ((jiffies / HZ) - tdbp->tdb_lifetime_usetime_c >
-		    tdbp->tdb_lifetime_usetime_h)) {
-			/* pfkey_expire(tdbp) */;
-			deltdbchain(tdbp);
-			spin_unlock_irqrestore(&tdb_lock, flags);
-			KLIPS_PRINT(debug_rcv,
-				    "klips_debug:ipsec_rcv: "
-				    "hard usetime lifetime of SA:%s has been reached, "
-				    "SA expired, incoming packet dropped.\n", sa);
-			if(stats) {
-				stats->rx_dropped++;
+		if(tdbp->tdb_lifetime_usetime_c) {
+			if(tdbp->tdb_lifetime_usetime_s &&
+			   ((jiffies / HZ) - tdbp->tdb_lifetime_usetime_c >
+			    tdbp->tdb_lifetime_usetime_s)) {
+				tdbp->tdb_state = SADB_SASTATE_DYING;
+				KLIPS_PRINT(debug_rcv,
+					    "klips_debug:ipsec_rcv: "
+					    "soft usetime lifetime of SA:%s has been reached, "
+					    "SA expiring, soft expire message sent up, "
+					    "incoming packet still processed.\n", sa);
+				/* pfkey_expire(tdbp) */;
+			}
+			if(tdbp->tdb_lifetime_usetime_h &&
+			   ((jiffies / HZ) - tdbp->tdb_lifetime_usetime_c >
+			    tdbp->tdb_lifetime_usetime_h)) {
+				/* pfkey_expire(tdbp) */;
+				deltdbchain(tdbp);
+				spin_unlock_irqrestore(&tdb_lock, tdb_flags);
+				KLIPS_PRINT(debug_rcv,
+					    "klips_debug:ipsec_rcv: "
+					    "hard usetime lifetime of SA:%s has been reached, "
+					    "SA expired, incoming packet dropped.\n", sa);
+				if(stats) {
+					stats->rx_dropped++;
+				}
+				goto rcvleave;
 			}
-			goto rcvleave;
 		}
 		
 		if(tdbp->tdb_lifetime_packets_s &&
 		   (tdbp->tdb_lifetime_packets_c > tdbp->tdb_lifetime_packets_s)) {
 			tdbp->tdb_state = SADB_SASTATE_DYING;
-			KLIPS_PRINT(debug_tunnel & DB_TN_CROUT,
+			KLIPS_PRINT(debug_rcv,
 				    "klips_debug:ipsec_rcv: "
 				    "soft packets lifetime of SA:%s has been reached, "
 				    "SA expiring, soft expire message sent up, "
@@ -533,7 +663,7 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 		   (tdbp->tdb_lifetime_packets_c > tdbp->tdb_lifetime_packets_h)) {
 			/* pfkey_expire(tdbp) */;
 			deltdbchain(tdbp);
-			spin_unlock_irqrestore(&tdb_lock, flags);
+			spin_unlock_irqrestore(&tdb_lock, tdb_flags);
 			KLIPS_PRINT(debug_rcv,
 				    "klips_debug:ipsec_rcv: "
 				    "hard packets lifetime of SA:%s has been reached, "
@@ -543,7 +673,7 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 			}
 			goto rcvleave;
 		}
-#endif /* CONFIG_IPSEC_PFKEYv2 */
+		/* #endif */ /* CONFIG_IPSEC_PFKEYv2 */
 		
 		/* authenticate, if required */
 		idat = dat + iphlen;
@@ -562,7 +692,7 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 			authlen = 0;
 			break;
 		default:
-			spin_unlock_irqrestore(&tdb_lock, flags);
+			spin_unlock_irqrestore(&tdb_lock, tdb_flags);
 			if(stats) {
 				stats->rx_errors++;
 			}
@@ -574,11 +704,11 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 #ifdef CONFIG_IPSEC_ESP
 		KLIPS_PRINT(proto == IPPROTO_ESP && debug_rcv, 
 			    "klips_debug:ipsec_rcv: packet from %s received with"
-			    " seq=%ld (iv)=0x%08lx%08lx iplen=%d esplen=%d sa=%s\n",
+			    " seq=%d (iv)=0x%08x%08x iplen=%d esplen=%d sa=%s\n",
 			    ipaddr_txt,
-			    ntohl(espp->esp_rpl),
-			    ntohl(*((__u32 *)(espp->esp_iv)    )),
-			    ntohl(*((__u32 *)(espp->esp_iv) + 1)),
+			    (__u32)ntohl(espp->esp_rpl),
+			    (__u32)ntohl(*((__u32 *)(espp->esp_iv)    )),
+			    (__u32)ntohl(*((__u32 *)(espp->esp_iv) + 1)),
 			    len, ilen, sa);
 #endif /* !CONFIG_IPSEC_ESP */
 		
@@ -600,8 +730,8 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 		/* If the sequence number == 0, expire SA, it had rolled */
 		if(tdbp->tdb_replaywin && !replay /* !tdbp->tdb_replaywin_lastseq */) {
 			deltdbchain(tdbp);
-			spin_unlock_irqrestore(&tdb_lock, flags);
-			KLIPS_PRINT(debug_tunnel & DB_TN_CROUT,
+			spin_unlock_irqrestore(&tdb_lock, tdb_flags);
+			KLIPS_PRINT(debug_rcv,
 				    "klips_debug:ipsec_tunnel_start_xmit: "
 				    "replay window counter rolled, expiring SA.\n");
 			if(stats) {
@@ -611,7 +741,7 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 		}
 
 		if (!ipsec_checkreplaywindow(tdbp, replay)) {
-			spin_unlock_irqrestore(&tdb_lock, flags);
+			spin_unlock_irqrestore(&tdb_lock, tdb_flags);
 			KLIPS_PRINT(debug_rcv & DB_RX_REPLAY,
 				    "klips_debug:ipsec_rcv: duplicate frame from %s,"
 				    " packet dropped\n",
@@ -641,7 +771,7 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 #ifdef CONFIG_IPSEC_AH
 				} else {
 					ipo = *ipp;
-					ipo.tos = 0;
+					ipo.tos = 0;	/* mutable RFC 2402 3.3.3.1.1.1 */
 					ipo.frag_off = 0;
 					ipo.ttl = 0;
 					ipo.check = 0;
@@ -698,7 +828,7 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 			}
 		
 			if(!authenticator) {
-				spin_unlock_irqrestore(&tdb_lock, flags);
+				spin_unlock_irqrestore(&tdb_lock, tdb_flags);
 				if(stats) {
 					stats->rx_dropped++;
 				}
@@ -706,9 +836,7 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 				goto rcvleave;
 			}
 
-			if (memcmp(hash, /* proto == IPPROTO_ESP ?
-					    &(dat[len - authlen]) : ahp->ah_data */
-				   authenticator, authlen)) {
+			if (memcmp(hash, authenticator, authlen)) {
 				KLIPS_PRINT(debug_rcv & DB_RX_INAU,
 					    "klips_debug:ipsec_rcv: auth failed on incoming "
 					    "packet from %s: hash=%08x%08x%08x "
@@ -716,13 +844,10 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 					    *(__u32*)&hash[0],
 					    *(__u32*)&hash[4],
 					    *(__u32*)&hash[8],
-/*					    proto == IPPROTO_ESP ? */
-					    *(__u32*)authenticator, /* &dat[len - 12] : *(__u32*)(ahp->ah_data), */
-/*					    proto == IPPROTO_ESP ? */
-					    *((__u32*)authenticator) + 1, /* &dat[len - 8] : *&((__u32*)(ahp->ah_data))[4], */
-/*					    proto == IPPROTO_ESP ? */
-					    *((__u32*)authenticator) + 2); /* &dat[len - 4] : *&((__u32*)(ahp->ah_data))[8]); */
-				spin_unlock_irqrestore(&tdb_lock, flags);
+					    *(__u32*)authenticator,
+					    *((__u32*)authenticator + 1),
+					    *((__u32*)authenticator + 2));
+				spin_unlock_irqrestore(&tdb_lock, tdb_flags);
 				if(stats) {
 					stats->rx_dropped++;
 				}
@@ -737,10 +862,22 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 			memset(hash, 0, sizeof(hash));
 		}
 
+		if (!ipsec_updatereplaywindow(tdbp, replay)) {
+			spin_unlock_irqrestore(&tdb_lock, tdb_flags);
+			KLIPS_PRINT(debug_rcv & DB_RX_REPLAY,
+				    "klips_debug:ipsec_rcv: duplicate frame from %s,"
+				    " packet dropped\n",
+				    ipaddr_txt);
+			if(stats) {
+				stats->rx_dropped++;
+			}
+			tdbp->tdb_replaywin_errs += 1;
+			goto rcvleave;
+		}
+		
 		switch(proto) {
 #ifdef CONFIG_IPSEC_ESP
 		case IPPROTO_ESP:
-/*		if(proto == IPPROTO_ESP) { */
 			switch(tdbp->tdb_encalg) {
 			case ESP_DES:
 			case ESP_3DES:
@@ -752,7 +889,7 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 				esphlen = offsetof(struct esp, esp_iv);
 				break;
 			default:
-				spin_unlock_irqrestore(&tdb_lock, flags);
+				spin_unlock_irqrestore(&tdb_lock, tdb_flags);
 				if(stats) {
 					stats->rx_errors++;
 				}
@@ -765,7 +902,7 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 			switch(tdbp->tdb_encalg) {
 			case ESP_DES:
 				if ((ilen) % 8) {
-					spin_unlock_irqrestore(&tdb_lock, flags);
+					spin_unlock_irqrestore(&tdb_lock, tdb_flags);
 					printk("klips_error:ipsec_rcv: got packet with esplen = %d from "
 					       "%s -- should be on 8 octet boundary, packet dropped\n",
 					       ilen, ipaddr_txt);
@@ -781,7 +918,7 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 				break;
 			case ESP_3DES:
 				if ((ilen) % 8) {
-					spin_unlock_irqrestore(&tdb_lock, flags);
+					spin_unlock_irqrestore(&tdb_lock, tdb_flags);
 					printk("klips_error:ipsec_rcv: got packet with esplen = %d from "
 					       "%s -- should be on 8 octet boundary, packet dropped\n",
 					       ilen, ipaddr_txt);
@@ -842,7 +979,7 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 		tdbp->tdb_lifetime_usetime_l = jiffies / HZ;
 		tdbp->tdb_lifetime_packets_c += 1;
 		
-		spin_unlock_irqrestore(&tdb_lock, flags);
+		spin_unlock_irqrestore(&tdb_lock, tdb_flags);
 
 		/*
 		 *	Discard the original IP header
@@ -861,6 +998,13 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 			ipp->tot_len = htons(ntohs(ipp->tot_len) - (esphlen + pad));
 			memmove((void *)(skb->data + esphlen),
 				(void *)(skb->data), iphlen);
+			if(skb->len < esphlen) {
+				printk(KERN_WARNING "klips_error:ipsec_rcv: "
+				       "tried to skb_pull esphlen=%d, %d available.  "
+				       "This should never happen, please report.\n",
+				       esphlen, (int)(skb->len));
+				goto rcvleave;
+			}
 			skb_pull(skb, esphlen);
 		break;
 #endif /* !CONFIG_IPSEC_ESP */
@@ -869,18 +1013,18 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 			ipp->tot_len = htons(ntohs(ipp->tot_len) - ahhlen);
 			memmove((void *)(skb->data + ahhlen),
 				(void *)(skb->data), iphlen);
+			if(skb->len < ahhlen) {
+				printk(KERN_WARNING "klips_error:ipsec_rcv: "
+				       "tried to skb_pull ahhlen=%d, %d available.  "
+				       "This should never happen, please report.\n",
+				       ahhlen, (int)(skb->len));
+				goto rcvleave;
+			}
 			skb_pull(skb, ahhlen);
 			break;
 #endif /* CONFIG_IPSEC_AH */
 		}
 
-/*		ipp->tot_len = htons(ntohs(ipp->tot_len) -
-				     (proto == IPPROTO_ESP ?
-				     (esphlen + pad) : ahhlen)); */
-/*		memmove((void *)(skb->data + ((proto == IPPROTO_ESP) ? esphlen : ahhlen)),
-		(void *)(skb->data), iphlen); */
-		
-/*		skb_pull(skb, proto == IPPROTO_ESP ? esphlen : ahhlen); */
 #ifdef CONFIG_IPSEC_ESP
 		if(proto == IPPROTO_ESP) {
 			KLIPS_PRINT(debug_rcv & DB_RX_PKTRX,
@@ -919,21 +1063,213 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 		
 		skb->protocol = htons(ETH_P_IP);
 		skb->ip_summed = 0;
+
+#ifdef INBOUND_POLICY_CHECK
+		tdbprev = tdbp;
+		tdbnext = tdbp->tdb_inext;
+		if(tdbnext) {
+			if(tdbnext->tdb_onext != tdbp) {
+				KLIPS_PRINT(debug_rcv,
+					    "klips_debug:ipsec_rcv: "
+					    "SA:%s, backpolicy does not agree with fwdpolicy.\n", sa);
+				if(stats) {
+					stats->rx_dropped++;
+				}
+				goto rcvleave;
+			} else {
+				KLIPS_PRINT(debug_rcv,
+					    "klips_debug:ipsec_rcv: "
+					    "SA:%s, backpolicy agrees with fwdpolicy.\n", sa);
+			}
+			if(!((ipp->protocol == IPPROTO_ESP ) ||
+			     (ipp->protocol == IPPROTO_AH  ) ||
+			     (ipp->protocol == IPPROTO_IPIP)
+#ifdef CONFIG_IPSEC_IPCOMP
+			     || (ipp->protocol == IPPROTO_COMP)
+#endif /* CONFIG_IPSEC_IPCOMP */
+				)) {
+				KLIPS_PRINT(debug_rcv,
+					    "klips_debug:ipsec_rcv: "
+					    "packet with incomplete policy dropped, SA:%s.\n", sa);
+				if(stats) {
+					stats->rx_dropped++;
+				}
+				goto rcvleave;
+			} else {
+				KLIPS_PRINT(debug_rcv,
+					    "klips_debug:ipsec_rcv: "
+					    "SA:%s, Another IPSEC header to process.\n", sa);
+			}
+		} else {
+			KLIPS_PRINT(debug_rcv,
+				    "klips_debug:ipsec_rcv: "
+				    "No tdb_inext from this SA:%s.\n", sa);
+		}
+#endif /* INBOUND_POLICY_CHECK */
 	} while((ipp->protocol == IPPROTO_ESP) ||
-		(ipp->protocol == IPPROTO_AH));
+		(ipp->protocol == IPPROTO_AH )
+#ifdef CONFIG_IPSEC_IPCOMP
+		|| (ipp->protocol == IPPROTO_COMP)
+#endif /* CONFIG_IPSEC_IPCOMP */
+		);
 	/* end decapsulation loop here */
 	
+#ifdef INBOUND_POLICY_CHECK_eroute
+	/*
+	 * First things first -- look us up in the erouting tables.
+	 */
+	matcher.sen_len = sizeof (struct sockaddr_encap);
+	matcher.sen_family = AF_ENCAP;
+	matcher.sen_type = SENT_IP4;
 	if(ipp->protocol == IPPROTO_IPIP) {
+		struct iphdr *ipp2;
+
+		ipp2 = (struct iphdr*) (((char*)ipp) + (ipp->ihl << 2));
+		matcher.sen_ip_src.s_addr = ipp2->saddr;
+		matcher.sen_ip_dst.s_addr = ipp2->daddr;
+	} else {
+		matcher.sen_ip_src.s_addr = ipp->saddr;
+		matcher.sen_ip_dst.s_addr = ipp->daddr;
+	}
+	
+	/*
+	 * The spinlock is to prevent any other process from accessing or deleting
+	 * the eroute while we are using and updating it.
+	 */
+	spin_lock_irqsave(&eroute_lock, eroute_flags);
+	
+	er = ipsec_findroute(&matcher);
+	if(er) {
+		policy_said = er->er_said;
+		policy_eaddr = er->er_eaddr;
+		policy_emask = er->er_emask;
+	}
+	
+	spin_unlock_irqrestore(&eroute_lock, eroute_flags);
+
+	if(er) {
+		if(tdbnext) {
+			tdbprev = tdbnext;
+			tdbnext = tdbp->tdb_inext;
+		}
+		
+		/*
+		 * The spinlock is to prevent any other process from accessing or deleting
+		 * the tdb while we are using and updating it.
+		 */
+		spin_lock_irqsave(&tdb_lock, tdb_flags);
+
+		policy_tdb = gettdb(&policy_said);
+		if (policy_tdb == NULL) {
+			spin_unlock_irqrestore(&tdb_lock, tdb_flags);
+			KLIPS_PRINT(debug_rcv,
+				    "klips_debug:ipsec_rcv: "
+				    "no Tunnel Descriptor Block for SA%s: "
+				    "incoming packet with no policy SA, dropped.\n", sa);
+			goto rcvleave;
+		}
+		
+		sa_len = satoa(policy_said, 0, sa, SATOA_BUF);
+
+		KLIPS_PRINT(debug_rcv,
+			    "klips_debug:ipsec_rcv: "
+			    "found policy Tunnel Descriptor Block -- SA:%s\n", sa);
+		while(1) {
+			if(policy_tdb->tdb_inext) {
+				policy_tdb = policy_tdb->tdb_inext;
+			} else {
+				break;
+			}
+		}
+		if(policy_tdb != tdbp) {
+			spin_unlock_irqrestore(&tdb_lock, tdb_flags);
+			KLIPS_PRINT(debug_rcv,
+				    "klips_debug:ipsec_rcv: "
+				    " Tunnel Descriptor Block for SA%s: "
+				    "incoming packet with different policy SA, dropped.\n", sa);
+			goto rcvleave;
+		}
+
+		tdbp->tdb_lifetime_bytes_c += len;
+		if(!tdbp->tdb_lifetime_usetime_c) {
+			tdbp->tdb_lifetime_usetime_c = jiffies / HZ;
+		}
+		tdbp->tdb_lifetime_usetime_l = jiffies / HZ;
+		tdbp->tdb_lifetime_packets_c += 1;
+		
+		spin_unlock_irqrestore(&tdb_lock, tdb_flags);
+	}
+#endif /* INBOUND_POLICY_CHECK */
+#ifdef INBOUND_POLICY_CHECK
+	if(tdbnext) {
+		sa_len = satoa(tdbnext->tdb_said, 0, sa, SATOA_BUF);
+
+		if(ipp->saddr != ((struct sockaddr_in*)(tdbnext->tdb_addr_s))->sin_addr.s_addr) {
+			ipaddr.s_addr = ipp->saddr;
+			addrtoa(ipaddr, 0, ipaddr_txt, sizeof(ipaddr_txt));
+			KLIPS_PRINT(debug_rcv,
+				    "klips_debug:ipsec_rcv: "
+				    "SA:%s, src=%s of pkt does not agree with IPIP policy.\n",
+				    sa, ipaddr_txt);
+			if(stats) {
+				stats->rx_dropped++;
+			}
+			goto rcvleave;
+		} else {
+			ipaddr.s_addr = ipp->saddr;
+			addrtoa(ipaddr, 0, ipaddr_txt, sizeof(ipaddr_txt));
+			KLIPS_PRINT(debug_rcv,
+				    "klips_debug:ipsec_rcv: "
+				    "SA:%s, src=%s of pkt agrees with IPIP policy.\n",
+				    sa, ipaddr_txt);
+		}
+		if(ipp->daddr != ((struct sockaddr_in*)(tdbnext->tdb_addr_d))->sin_addr.s_addr) {
+			ipaddr.s_addr = ipp->daddr;
+			addrtoa(ipaddr, 0, ipaddr_txt, sizeof(ipaddr_txt));
+			KLIPS_PRINT(debug_rcv,
+				    "klips_debug:ipsec_rcv: "
+				    "SA:%s, dst=%s of pkt does not agree with IPIP policy.\n",
+				    sa, ipaddr_txt);
+			if(stats) {
+				stats->rx_dropped++;
+			}
+			goto rcvleave;
+		} else {
+			ipaddr.s_addr = ipp->daddr;
+			addrtoa(ipaddr, 0, ipaddr_txt, sizeof(ipaddr_txt));
+			KLIPS_PRINT(debug_rcv,
+				    "klips_debug:ipsec_rcv: "
+				    "SA:%s, dst=%s of pkt agree with IPIP policy.\n",
+				    sa, ipaddr_txt);
+		}
+		
+		tdbnext->tdb_lifetime_bytes_c += len;
+		if(!tdbnext->tdb_lifetime_usetime_c) {
+			tdbnext->tdb_lifetime_usetime_c = jiffies / HZ;
+		}
+		tdbnext->tdb_lifetime_usetime_l = jiffies / HZ;
+		tdbnext->tdb_lifetime_packets_c += 1;
+	}
+		
+#endif /* INBOUND_POLICY_CHECK */
+
+	if(ipp->protocol == IPPROTO_IPIP) {
+		if(skb->len < iphlen) {
+			printk(KERN_WARNING "klips_debug:ipsec_rcv: "
+			       "tried to skb_pull iphlen=%d, %d available.  "
+			       "This should never happen, please report.\n",
+			       iphlen, (int)(skb->len));
+			goto rcvleave;
+		}
 		skb_pull(skb, iphlen);
 
 #ifdef NET_21
-		skb->nh.raw=skb->data;
+		ipp = (struct iphdr *)skb->nh.raw = skb->data;
 		skb->h.raw = skb->nh.raw + (skb->nh.iph->ihl << 2);
 		
 		memset(&(IPCB(skb)->opt), 0, sizeof(struct ip_options));
 #else /* NET_21 */
-		skb->h.iph=(struct iphdr *)skb->data;
-		skb->ip_hdr=(struct iphdr *)skb->data;
+		ipp = skb->ip_hdr = skb->h.iph = (struct iphdr *)skb->data;
 
 		memset(skb->proto_priv, 0, sizeof(struct options));
 #endif /* NET_21 */
@@ -942,15 +1278,13 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 		skb->ip_summed = 0;
 		KLIPS_PRINT(debug_rcv & DB_RX_PKTRX,
 			    "klips_debug:ipsec_rcv: IPIP tunnel stripped.\n");
-#ifdef NET_21
-		KLIPS_IP_PRINT(debug_rcv & DB_RX_PKTRX, skb->nh.iph);
-#else /* NET_21 */
-		KLIPS_IP_PRINT(debug_rcv & DB_RX_PKTRX, skb->ip_hdr);
-#endif /* NET_21 */
+		KLIPS_IP_PRINT(debug_rcv & DB_RX_PKTRX, ipp);
 	}
 
 #ifdef NET_21
-	stats->rx_bytes += skb->len;
+	if(stats) {
+		stats->rx_bytes += skb->len;
+	}
 	if(skb->dst) {
 		dst_release(skb->dst);
 		skb->dst = NULL;
@@ -964,14 +1298,18 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 		skb->mac.raw = skb->data - hard_header_len;
 	}
 #endif /* NET_21 */
+#ifdef SKB_RESET_NFCT
+        nf_conntrack_put(skb->nfct);
+        skb->nfct = NULL;
+#endif
 	KLIPS_PRINT(debug_rcv & DB_RX_PKTRX,
 		    "klips_debug:ipsec_rcv: netif_rx() called.\n");
 	netif_rx(skb);
 
 	MOD_DEC_USE_COUNT;
 	return(0);
-
-     rcvleave:
+	
+ rcvleave:
  	if(skb) {
 #ifdef NET_21
                 kfree_skb(skb);
@@ -1008,6 +1346,53 @@ struct inet_protocol esp_protocol =
 
 /*
  * $Log: ipsec_rcv.c,v $
+ * Revision 1.48  2000/05/10 19:14:40  rgb
+ * Only check usetime against soft and hard limits if the tdb has been
+ * used.
+ * Cast output of ntohl so that the broken prototype doesn't make our
+ * compile noisy.
+ *
+ * Revision 1.47  2000/05/09 17:45:43  rgb
+ * Fix replay bitmap corruption bug upon receipt of bogus packet
+ * with correct SPI.  This was a DoS.
+ *
+ * Revision 1.46  2000/03/27 02:31:58  rgb
+ * Fixed authentication failure printout bug.
+ *
+ * Revision 1.45  2000/03/22 16:15:37  rgb
+ * Fixed renaming of dev_get (MB).
+ *
+ * Revision 1.44  2000/03/16 08:17:24  rgb
+ * Hardcode PF_KEYv2 support.
+ * Fixed minor bug checking AH header length.
+ *
+ * Revision 1.43  2000/03/14 12:26:59  rgb
+ * Added skb->nfct support for clearing netfilter conntrack bits (MB).
+ *
+ * Revision 1.42  2000/01/26 10:04:04  rgb
+ * Fixed inbound policy checking on transport mode bug.
+ * Fixed noisy 2.0 printk arguments.
+ *
+ * Revision 1.41  2000/01/24 20:58:02  rgb
+ * Improve debugging/reporting support for (disabled) inbound
+ * policy checking.
+ *
+ * Revision 1.40  2000/01/22 23:20:10  rgb
+ * Fixed up inboud policy checking code.
+ * Cleaned out unused crud.
+ *
+ * Revision 1.39  2000/01/21 06:15:29  rgb
+ * Added sanity checks on skb_push(), skb_pull() to prevent panics.
+ * Fixed cut-and-paste debug_tunnel to debug_rcv.
+ * Added inbound policy checking code, disabled.
+ * Simplified output code by updating ipp to post-IPIP decapsulation.
+ *
+ * Revision 1.38  1999/12/22 05:08:36  rgb
+ * Checked for null skb, skb->dev, skb->data, skb->dev->name, dev->name,
+ * protocol and take appropriate action for sanity.
+ * Set ipsecdev to NULL if device could not be determined.
+ * Fixed NULL stats access bug if device could not be determined.
+ *
  * Revision 1.37  1999/12/14 20:07:59  rgb
  * Added a default switch case to catch bogus encalg values.
  *
