@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: pfkey_v2.c,v 1.23 1999/12/09 23:22:16 rgb Exp $
+ * RCSID $Id: pfkey_v2.c,v 1.31 2000/05/16 03:04:00 rgb Exp $
  */
 
 /*
@@ -71,13 +71,17 @@ int debug_pfkey = 0;
 
 #define min(a,b)	(((a)<(b))?(a):(b))
 
-#define SENDERR(_x) do { len = -(_x); goto errlab; } while (0)
+#define SENDERR(_x) do { error = -(_x); goto errlab; } while (0)
 
 #ifdef NET_21_
 static atomic_t pfkey_nr_socks = ATOMIC_INI(0);
 #endif /* NET_21 */
 
-struct proto_ops pfkey_dgram_ops;
+#ifndef SOCKOPS_WRAPPED
+#define SOCKOPS_WRAPPED(name) name
+#endif
+
+struct proto_ops SOCKOPS_WRAPPED(pfkey_ops);
 struct sock *pfkey_sock_list = NULL;
 
 struct socket_list *pfkey_open_sockets = NULL;
@@ -151,6 +155,7 @@ pfkey_state_change(struct sock *sk)
 }
 #endif /* !NET_21 */
 
+#ifndef NET_21
 static void
 pfkey_data_ready(struct sock *sk, int len)
 {
@@ -162,7 +167,6 @@ pfkey_data_ready(struct sock *sk, int len)
 	}
 }
 
-#ifndef NET_21
 static void
 pfkey_write_space(struct sock *sk)
 {
@@ -173,6 +177,7 @@ pfkey_write_space(struct sock *sk)
 		sock_wake_async(sk->socket, 2);
 	}
 }
+#endif /* !NET_21 */
 
 static void
 pfkey_insert_socket(struct sock *sk)
@@ -211,7 +216,6 @@ pfkey_remove_socket(struct sock *sk)
 		    "klips_debug:pfkey_remove_socket: not found.\n");
 	return;
 }
-#endif /* !NET_21 */
 
 /* static */ void
 pfkey_destroy_socket(struct sock *sk)
@@ -220,7 +224,7 @@ pfkey_destroy_socket(struct sock *sk)
 
 	KLIPS_PRINT(debug_pfkey,
 		    "klips_debug:pfkey_destroy_socket: .\n");
-#ifdef NET_21
+#ifdef NET_21_
 	/* sklist_destroy_socket(&pfkey_sock_list, sk); */
 	sklist_remove_socket(&pfkey_sock_list, sk);
 	KLIPS_PRINT(debug_pfkey,
@@ -287,7 +291,6 @@ if(0 && debug_pfkey) {		KLIPS_PRINT(debug_pfkey,
 		printk(" is_clone:%d", skb->is_clone);
 		printk(" cloned:%d", skb->cloned);
 		printk(" pkt_type:%d", skb->pkt_type);
-		printk(" pkt_bridged:%d", skb->pkt_bridged);
 		printk(" ip_summed:%d", skb->ip_summed);
 		printk(" priority:%d", skb->priority);
 		printk(" protocol:%d", skb->protocol);
@@ -317,68 +320,6 @@ if(0 && debug_pfkey) {		KLIPS_PRINT(debug_pfkey,
 			    "klips_debug:pfkey_destroy_socket: skb=%p leaking.\n", skb);
 #endif
 
-#ifdef THIS_IS_A_QUICKREF_OF_THE_STRUCT_SK_BUFF
-	struct sk_buff	* next;			/* Next buffer in list 				*/
-	struct sk_buff	* prev;			/* Previous buffer in list 			*/
-	struct sk_buff_head * list;		/* List we are on				*/
-	struct sock	*sk;			/* Socket we are owned by 			*/
-	struct timeval	stamp;			/* Time we arrived				*/
-	struct device	*dev;			/* Device we arrived on/are leaving by		*/
-
-	/* Transport layer header */
-	union
-	{
-		struct tcphdr	*th;
-		struct udphdr	*uh;
-		struct icmphdr	*icmph;
-		struct igmphdr	*igmph;
-		struct iphdr	*ipiph;
-		struct spxhdr	*spxh;
-		unsigned char	*raw;
-	} h;
-
-	/* Network layer header */
-	union
-	{
-		struct iphdr	*iph;
-		struct ipv6hdr	*ipv6h;
-		struct arphdr	*arph;
-		struct ipxhdr	*ipxh;
-		unsigned char	*raw;
-	} nh;
-  
-	/* Link layer header */
-	union 
-	{	
-	  	struct ethhdr	*ethernet;
-	  	unsigned char 	*raw;
-	} mac;
-
-	struct  dst_entry *dst;
-
-	char		cb[48];	 
-
-	unsigned int 	len;			/* Length of actual data			*/
-	unsigned int	csum;			/* Checksum 					*/
-	volatile char 	used;			/* Data moved to user and not MSG_PEEK		*/
-	unsigned char	is_clone,		/* We are a clone				*/
-			cloned, 		/* head may be cloned (check refcnt to be sure). */
-  			pkt_type,		/* Packet class					*/
-  			pkt_bridged,		/* Tracker for bridging 			*/
-  			ip_summed;		/* Driver fed us an IP checksum			*/
-	__u32		priority;		/* Packet queueing priority			*/
-	atomic_t	users;			/* User count - see datagram.c,tcp.c 		*/
-	unsigned short	protocol;		/* Packet protocol from driver. 		*/
-	unsigned short	security;		/* Security level of packet			*/
-	unsigned int	truesize;		/* Buffer size 					*/
-
-	unsigned char	*head;			/* Head of buffer 				*/
-	unsigned char	*data;			/* Data head pointer				*/
-	unsigned char	*tail;			/* Tail pointer					*/
-	unsigned char 	*end;			/* End pointer					*/
-	void 		(*destructor)(struct sk_buff *);	/* Destruct function		*/
-#endif
-
 #else /* NET_21 */
 		KLIPS_PRINT(debug_pfkey,
 			    "klips_debug:pfkey_destroy_socket: skb=%p dequeued and freed.\n", skb);
@@ -427,13 +368,15 @@ pfkey_upmsg(struct socket *sock, struct sadb_msg *pfkey_msg)
 	}
 
 	KLIPS_PRINT(debug_pfkey,
-		    "klips_debug:pfkey_upmsg: allocating %d bytes.\n",
+		    "klips_debug:pfkey_upmsg: allocating %d bytes...\n",
 		    pfkey_msg->sadb_msg_len * IPSEC_PFKEYv2_ALIGN);
 	if(!(skb = alloc_skb(pfkey_msg->sadb_msg_len * IPSEC_PFKEYv2_ALIGN, GFP_ATOMIC) )) {
 		KLIPS_PRINT(debug_pfkey,
 			    "klips_debug:pfkey_upmsg: no buffers left to send up a message.\n");
 		return -ENOBUFS;
 	}
+	KLIPS_PRINT(debug_pfkey,
+		    "klips_debug:pfkey_upmsg: ...allocated at %p.\n", skb);
 	
 #if 0
 	memset(skb, 0, sizeof(*skb));  /* this is a bad idea because it resets the data pointers */
@@ -469,7 +412,9 @@ pfkey_upmsg(struct socket *sock, struct sadb_msg *pfkey_msg)
 		kfree_skb(skb, FREE_WRITE);
 #endif /* NET_21 */
 		KLIPS_PRINT(debug_pfkey,
-			    "klips_debug:pfkey_upmsg: error calling sock_queue_rcv_skb.\n");
+			    "klips_debug:pfkey_upmsg: "
+			    "error=%d calling sock_queue_rcv_skb with skb=%p.\n",
+			    error, skb);
 		return error;
 	}
 	return 0;
@@ -536,15 +481,15 @@ pfkey_create(struct socket *sock, int protocol)
 
 	sk->destruct = NULL; /* pfkey_destruct; */
 	sk->reuse = 1;
-	sock->ops = &pfkey_dgram_ops;
+	sock->ops = &SOCKOPS_WRAPPED(pfkey_ops);
 
 	sk->zapped=0;
 	sk->family = PF_KEY;
 /*	sk->num = protocol; */
 	sk->protocol = protocol;
 	key_pid(sk) = current->pid;
-	sk->data_ready=pfkey_data_ready;
-	sklist_insert_socket(&pfkey_sock_list, sk);
+	/*	sklist_insert_socket(&pfkey_sock_list, sk); */
+	pfkey_insert_socket(sk);
 	KLIPS_PRINT(debug_pfkey,
 		    "klips_debug:pfkey_create: sock->fasync_list=%p sk->sleep=%p.\n",
 		    sock->fasync_list,
@@ -616,7 +561,11 @@ pfkey_dup(struct socket *newsock, struct socket *oldsock)
 #endif /* !NET_21 */
 
 /* static */ int
+#ifdef NETDEV_23
+pfkey_release(struct socket *sock)
+#else
 pfkey_release(struct socket *sock, struct socket *peersock)
+#endif
 {
 	struct sock *sk;
 
@@ -662,12 +611,13 @@ pfkey_release(struct socket *sock, struct socket *peersock)
 	pfkey_destroy_socket(sk);
 #endif /* NET_21 */
 
+	pfkey_list_remove_socket(sock, pfkey_open_sockets);
+	pfkey_list_remove_socket(sock, pfkey_registered_sockets);
+
 	MOD_DEC_USE_COUNT;
 	KLIPS_PRINT(debug_pfkey,
 		    "klips_debug:pfkey_release: succeeded.\n");
 
-	pfkey_list_remove_socket(sock, pfkey_open_sockets);
-	pfkey_list_remove_socket(sock, pfkey_registered_sockets);
 	return 0;
 }
 
@@ -1029,16 +979,22 @@ pfkey_sendmsg(struct socket *sock, struct msghdr *msg, int len, int nonblock, in
 		struct sock *sockp = pfkey_sock_list;
 
 		pfkey_msg->sadb_msg_errno = error;
-		KLIPS_PRINT(debug_pfkey,
-			    "klips_debug:pfkey_sendmsg: send up msg.\n");
 		while(sockp != NULL) {
+			KLIPS_PRINT(debug_pfkey,
+				    "klips_debug:pfkey_sendmsg: "
+				    "send up pfkey_msg=%p to socket=%p.\n",
+				    pfkey_msg, sockp->socket);
 			pfkey_upmsg(sockp->socket, pfkey_msg);
 			sockp = sockp->next;
 		}
 		kfree_s((void*)pfkey_msg, len);
 	}
 
-	return len;
+	if(error) {
+		return error;
+	} else {
+		return len;
+	}
 }
 
 /*
@@ -1155,7 +1111,25 @@ struct net_proto_family pfkey_family_ops = {
 	pfkey_create
 };
 
-struct proto_ops pfkey_dgram_ops = {
+struct proto_ops SOCKOPS_WRAPPED(pfkey_ops) = {
+#ifdef NETDEV_23
+	family:		PF_KEY,
+	release:	pfkey_release,
+	bind:		sock_no_bind,
+	connect:	sock_no_connect,
+	socketpair:	sock_no_socketpair,
+	accept:		sock_no_accept,
+	getname:	sock_no_getname,
+	poll:		datagram_poll,
+	ioctl:		sock_no_ioctl,
+	listen:		sock_no_listen,
+	shutdown:	pfkey_shutdown,
+	setsockopt:	sock_no_setsockopt,
+	getsockopt:	sock_no_getsockopt,
+	sendmsg:	pfkey_sendmsg,
+	recvmsg:	pfkey_recvmsg,
+	mmap:		sock_no_mmap,
+#else
 	PF_KEY,
 	sock_no_dup,
 	pfkey_release,
@@ -1173,7 +1147,14 @@ struct proto_ops pfkey_dgram_ops = {
 	sock_no_fcntl,
 	pfkey_sendmsg,
 	pfkey_recvmsg
+#endif
 };
+
+#ifdef NETDEV_23
+#include <linux/smp_lock.h>
+SOCKOPS_WRAP(pfkey, PF_KEY);
+#endif
+
 #else /* NET_21 */
 struct proto_ops pfkey_proto_ops = {
 	PF_KEY,
@@ -1202,14 +1183,18 @@ struct proto_ops pfkey_proto_ops = {
 static
 #endif /* PROC_FS_2325 */
 int
-pfkey_get_info(char *buffer, char **start, off_t offset, int length, int dummy)
+pfkey_get_info(char *buffer, char **start, off_t offset, int length
+#ifndef  PROC_NO_DUMMY
+, int dummy
+#endif /* !PROC_NO_DUMMY */
+)
 {
 	off_t pos=0;
 	off_t begin=0;
 	int len=0;
 	struct sock *sk=pfkey_sock_list;
 	
-	len+= sprintf(buffer,"sock     pid  d sleep    socket   next     prev     e destruct r z fa n p r  w o sndbf stamp    Flags    Type     St\n");
+	len+= sprintf(buffer,"sock     pid  d sleep    socket   next     prev     e destruct r z fa n p r w o sndbf stamp    Flags    Type     St\n");
 	
 	while(sk!=NULL) {
 #ifdef NET_21
@@ -1371,6 +1356,33 @@ pfkey_proto_init(struct net_proto *pro)
 
 /*
  * $Log: pfkey_v2.c,v $
+ * Revision 1.31  2000/05/16 03:04:00  rgb
+ * Updates for 2.3.99pre8 from MB.
+ *
+ * Revision 1.30  2000/05/10 19:22:21  rgb
+ * Use sklist private functions for 2.3.xx compatibility.
+ *
+ * Revision 1.29  2000/03/22 16:17:03  rgb
+ * Fixed SOCKOPS_WRAPPED macro for SMP (MB).
+ *
+ * Revision 1.28  2000/02/21 19:30:45  rgb
+ * Removed references to pkt_bridged for 2.3.47 compatibility.
+ *
+ * Revision 1.27  2000/02/14 21:07:00  rgb
+ * Fixed /proc/net/pf-key legend spacing.
+ *
+ * Revision 1.26  2000/01/22 03:46:59  rgb
+ * Fixed pfkey error return mechanism so that we are able to free the
+ * local copy of the pfkey_msg, plugging a memory leak and silencing
+ * the bad object free complaints.
+ *
+ * Revision 1.25  2000/01/21 06:19:44  rgb
+ * Moved pfkey_list_remove_socket() calls to before MOD_USE_DEC_COUNT.
+ * Added debugging to pfkey_upmsg.
+ *
+ * Revision 1.24  2000/01/10 16:38:23  rgb
+ * MB fixups for 2.3.x.
+ *
  * Revision 1.23  1999/12/09 23:22:16  rgb
  * Added more instrumentation for debugging 2.0 socket
  * selection/reading.
