@@ -14,7 +14,7 @@
  * for more details.
  */
 
-char spi_c_version[] = "RCSID $Id: spi.c,v 1.53 1999/12/10 17:35:37 rgb Exp $";
+char spi_c_version[] = "RCSID $Id: spi.c,v 1.59 2000/06/21 16:51:27 rgb Exp $";
 
 #include <asm/types.h>
 #include <sys/types.h>
@@ -41,6 +41,10 @@ char spi_c_version[] = "RCSID $Id: spi.c,v 1.53 1999/12/10 17:35:37 rgb Exp $";
 #include <stdlib.h>
 #include <freeswan.h>
 #include <linux/autoconf.h>	/* CONFIG_IPSEC_PFKEYv2 */
+/* permanently turn it on since netlink support has been disabled */
+#ifndef CONFIG_IPSEC_PFKEYv2
+     #define CONFIG_IPSEC_PFKEYv2
+#endif /* CONFIG_IPSEC_PFKEYv2 */
 #ifdef CONFIG_IPSEC_PFKEYv2
      #include <signal.h>
      #include <sys/socket.h>
@@ -84,23 +88,20 @@ uint32_t pfkey_seq = 0;
 static const char *usage_string = "\
 Usage:\n\
 	in the following, <SA> is: --edst <dstaddr> --spi <spi> --proto <proto>\n\
+                               OR: --said <proto><spi>@<dstaddr>
 spi --clear\n\
 spi --help\n\
 spi --version\n\
-spi --del\n\
+spi\n\
+spi --del <SA>\n\
 spi --ip4 <SA> --src <encap-src> --dst <encap-dst>\n\
-spi --ah md5 <SA> --authkey <key>\n\
 spi --ah <algo> <SA> [ --replay_window <replay_window> ] --authkey <key>\n\
 	where <algo> is one of:	hmac-md5-96 | hmac-sha1-96\n\
-spi --esp des-old <SA> --iv <iv> --authkey <key>\n\
 spi --esp <algo> <SA> [ --replay_window <replay-window> ] --enckey <ekey> --authkey <akey>\n\
-	where <algo> is one of:	des-md5 | 3des-md5\n\
-				des-md5-96 | 3des-md5-96\n\
-				des-sha1-96 | 3des-sha1-96\n\
-spi --esp <algo> <SA> [ --replay_window <replay-window> ] --authkey <akey>\n\
-	where <algo> is one of:	null-md5-96 | null-sha1-96 \n\
-spi --esp <algo> <SA> [ --replay_window <replay-window> ] --iv <iv> --enckey <ekey>\n\
-	where <algo> is one of:	des | 3des\n\
+	where <algo> is one of:	3des-md5-96 | 3des-sha1-96\n\
+spi --esp <algo> <SA> [ --replay_window <replay-window> ] --enckey <ekey>\n\
+	where <algo> is:	3des\n\
+[ --debug ] is optional to any spi command.
 ";
 
 struct cmd
@@ -113,9 +114,7 @@ struct cmd
 int f_ip4(void);
 int f_ah(void);
 int f_esp(void);
-int f_md5(void);
 int f_hmac(void);
-int f_des(void);
 int f_blkrply(void);
 int f_del(void);
 int f_clr(void);
@@ -188,42 +187,6 @@ f_clr(void)
 	return 0;
 }
 
-int
-f_md5(void)
-{
-	struct ahmd5_xdata *xd;
-
-	if(authkeylen > AHMD5_KMAX) {
-		fprintf(stderr, "%s: Failed -- (%s) requires an authentication key length < %d bytes, found %d in '%s' for SA:%s (1 byte = 2 hexadecimal digits).\n",
-				program_name, alg_opt, AHMD5_KMAX, authkeylen, akey_opt, sa);
-#if 0
-		fprintf(stderr, "%s: authentication key invalid or not specified -- must be 16 bytes.\n",
-			program_name);
-#endif
-	}
-	em = (struct encap_msghdr *)malloc(sizeof(struct encap_msghdr) +
-		  sizeof(struct ahmd5_xdata));
-	if(em == NULL) {
-		fprintf(stderr, "%s: Memory allocation error\n", program_name);
-		exit(1);
-	}
-
-	xd = (struct ahmd5_xdata *)(em->em_dat);
-
-	em->em_msglen = EMT_SETSPI_FLEN + 2 * sizeof (u_short) + authkeylen;
-	if(proto != SA_AH) {
-		fprintf(stderr, "%s: SA protocol selector (ah) does not agree with the Authentication algorithm requested (%s) for SA:%s.\n",
-			program_name, alg_opt, sa);
-	}
-	em->em_alg = alg;
-	
-	xd->amx_klen = authkeylen;
-	xd->amx_alen = 16;
-
-	memcpy(&(xd->amx_key), authkey, authkeylen);
-	return 0;
-}
-
 int
 f_hmac(void)
 {
@@ -284,43 +247,6 @@ f_hmac(void)
 	return 0;
 }
 
-int
-f_des(void)
-{
-	struct espdesold_xdata *xd;
-	
-	if((em = malloc(sizeof(struct encap_msghdr) + sizeof(struct espdesold_xdata))) == NULL) {
-		fprintf(stderr, "%s: Memory allocation error\n", program_name);
-		exit(1);
-	}
-	em->em_msglen = EMT_SETSPI_FLEN + EMT_ESPDESCBC_ULEN;
-	if(proto != SA_ESP) {
-		fprintf(stderr, "%s: SA protocol selector (esp) does not agree with the Encryption algorithm requested (%s) for SA:%s.\n",
-			program_name, alg_opt, sa);
-	}
-	em->em_alg = XF_ESPDESOLD;
-
-	if ((ivlen != 4) && (ivlen != 8)) {
-		fprintf(stderr, "%s: Failed -- requires IV lengths of 4 or 8 bytes, found %d in '%s' for SA:%s (1 byte = 2 hexadecimal digits).\n",
-			program_name, ivlen, iv_opt, sa);
-		exit(1);
-	}
-	
-	xd = (struct espdesold_xdata *)em->em_dat;
-	xd->edx_ivlen = ivlen;
-
-	memcpy(&(xd->edx_iv), iv, ivlen);
-
-	if (enckeylen != 8) {
-		fprintf(stderr, "%s: Failed -- requires a key length of 8 bytes, found %d in '%s' for SA:%s (1 byte = 2 hexadecimal digits).\n",
-			program_name, enckeylen, ekey_opt, sa);
-		exit(1);
-	}
-	
-	memcpy(&(xd->edx_rk), enckey, enckeylen);
-	return 0;
-}
-
 int
 f_blkrply(void)
 {
@@ -337,13 +263,11 @@ f_blkrply(void)
 	ed->eme_flags = 0;
         if ((alg != XF_ESPNULLMD596) && (alg != XF_ESPNULLSHA196)) {
 		switch(alg) {
-		case XF_ESPDESMD5:
 		case XF_ESPDESMD596:
 		case XF_ESPDESSHA196:
 		case XF_ESPDES:
 			eklen = EMT_ESPDES_KEY_SZ;
 			break;
-		case XF_ESP3DESMD5:
 		case XF_ESP3DESMD596:
 		case XF_ESP3DESSHA196:
 		case XF_ESP3DES:
@@ -365,9 +289,6 @@ f_blkrply(void)
 	}
 	
 	switch(alg) {
-	case XF_ESPDESMD5:
-	case XF_ESP3DESMD5:
-
 	case XF_ESPDESMD596:
 	case XF_ESP3DESMD596:
 	case XF_ESPNULLMD596:
@@ -406,6 +327,7 @@ f_blkrply(void)
 	return 0;
 }
 
+#if 0
 static int
 resolve_ip(char *arg, struct in_addr *ip)
 {
@@ -432,6 +354,7 @@ resolve_ip(char *arg, struct in_addr *ip)
 	memcpy((char *) ip, (char *) hostp->h_addr_list[0], hostp->h_length);
 	return(0);
 }
+#endif
 
 #ifdef CONFIG_IPSEC_PFKEYv2
 #ifdef SIGS_FROM_BELOW
@@ -627,7 +550,7 @@ main(int argc, char *argv[])
 	char *endptr;
 	__u32 spi = 0;
 	int c, previous = -1;
-	int ret;
+/*	int ret; */
 	struct sa_id said;
 	size_t sa_len;
 	const char* error_s;
@@ -669,14 +592,8 @@ main(int argc, char *argv[])
 					program_name);
 				exit(1);
 			}
-			if(!strcmp(optarg, "md5")) {
-				alg = XF_AHMD5;
-			} else if(!strcmp(optarg, "hmac-md5")) {
+			if       (!strcmp(optarg, "hmac-md5-96")) {
 				alg = XF_AHHMACMD5;
-			} else if(!strcmp(optarg, "hmac-md5-96")) {
-				alg = XF_AHHMACMD5;
-			} else if(!strcmp(optarg, "hmac-sha1")) {
-				alg = XF_AHHMACSHA1;
 			} else if(!strcmp(optarg, "hmac-sha1-96")) {
 				alg = XF_AHHMACSHA1;
 			} else {
@@ -695,13 +612,7 @@ main(int argc, char *argv[])
 					program_name);
 				exit(1);
 			}
-			if       (!strcmp(optarg, "des-old")) {
-				alg = XF_ESPDESOLD;
-			} else if(!strcmp(optarg, "des-md5")) {
-				alg = XF_ESPDESMD5;
-			} else if(!strcmp(optarg, "3des-md5")) {
-				alg = XF_ESP3DESMD5;
-			} else if(!strcmp(optarg, "des-md5-96")) {
+			if       (!strcmp(optarg, "des-md5-96")) {
 				alg = XF_ESPDESMD596;
 			} else if(!strcmp(optarg, "3des-md5-96")) {
 				alg = XF_ESP3DESMD596;
@@ -774,6 +685,14 @@ main(int argc, char *argv[])
 					program_name, optarg, edst_opt);
 				exit (1);
 			}				
+			if((error_s = atoaddr(optarg, 0, &edst)) != 0) {
+				if(error_s) {
+					fprintf(stderr, "%s: Error, %s converting --edst argument:%s\n",
+						program_name, error_s, optarg);
+					exit (1);
+				}
+			}
+#if 0
 			if((ret = resolve_ip(optarg, &edst)) != 0) {
 				if(ret == 1) {
 					fprintf(stderr, "%s: Destination address cannot be a network: %s\n",
@@ -785,6 +704,7 @@ main(int argc, char *argv[])
 					exit (1);
 				}
 			}
+#endif
 			edst_opt = optarg;
 			if(debug) {
 				addrtoa(edst, 0, ipaddr_txt, sizeof(ipaddr_txt));
@@ -974,6 +894,14 @@ main(int argc, char *argv[])
 					program_name, optarg, dst_opt);
 				exit (1);
 			}				
+			if((error_s = atoaddr(optarg, 0, &dst)) != 0) {
+				if(error_s) {
+					fprintf(stderr, "%s: Error, %s converting --dst argument:%s\n",
+						program_name, error_s, optarg);
+					exit (1);
+				}
+			}
+#if 0
 			if((ret = resolve_ip(optarg, &dst)) != 0) {
 				if(ret == 1) {
 					fprintf(stderr, "%s: Tunnel destination address cannot be a network: %s\n",
@@ -985,6 +913,7 @@ main(int argc, char *argv[])
 					exit (1);
 				}
 			}
+#endif
 			dst_opt = optarg;
 			if(debug) {
 				addrtoa(dst, 0, ipaddr_txt, sizeof(ipaddr_txt));
@@ -997,6 +926,14 @@ main(int argc, char *argv[])
 					program_name, optarg, src_opt);
 				exit (1);
 			}				
+			if((error_s = atoaddr(optarg, 0, &src)) != 0) {
+				if(error_s) {
+					fprintf(stderr, "%s: Error, %s converting --src argument:%s\n",
+						program_name, error_s, optarg);
+					exit (1);
+				}
+			}
+#if 0
 			if((ret = resolve_ip(optarg, &src)) != 0) {
 				if(ret == 1) {
 					fprintf(stderr, "%s: Tunnel source address cannot be a network: %s\n",
@@ -1008,6 +945,7 @@ main(int argc, char *argv[])
 					exit (1);
 				}
 			}
+#endif
 			src_opt = optarg;
 			if(debug) {
 				addrtoa(dst, 0, ipaddr_txt, sizeof(ipaddr_txt));
@@ -1046,12 +984,8 @@ main(int argc, char *argv[])
 	switch(alg) {
 	case XF_IP4:
 	case XF_DEL:
-	case XF_AHMD5:
 	case XF_AHHMACMD5:
 	case XF_AHHMACSHA1:
-	case XF_ESPDESOLD:
-	case XF_ESPDESMD5:
-	case XF_ESP3DESMD5:
 	case XF_ESPDESMD596:
 	case XF_ESP3DESMD596:
 	case XF_ESPDESSHA196:
@@ -1108,18 +1042,10 @@ main(int argc, char *argv[])
 	case XF_IP4:
 		f_ip4();
 		break;
-	case XF_AHMD5:
-		f_md5();
-		break;
 	case XF_AHHMACMD5:
 	case XF_AHHMACSHA1:
 		f_hmac();
 		break;
-	case XF_ESPDESOLD:
-		f_des();
-		break;
-	case XF_ESPDESMD5:
-	case XF_ESP3DESMD5:
 	case XF_ESPDESMD596:
 	case XF_ESP3DESMD596:
 	case XF_ESPDESSHA196:
@@ -1230,7 +1156,7 @@ main(int argc, char *argv[])
 #ifdef PFKEY_REGISTER
 	/* for registering SA types that can be negociated */
 	/* Register the fact that I can negociate AH SA's */
-	memset((caddr_t)extensions, 0, sizeof(struct sadb_ext *) * (SADB_EXT_MAX + 1));
+	pfkey_extensions_init(extensions);
 	if((error = pfkey_msg_hdr_build(&extensions[0],
 					SADB_REGISTER,
 					SADB_SATYPE_AH,
@@ -1239,25 +1165,33 @@ main(int argc, char *argv[])
 					getpid()))) {
 		fprintf(stderr, "%s: Trouble building message header, error=%d.\n",
 			program_name, error);
+		pfkey_extensions_free(extensions);
 		exit(1);
 	}
-	if((error = pfkey_msg_build(extensions, EXT_BITS_IN))) {
+	if((error = pfkey_msg_build(&pfkey_msg, extensions, EXT_BITS_IN))) {
 		fprintf(stderr, "%s: Trouble building pfkey message, error=%d.\n",
 			program_name, error);
+		pfkey_extensions_free(extensions);
+		pfkey_msg_free(&pfkey_msg);
 		exit(1);
 	}
-	pfkey_msg = (struct sadb_msg*)extensions[0];
+/*	pfkey_msg = (struct sadb_msg*)extensions[0]; */
      
 	if(write(pfkey_sock, pfkey_msg,
 		 pfkey_msg->sadb_msg_len * IPSEC_PFKEYv2_ALIGN) !=
 	   pfkey_msg->sadb_msg_len * IPSEC_PFKEYv2_ALIGN) {
 		/* cleanup code here */
 		fprintf(stderr, "%s: Trouble writing to channel PF_KEY.\n", program_name);
+		pfkey_extensions_free(extensions);
+		pfkey_msg_free(&pfkey_msg);
 		exit(1);
 	}
 	
+	pfkey_extensions_free(extensions);
+	pfkey_msg_free(&pfkey_msg);
+
 	/* Register the fact that I can negociate ESP SA's */
-	memset((caddr_t)extensions, 0, sizeof(struct sadb_ext *) * (SADB_EXT_MAX + 1));
+	pfkey_extensions_init(extensions);
 	if((error = pfkey_msg_hdr_build(&extensions[0],
 					SADB_REGISTER,
 					SADB_SATYPE_ESP,
@@ -1266,28 +1200,36 @@ main(int argc, char *argv[])
 					getpid()))) {
 		fprintf(stderr, "%s: Trouble building message header, error=%d.\n",
 			program_name, error);
+		pfkey_extensions_free(extensions);
 		exit(1);
 	}
-	if((error = pfkey_msg_build(extensions, EXT_BITS_IN))) {
+	if((error = pfkey_msg_build(&pfkey_msg, extensions, EXT_BITS_IN))) {
 		fprintf(stderr, "%s: Trouble building pfkey message, error=%d.\n",
 			program_name, error);
+		pfkey_extensions_free(extensions);
+		pfkey_msg_free(&pfkey_msg);
 		exit(1);
 	}
-	pfkey_msg = (struct sadb_msg*)extensions[0];
+/*	pfkey_msg = (struct sadb_msg*)extensions[0]; */
 	
 	if(write(pfkey_sock, pfkey_msg,
 		 pfkey_msg->sadb_msg_len * IPSEC_PFKEYv2_ALIGN) !=
 	   pfkey_msg->sadb_msg_len * IPSEC_PFKEYv2_ALIGN) {
 		fprintf(stderr, "%s: Trouble writing to channel PF_KEY.\n", program_name);
 		/* cleanup code here */
+		pfkey_extensions_free(extensions);
+		pfkey_msg_free(&pfkey_msg);
 		exit(1);
 	}
+
+	pfkey_extensions_free(extensions);
+	pfkey_msg_free(&pfkey_msg);
 #endif /* PFKEY_REGISTER */
 	
 	/* Build an SADB_ADD message to send down. */
 	/* It needs <base, SA, address(SD), key(AE)> minimum. */
 	/*   Lifetime(HS) could be added before addresses. */
-	memset((caddr_t)extensions, 0, sizeof(struct sadb_ext *) * (SADB_EXT_MAX + 1));
+	pfkey_extensions_init(extensions);
 	if(debug) {
 		fprintf(stdout, "%s: extensions=%p &extensions=%p extensions[0]=%p &extensions[0]=%p cleared.\n",
 			program_name,
@@ -1297,15 +1239,19 @@ main(int argc, char *argv[])
 			&extensions[0]);
 	}
 	if((error = pfkey_msg_hdr_build(&extensions[0],
-			    (alg == XF_DEL ? SADB_DELETE : alg == XF_CLR ? SADB_FLUSH : SADB_ADD),
-			    (proto == SA_ESP ? SADB_SATYPE_ESP :
-			     (proto == SA_AH ? SADB_SATYPE_AH :
-			      (proto == SA_IPIP ? SADB_X_SATYPE_IPIP : SADB_SATYPE_UNSPEC))),
-			    0,
+					(alg == XF_DEL ? SADB_DELETE : alg == XF_CLR ? SADB_FLUSH : SADB_ADD),
+					proto2satype(proto),
+#if 0
+					(proto == SA_ESP ? SADB_SATYPE_ESP :
+					 (proto == SA_AH ? SADB_SATYPE_AH :
+					  (proto == SA_IPIP ? SADB_X_SATYPE_IPIP : SADB_SATYPE_UNSPEC))),
+#endif
+					0,
 			    ++pfkey_seq,
 			    getpid()))) {
 		fprintf(stderr, "%s: Trouble building message header, error=%d.\n",
 			program_name, error);
+		pfkey_extensions_free(extensions);
 		exit(1);
 	}
 	if(debug) {
@@ -1333,7 +1279,6 @@ main(int argc, char *argv[])
 	case XF_ESPNULLSHA196:
 		authalg = SADB_AALG_SHA1HMAC;
 		break;
-	case XF_AHMD5:
 	case XF_ESPDESMD5:
 	case XF_ESP3DESMD5:
 	default:
@@ -1354,9 +1299,6 @@ main(int argc, char *argv[])
 	case XF_ESPNULLSHA196:
 		encryptalg = SADB_EALG_NULL;
 		break;
-	case XF_ESPDESOLD:
-	case XF_ESPDESMD5:
-	case XF_ESP3DESMD5:
 	default:
 		encryptalg = SADB_EALG_NONE;
 	}
@@ -1371,6 +1313,7 @@ main(int argc, char *argv[])
 					   0))) {
 			fprintf(stderr, "%s: Trouble building sa extension, error=%d.\n",
 				program_name, error);
+			pfkey_extensions_free(extensions);
 			exit(1);
 		}
 		if(debug) {
@@ -1394,6 +1337,7 @@ main(int argc, char *argv[])
 						 -1))) {
 			fprintf(stderr, "%s: Trouble building lifetime_s extension, error=%d.\n",
 				program_name, error);
+			pfkey_extensions_free(extensions);
 			exit(1);
 		}
 
@@ -1410,6 +1354,7 @@ main(int argc, char *argv[])
 						 -1))) {
 			fprintf(stderr, "%s: Trouble building lifetime_h extension, error=%d.\n",
 				program_name, error);
+			pfkey_extensions_free(extensions);
 			exit(1);
 		}
 
@@ -1440,6 +1385,7 @@ main(int argc, char *argv[])
 				0, ipaddr_txt, sizeof(ipaddr_txt));
 			fprintf(stderr, "%s: Trouble building address_s extension (%s), error=%d.\n",
 				program_name, ipaddr_txt, error);
+			pfkey_extensions_free(extensions);
 			exit(1);
 		}
 		if(debug) {
@@ -1470,6 +1416,7 @@ main(int argc, char *argv[])
 				0, ipaddr_txt, sizeof(ipaddr_txt));
 			fprintf(stderr, "%s: Trouble building address_d extension (%s), error=%d.\n",
 				program_name, ipaddr_txt, error);
+			pfkey_extensions_free(extensions);
 			exit(1);
 		}
 		if(debug) {
@@ -1491,6 +1438,7 @@ main(int argc, char *argv[])
 						(struct sockaddr*)&pfkey_address_p_ska))) {
 			fprintf(stderr, "%s: Trouble building address_p extension, error=%d.\n",
 				program_name, error);
+			pfkey_extensions_free(extensions);
 			exit(1);
 		}
 		if(debug) {
@@ -1513,6 +1461,7 @@ main(int argc, char *argv[])
 						    authkey))) {
 				fprintf(stderr, "%s: Trouble building key_a extension, error=%d.\n",
 					program_name, error);
+				pfkey_extensions_free(extensions);
 				exit(1);
 			}
 			if(debug) {
@@ -1536,6 +1485,7 @@ main(int argc, char *argv[])
 						    enckey))) {
 				fprintf(stderr, "%s: Trouble building key_e extension, error=%d.\n",
 					program_name, error);
+				pfkey_extensions_free(extensions);
 				exit(1);
 			}
 			if(debug) {
@@ -1554,6 +1504,7 @@ main(int argc, char *argv[])
 				      pfkey_ident_s_ska))) {
 			fprintf(stderr, "%s: Trouble building ident_s extension, error=%d.\n",
 				program_name, error);
+			pfkey_extensions_free(extensions);
 			exit(1);
 		}
 		if(subnettoa(addr, mask, format, pfkey_ident_s_ska,
@@ -1568,6 +1519,7 @@ main(int argc, char *argv[])
 					      pfkey_ident_d_ska))) {
 			fprintf(stderr, "%s: Trouble building ident_d extension, error=%d.\n",
 				program_name, error);
+			pfkey_extensions_free(extensions);
 			exit(1);
 		}
 		if(subnettoa(addr, mask, format, pfkey_ident_d_ska,
@@ -1586,22 +1538,17 @@ main(int argc, char *argv[])
 		fprintf(stdout, "%s: assembling pfkey msg....\n",
 			program_name);
 	}
-	if((error = pfkey_msg_build(extensions, EXT_BITS_IN))) {
+	if((error = pfkey_msg_build(&pfkey_msg, extensions, EXT_BITS_IN))) {
 		fprintf(stderr, "%s: Trouble building pfkey message, error=%d.\n",
 			program_name, error);
+		pfkey_extensions_free(extensions);
+		pfkey_msg_free(&pfkey_msg);
 		exit(1);
 	}
 	if(debug) {
 		fprintf(stdout, "%s: assembled.\n",
 			program_name);
 	}
-	pfkey_msg = (struct sadb_msg*)extensions[0];
-	if(debug) {
-		fprintf(stdout, "%s: extensions[0]=%p assigned to pfkey_msg=%p.\n",
-			program_name,
-			extensions[0],
-			pfkey_msg);
-	}
 	if(debug) {
 		fprintf(stdout, "%s: writing pfkey msg.\n",
 			program_name);
@@ -1612,7 +1559,8 @@ main(int argc, char *argv[])
 	   pfkey_msg->sadb_msg_len * IPSEC_PFKEYv2_ALIGN) {
 		fprintf(stderr, "%s: pfkey write failed, returning %d with errno=%d.\n",
 			program_name, error, errno);
-		/* cleanup code here */
+		pfkey_extensions_free(extensions);
+		pfkey_msg_free(&pfkey_msg);
 		switch(errno) {
 		case EACCES:
 			fprintf(stderr, "access denied.  ");
@@ -1701,8 +1649,8 @@ main(int argc, char *argv[])
 	pfkey_sig_handler(0); /* solicit upmsg */
 #endif
 	if(pfkey_msg) {
-		memset((caddr_t)&pfkey_msg, 0, pfkey_msg->sadb_msg_len * IPSEC_PFKEYv2_ALIGN);
-		free(pfkey_msg);
+		pfkey_extensions_free(extensions);
+		pfkey_msg_free(&pfkey_msg);
 	}
 	if(debug) {
 		fprintf(stdout, "%s: pfkey message buffer freed.\n",
@@ -1796,6 +1744,29 @@ main(int argc, char *argv[])
 
 /*
  * $Log: spi.c,v $
+ * Revision 1.59  2000/06/21 16:51:27  rgb
+ * Added no additional argument option to usage text.
+ *
+ * Revision 1.58  2000/03/16 06:40:49  rgb
+ * Hardcode PF_KEYv2 support.
+ *
+ * Revision 1.57  2000/01/22 23:22:46  rgb
+ * Use new function proto2satype().
+ *
+ * Revision 1.56  2000/01/21 09:42:32  rgb
+ * Replace resolve_ip() with atoaddr() from freeswanlib.
+ *
+ * Revision 1.55  2000/01/21 06:24:57  rgb
+ * Blasted any references in usage and code to deleted algos.
+ * Removed DES usage.
+ * Changed usage of memset on extensions to pfkey_extensions_init().
+ *
+ * Revision 1.54  1999/12/29 21:17:41  rgb
+ * Changed pfkey_msg_build() I/F to include a struct sadb_msg**
+ * parameter for cleaner manipulation of extensions[] and to guard
+ * against potential memory leaks.
+ * Changed the I/F to pfkey_msg_free() for the same reason.
+ *
  * Revision 1.53  1999/12/10 17:35:37  rgb
  * Added address debugging.
  * Fixed undetected spi followed by said sanity check bug.
