@@ -12,14 +12,14 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: pfkey_v2_build.c,v 1.7 1999/12/09 23:12:20 rgb Exp $
+ * RCSID $Id: pfkey_v2_build.c,v 1.13 2000/06/02 22:54:14 rgb Exp $
  */
 
 /*
  *		Template from klips/net/ipsec/ipsec/ipsec_parser.c.
  */
 
-char pfkey_v2_build_c_version[] = "$Id: pfkey_v2_build.c,v 1.7 1999/12/09 23:12:20 rgb Exp $";
+char pfkey_v2_build_c_version[] = "$Id: pfkey_v2_build.c,v 1.13 2000/06/02 22:54:14 rgb Exp $";
 
 /*
  * Some ugly stuff to allow consistent debugging code for use in the
@@ -28,48 +28,50 @@ char pfkey_v2_build_c_version[] = "$Id: pfkey_v2_build.c,v 1.7 1999/12/09 23:12:
 
 #ifdef __KERNEL__
 
-#include <linux/kernel.h>  /* for printk */
+# include <linux/kernel.h>  /* for printk */
 
-#include <linux/malloc.h> /* kmalloc() */
-#include <linux/errno.h>  /* error codes */
-#include <linux/types.h>  /* size_t */
-#include <linux/interrupt.h> /* mark_bh */
+# include <linux/malloc.h> /* kmalloc() */
+# include <linux/errno.h>  /* error codes */
+# include <linux/types.h>  /* size_t */
+# include <linux/interrupt.h> /* mark_bh */
 
-#include <linux/netdevice.h>   /* struct device, and other headers */
-#include <linux/etherdevice.h> /* eth_type_trans */
-#include <linux/ip.h>          /* struct iphdr */ 
-#if defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE)
-#include <linux/ipv6.h>        /* struct ipv6hdr */
-#endif /* if defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE) */
+# include <linux/netdevice.h>   /* struct device, and other headers */
+# include <linux/etherdevice.h> /* eth_type_trans */
+# include <linux/ip.h>          /* struct iphdr */ 
+# if defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE)
+#  include <linux/ipv6.h>        /* struct ipv6hdr */
+# endif /* if defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE) */
 extern int debug_pfkey;
 
-#define DEBUGGING(format,args...) \
+# define DEBUGGING(format,args...) \
          ((debug_pfkey) ? printk(KERN_INFO "klips_debug:" format , ## args) : 0)
-#define MALLOC(size) kmalloc(size, GFP_ATOMIC)
-#define FREE(obj) kfree(obj)
+# define MALLOC(size) kmalloc(size, GFP_ATOMIC)
+# define FREE(obj) kfree(obj)
 #else /* __KERNEL__ */
 
-#include <sys/types.h>
-#include <linux/types.h>
-#include <linux/errno.h>
-#include <malloc.h>
-#include <string.h> /* memset */
+# include <sys/types.h>
+# include <linux/types.h>
+# include <linux/errno.h>
+# include <malloc.h>
+# include <string.h> /* memset */
 
-#include "../pluto/constants.h" 
-#include "../pluto/defs.h"  /* for PRINTF_LIKE */
-#include "../pluto/log.h"  /* for debugging and DBG_log */
+# include "../pluto/constants.h" 
+# include "../pluto/defs.h"  /* for PRINTF_LIKE */
+# include "../pluto/log.h"  /* for debugging and DBG_log */
 
 extern unsigned int debugging;  /* bits selecting what to report */
 unsigned int pfkey_lib_debug = 0;
 
-#ifdef PLUTO
-#define DEBUGGING(format,args...)  { DBG_log("pfkey_lib_debug:" format, ## args);  }
-#else
-#define DEBUGGING(format,args...)  if(pfkey_lib_debug) { printf("pfkey_lib_debug:" format, ## args); } else { ; }
-#endif
-#define MALLOC(size) malloc(size)
-#define FREE(obj) free(obj)
-#endif
+/* #define PLUTO */
+
+# ifdef PLUTO
+#  define DEBUGGING(format,args...)  { DBG_log("pfkey_lib_debug:" format, ## args);  }
+# else
+#  define DEBUGGING(format,args...)  if(pfkey_lib_debug) { printf("pfkey_lib_debug:" format, ## args); } else { ; }
+# endif
+# define MALLOC(size) malloc(size)
+# define FREE(obj) free(obj)
+#endif /* __KERNEL__ */
 
 #include <freeswan.h>
 #include <pfkeyv2.h>
@@ -92,6 +94,10 @@ pfkey_extensions_free(struct sadb_ext *extensions[SADB_EXT_MAX + 1])
 {
 	int i;
 	
+	if(!extensions) {
+		return;
+	}
+
 	if(extensions[0]) {
 		memset(extensions[0], 0, sizeof(struct sadb_msg));
 		FREE(extensions[0]);
@@ -108,12 +114,12 @@ pfkey_extensions_free(struct sadb_ext *extensions[SADB_EXT_MAX + 1])
 }
 
 void
-pfkey_msg_free(struct sadb_msg *pfkey_msg)
+pfkey_msg_free(struct sadb_msg **pfkey_msg)
 {
-	if(pfkey_msg) {
-		memset(pfkey_msg, 0, pfkey_msg->sadb_msg_len * IPSEC_PFKEYv2_ALIGN);
-		FREE(pfkey_msg);
-		pfkey_msg = NULL;
+	if(*pfkey_msg) {
+		memset(*pfkey_msg, 0, (*pfkey_msg)->sadb_msg_len * IPSEC_PFKEYv2_ALIGN);
+		FREE(*pfkey_msg);
+		*pfkey_msg = NULL;
 	}
 }
 
@@ -353,7 +359,7 @@ pfkey_address_build(struct sadb_ext**	pfkey_ext,
 	struct sadb_address *pfkey_address = (struct sadb_address *)*pfkey_ext;
 	
 	DEBUGGING(
-		"pfkey_address_build:\n");
+		"pfkey_address_build: exttype=%d proto=%d prefixlen=%d\n", exttype, proto, prefixlen);
 	/* sanity checks... */
 	if(pfkey_address) {
 		DEBUGGING(
@@ -361,31 +367,60 @@ pfkey_address_build(struct sadb_ext**	pfkey_ext,
 		SENDERR(EINVAL);
 	}
 	
+	switch(exttype) {	
+	case SADB_EXT_ADDRESS_SRC:
+	case SADB_EXT_ADDRESS_DST:
+	case SADB_EXT_ADDRESS_PROXY:
+	case SADB_X_EXT_ADDRESS_DST2:
+	case SADB_X_EXT_ADDRESS_SRC_FLOW:
+	case SADB_X_EXT_ADDRESS_DST_FLOW:
+	case SADB_X_EXT_ADDRESS_SRC_MASK:
+	case SADB_X_EXT_ADDRESS_DST_MASK:
+		break;
+	default:
+		DEBUGGING( 
+			"pfkey_address_build: unrecognised ext_type=%d.\n", 
+			exttype); 
+		SENDERR(EINVAL); 
+	}
+
 	switch(address->sa_family) {
 	case AF_INET:
 		DEBUGGING(
 			"pfkey_address_build: found address family AF_INET.\n");
 		saddr_len = sizeof(struct sockaddr_in);
+#ifdef WITH_SOCKADDR_STORAGE
+		saddr = (char*)&((struct sockaddr*)address);
+#else
 		saddr = (char*)&((struct sockaddr_in*)address)->sin_addr;
+#endif /* WITH_SOCKADDR_STORAGE */
 		break;
-#if defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE)
+#ifdef WITH_SOCKADDR_STORAGE
+#if defined(WITH_IPV6) || defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE)
 	case AF_INET6:
 		DEBUGGING(
 			"pfkey_address_build: found address family AF_INET6.\n");
 		saddr_len = sizeof(struct sockaddr_in6);
-		saddr = (char*)&((struct sockaddr_in6*)address)->sin6_addr;
+		saddr = (char*)&((struct sockaddr*)address);
 		break;
-#endif /* defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE) */
+#endif /* defined(WITH_IPV6) || defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE) */
+#endif /* WITH_SOCKADDR_STORAGE */
 	default:
 		DEBUGGING(
 			"pfkey_address_build: address->sa_family=%d not supported.\n",
 			address->sa_family);
 		SENDERR(EPFNOSUPPORT);
 	}
-	
+
+#ifdef WITH_SOCKADDR_STORAGE	
+	addrtoa(*((struct sockaddr_storage*)saddr),
+		0,
+		ipaddr_txt, sizeof(ipaddr_txt));
+#else
 	addrtoa(*((struct in_addr*)saddr),
 		0,
 		ipaddr_txt, sizeof(ipaddr_txt));
+#endif /* WITH_SOCKADDR_STORAGE */
 	DEBUGGING(
 		"pfkey_address_build: found address=%s.\n",
 		ipaddr_txt);
@@ -395,27 +430,16 @@ pfkey_address_build(struct sadb_ext**	pfkey_ext,
 		SENDERR(EAFNOSUPPORT); /* not supported yet */
 	}
 
-	if( ! ( (exttype == SADB_EXT_ADDRESS_SRC) ||
-		(exttype == SADB_EXT_ADDRESS_DST) ||
-		(exttype == SADB_EXT_ADDRESS_PROXY) ||
-		(exttype == SADB_X_EXT_ADDRESS_DST2) ))
-	{
-		DEBUGGING( 
-			"pfkey_address_build: unrecognised ext_type=%d.\n", 
-			exttype); 
-		SENDERR(EINVAL); 
-	}
-
 	if(!(*pfkey_ext = (struct sadb_ext*)
 	     pfkey_address = (struct sadb_address*)
-	     MALLOC(ALIGN(sizeof(struct sadb_address) + saddr_len, IPSEC_PFKEYv2_ALIGN) ))) {
+	     MALLOC(ALIGN_N(sizeof(struct sadb_address) + saddr_len, IPSEC_PFKEYv2_ALIGN) ))) {
 		DEBUGGING(
 			"pfkey_lifetime_build: memory allocation failed\n");
 		SENDERR(ENOMEM);
 	}
 	memset(pfkey_address,
 	       0,
-	       ALIGN(sizeof(struct sadb_address) + saddr_len,
+	       ALIGN_N(sizeof(struct sadb_address) + saddr_len,
 		     IPSEC_PFKEYv2_ALIGN));
 	       
 	pfkey_address->sadb_address_len = DIVUP(sizeof(struct sadb_address) + saddr_len,
@@ -556,14 +580,14 @@ pfkey_ident_build(struct sadb_ext**	pfkey_ext,
 	    
 	if(!(*pfkey_ext = (struct sadb_ext*)
 	     pfkey_ident = (struct sadb_ident*)
-	     MALLOC(ALIGN(sizeof(struct sadb_key) + strlen(ident_string), IPSEC_PFKEYv2_ALIGN)))) {
+	     MALLOC(ALIGN_N(sizeof(struct sadb_key) + strlen(ident_string), IPSEC_PFKEYv2_ALIGN)))) {
 		DEBUGGING(
 			"pfkey_ident_build: memory allocation failed\n");
 		SENDERR(ENOMEM);
 	}
 	memset(pfkey_ident,
 	       0,
-	       ALIGN(sizeof(struct sadb_ident) + strlen(ident_string),
+	       ALIGN_N(sizeof(struct sadb_ident) + strlen(ident_string),
 		     IPSEC_PFKEYv2_ALIGN));
 	
 	pfkey_ident->sadb_ident_len = DIVUP(sizeof(struct sadb_ident) + strlen(ident_string),
@@ -581,14 +605,14 @@ pfkey_ident_build(struct sadb_ext**	pfkey_ext,
 	/* Which one is better, I don't know... */
 #if 0
 	for(i = strlen(ident_string);
-	    i < ALIGN(sizeof(struct sadb_ident) + strlen(ident_string), IPSEC_PFKEYv2_ALIGN);
+	    i < ALIGN_N(sizeof(struct sadb_ident) + strlen(ident_string), IPSEC_PFKEYv2_ALIGN);
 	    i++) {
 		((char*)pfkey_ident)[i] = 0;
 	}
 #else
 	memset(((char*)pfkey_ident) + sizeof(struct sadb_ident) + strlen(ident_string),
 	       0,
-	       ALIGN(sizeof(struct sadb_ident) + strlen(ident_string), IPSEC_PFKEYv2_ALIGN) -
+	       ALIGN_N(sizeof(struct sadb_ident) + strlen(ident_string), IPSEC_PFKEYv2_ALIGN) -
 	       sizeof(struct sadb_ident) + strlen(ident_string));
 #endif
 
@@ -601,11 +625,15 @@ pfkey_sens_build(struct sadb_ext**	pfkey_ext,
 		 uint32_t		dpd,
 		 uint8_t		sens_level,
 		 uint8_t		sens_len,
+		 uint64_t*		sens_bitmap,
 		 uint8_t		integ_level,
-		 uint8_t		integ_len)
+		 uint8_t		integ_len,
+		 uint64_t*		integ_bitmap)
 {
 	int error = 0;
 	struct sadb_sens *pfkey_sens = (struct sadb_sens *)*pfkey_ext;
+	int i;
+	uint64_t* bitmap;
 
 	/* sanity checks... */
 	if(pfkey_sens) {
@@ -621,16 +649,19 @@ pfkey_sens_build(struct sadb_ext**	pfkey_ext,
 
 	if(!(*pfkey_ext = (struct sadb_ext*)
 	     pfkey_sens = (struct sadb_sens*)
-	     MALLOC(sizeof(struct sadb_sens)))) {
+	     MALLOC(sizeof(struct sadb_sens) +
+		    (sens_len + integ_len) * sizeof(uint64_t)))) {
 		DEBUGGING(
 			"pfkey_sens_build: memory allocation failed\n");
 		SENDERR(ENOMEM);
 	}
 	memset(pfkey_sens,
 	       0,
-	       sizeof(struct sadb_sens));
+	       sizeof(struct sadb_sens) +
+	       (sens_len + integ_len) * sizeof(uint64_t));
 	
-	pfkey_sens->sadb_sens_len = sizeof(struct sadb_sens) / IPSEC_PFKEYv2_ALIGN;
+	pfkey_sens->sadb_sens_len = (sizeof(struct sadb_sens) +
+		    (sens_len + integ_len) * sizeof(uint64_t)) / IPSEC_PFKEYv2_ALIGN;
 	pfkey_sens->sadb_sens_exttype = SADB_EXT_SENSITIVITY;
 	pfkey_sens->sadb_sens_dpd = dpd;
 	pfkey_sens->sadb_sens_sens_level = sens_level;
@@ -639,17 +670,30 @@ pfkey_sens_build(struct sadb_ext**	pfkey_ext,
 	pfkey_sens->sadb_sens_integ_len = integ_len;
 	pfkey_sens->sadb_sens_reserved = 0;
 
+	bitmap = (uint64_t*)((char*)pfkey_ext + sizeof(struct sadb_sens));
+	for(i = 0; i < sens_len; i++) {
+		*bitmap = sens_bitmap[i];
+		bitmap++;
+	}
+	for(i = 0; i < integ_len; i++) {
+		*bitmap = integ_bitmap[i];
+		bitmap++;
+	}
+
 errlab:
 	return error;
 }
 
 int
 pfkey_prop_build(struct sadb_ext**	pfkey_ext,
-		 uint8_t		replay)
+		 uint8_t		replay,
+		 unsigned int		comb_num,
+		 struct sadb_comb*	comb)
 {
 	int error = 0;
 	int i;
 	struct sadb_prop *pfkey_prop = (struct sadb_prop *)*pfkey_ext;
+	struct sadb_comb *combp;
 
 	/* sanity checks... */
 	if(pfkey_prop) {
@@ -665,16 +709,19 @@ pfkey_prop_build(struct sadb_ext**	pfkey_ext,
 
 	if(!(*pfkey_ext = (struct sadb_ext*)
 	     pfkey_prop = (struct sadb_prop*)
-	     MALLOC(sizeof(struct sadb_prop)))) {
+	     MALLOC(sizeof(struct sadb_prop) +
+		    comb_num * sizeof(struct sadb_comb)))) {
 		DEBUGGING(
 			"pfkey_prop_build: memory allocation failed\n");
 		SENDERR(ENOMEM);
 	}
 	memset(pfkey_prop,
 	       0,
-	       sizeof(struct sadb_prop));
+	       sizeof(struct sadb_prop) +
+		    comb_num * sizeof(struct sadb_comb));
 	
-	pfkey_prop->sadb_prop_len = sizeof(struct sadb_prop) / IPSEC_PFKEYv2_ALIGN;
+	pfkey_prop->sadb_prop_len = (sizeof(struct sadb_prop) +
+		    comb_num * sizeof(struct sadb_comb)) / IPSEC_PFKEYv2_ALIGN;
 
 	pfkey_prop->sadb_prop_exttype = SADB_EXT_SUPPORTED_AUTH;
 	pfkey_prop->sadb_prop_replay = replay;
@@ -683,6 +730,12 @@ pfkey_prop_build(struct sadb_ext**	pfkey_ext,
 		pfkey_prop->sadb_prop_reserved[i] = 0;
 	}
 
+	combp = (struct sadb_comb*)((char*)pfkey_ext + sizeof(struct sadb_prop));
+	for(i = 0; i < comb_num; i++) {
+		*combp = comb[i];
+		combp++;
+	}
+
 #if 0
   uint8_t sadb_comb_auth;
   uint8_t sadb_comb_encrypt;
@@ -707,7 +760,9 @@ errlab:
 
 int
 pfkey_supported_build(struct sadb_ext**	pfkey_ext,
-		      uint16_t		exttype)
+		      uint16_t		exttype,
+		      unsigned int	alg_num,
+		      struct sadb_alg*	alg)
 {
 	int error = 0;
 	unsigned int i;
@@ -721,11 +776,6 @@ pfkey_supported_build(struct sadb_ext**	pfkey_ext,
 		SENDERR(EINVAL);
 	}
 
-	DEBUGGING(
-		"pfkey_supported_build: Sorry, I can't build exttype=%d yet.\n",
-		(*pfkey_ext)->sadb_ext_type);
-	SENDERR(EINVAL); /* don't process these yet */
-
 	if( !((exttype == SADB_EXT_SUPPORTED_AUTH) || (exttype == SADB_EXT_SUPPORTED_ENCRYPT))) {
 		DEBUGGING(
 			"pfkey_supported_build: unsupported extension type=%d.\n",
@@ -733,39 +783,42 @@ pfkey_supported_build(struct sadb_ext**	pfkey_ext,
 		SENDERR(EINVAL);
 	}
 
-	pfkey_alg = (struct sadb_alg*)((char*)pfkey_ext + sizeof(struct sadb_supported));
-	for(i = 0;
-	    i < pfkey_supported->sadb_supported_len -
-		    (sizeof(struct sadb_supported) / IPSEC_PFKEYv2_ALIGN);
-	    i++) {
-		/* process algo description */
-		if(!pfkey_alg->sadb_alg_reserved) {
-			DEBUGGING(
-				"pfkey_supported_build: alg#%d, alg_id=%d, res=%d, must be zero.\n",
-				i,
-				pfkey_alg->sadb_alg_id,
-				pfkey_alg->sadb_alg_reserved);
-			SENDERR(EINVAL);
-		}
-		pfkey_alg += sizeof(struct sadb_alg);
-	}
-	
 	if(!(*pfkey_ext = (struct sadb_ext*)
 	     pfkey_supported = (struct sadb_supported*)
-	     MALLOC(sizeof(struct sadb_supported)))) {
+	     MALLOC(sizeof(struct sadb_supported) +
+					       alg_num *
+					       sizeof(struct sadb_alg)))) {
 		DEBUGGING(
 			"pfkey_supported_build: memory allocation failed\n");
 		SENDERR(ENOMEM);
 	}
 	memset(pfkey_supported,
 	       0,
-	       sizeof(struct sadb_supported));
+	       sizeof(struct sadb_supported) +
+					       alg_num *
+					       sizeof(struct sadb_alg));
 	
-	pfkey_supported->sadb_supported_len = sizeof(struct sadb_supported) / IPSEC_PFKEYv2_ALIGN;
+	pfkey_supported->sadb_supported_len = (sizeof(struct sadb_supported) +
+					       alg_num *
+					       sizeof(struct sadb_alg)) /
+						IPSEC_PFKEYv2_ALIGN;
 	pfkey_supported->sadb_supported_exttype = exttype;
 	pfkey_supported->sadb_supported_reserved = 0;
 
+	pfkey_alg = (struct sadb_alg*)((char*)pfkey_supported + sizeof(struct sadb_supported));
+	for(i = 0; i < alg_num; i++) {
+		*pfkey_alg = alg[i];
+		/* process algo description */
+		pfkey_alg->sadb_alg_reserved = 0;
+		pfkey_alg++;
+	}
+	
 #if 0
+	DEBUGGING(
+		"pfkey_supported_build: Sorry, I can't build exttype=%d yet.\n",
+		(*pfkey_ext)->sadb_ext_type);
+	SENDERR(EINVAL); /* don't process these yet */
+
   uint8_t sadb_alg_id;
   uint8_t sadb_alg_ivlen;
   uint16_t sadb_alg_minbits;
@@ -870,7 +923,6 @@ errlab:
 
 int
 pfkey_x_satype_build(struct sadb_ext**	pfkey_ext,
-		     uint16_t		exttype,
 		     uint8_t		satype)
 {
 	int error = 0;
@@ -921,6 +973,70 @@ errlab:
 	return error;
 }
 
+int
+pfkey_x_debug_build(struct sadb_ext**	pfkey_ext,
+		    uint32_t            tunnel,
+		    uint32_t		netlink,
+		    uint32_t		xform,
+		    uint32_t		eroute,
+		    uint32_t		spi,
+		    uint32_t		radij,
+		    uint32_t		esp,
+		    uint32_t		ah,
+		    uint32_t		rcv,
+		    uint32_t            pfkey)
+{
+	int error = 0;
+	int i;
+	struct sadb_x_debug *pfkey_x_debug = (struct sadb_x_debug *)*pfkey_ext;
+
+	DEBUGGING(
+		"pfkey_x_debug_build:\n");
+	/* sanity checks... */
+	if(pfkey_x_debug) {
+		DEBUGGING(
+			"pfkey_x_debug_build:why is pfkey_x_debug already pointing to something?\n");
+		SENDERR(EINVAL);
+	}
+	
+	DEBUGGING(
+		"pfkey_x_debug_build:tunnel=%x netlink=%x xform=%x eroute=%x spi=%x radij=%x esp=%x ah=%x rcv=%x pfkey=%x?\n",
+		tunnel, netlink, xform, eroute, spi, radij, esp, ah, rcv, pfkey);
+
+	if(!(*pfkey_ext = (struct sadb_ext*)pfkey_x_debug = (struct sadb_x_debug*)
+	     MALLOC(sizeof(struct sadb_x_debug)))) {
+		DEBUGGING(
+			"pfkey_x_debug_build: memory allocation failed\n");
+		SENDERR(ENOMEM);
+	}
+#if 0
+	memset(pfkey_x_debug,
+	       0,
+	       sizeof(struct sadb_x_debug));
+#endif
+	
+        pfkey_x_debug->sadb_x_debug_len = sizeof(struct sadb_x_debug) / IPSEC_PFKEYv2_ALIGN;
+	pfkey_x_debug->sadb_x_debug_exttype = SADB_X_EXT_DEBUG;
+
+	pfkey_x_debug->sadb_x_debug_tunnel = tunnel;
+	pfkey_x_debug->sadb_x_debug_netlink = netlink;
+	pfkey_x_debug->sadb_x_debug_xform = xform;
+	pfkey_x_debug->sadb_x_debug_eroute = eroute;
+	pfkey_x_debug->sadb_x_debug_spi = spi;
+	pfkey_x_debug->sadb_x_debug_radij = radij;
+	pfkey_x_debug->sadb_x_debug_esp = esp;
+	pfkey_x_debug->sadb_x_debug_ah = ah;
+	pfkey_x_debug->sadb_x_debug_rcv = rcv;
+	pfkey_x_debug->sadb_x_debug_pfkey = pfkey;
+
+	for(i=0; i<4; i++) {
+		pfkey_x_debug->sadb_x_debug_reserved[i] = 0;
+	}
+
+errlab:
+	return error;
+}
+
 #if I_DONT_THINK_THIS_WILL_BE_USEFUL
 int (*ext_default_builders[SADB_EXT_MAX +1])(struct sadb_msg*, struct sadb_ext*)
  =
@@ -945,17 +1061,21 @@ int (*ext_default_builders[SADB_EXT_MAX +1])(struct sadb_msg*, struct sadb_ext*)
 	pfkey_x_kmprivate_build,
 	pfkey_x_satype_build,
 	pfkey_sa_build,
-	pfkey_address_build
+	pfkey_address_build,
+	pfkey_address_build,
+	pfkey_address_build,
+	pfkey_address_build,
+	pfkey_address_build,
+	pfkey_x_ext_debug_build
 };
 #endif
 
 int
-pfkey_msg_build(struct sadb_ext *extensions[], int dir)
+pfkey_msg_build(struct sadb_msg **pfkey_msg, struct sadb_ext *extensions[], int dir)
 {
 	int error = 0;
 	int ext;
 	int total_size;
-	struct sadb_msg *pfkey_msg;
 	struct sadb_ext *pfkey_ext;
 	int extensions_seen = 0;
 	struct sadb_ext *extensions_check[SADB_EXT_MAX + 1];
@@ -967,13 +1087,13 @@ pfkey_msg_build(struct sadb_ext *extensions[], int dir)
 	}
 
 	total_size = sizeof(struct sadb_msg) / IPSEC_PFKEYv2_ALIGN;
-	for(ext = 1; ext < SADB_EXT_MAX; ext++) {
+	for(ext = 1; ext <= SADB_EXT_MAX; ext++) {
 		if(extensions[ext]) {
 			total_size += (extensions[ext])->sadb_ext_len;
 		}
         }                
 
-	if(!(pfkey_msg = (struct sadb_msg*)MALLOC(total_size * IPSEC_PFKEYv2_ALIGN))) {
+	if(!(*pfkey_msg = (struct sadb_msg*)MALLOC(total_size * IPSEC_PFKEYv2_ALIGN))) {
 		DEBUGGING(
 			"pfkey_msg_build: memory allocation failed\n");
 		SENDERR(ENOMEM);
@@ -981,31 +1101,29 @@ pfkey_msg_build(struct sadb_ext *extensions[], int dir)
 
 	DEBUGGING(
 		"pfkey_msg_build: pfkey_msg=%p allocated %d bytes, &(extensions[0])=%p\n",
-		pfkey_msg,
+		*pfkey_msg,
 		total_size * IPSEC_PFKEYv2_ALIGN,
 		&(extensions[0]));
-	memcpy(pfkey_msg,
+	memcpy(*pfkey_msg,
 	       extensions[0],
 	       sizeof(struct sadb_msg));
-	FREE(extensions[0]);
-	extensions[0] = (struct sadb_ext*)pfkey_msg;
-	pfkey_msg->sadb_msg_len = total_size;
-	pfkey_msg->sadb_msg_reserved = 0;
+	(*pfkey_msg)->sadb_msg_len = total_size;
+	(*pfkey_msg)->sadb_msg_reserved = 0;
 	extensions_seen =  1 ;
 
-	pfkey_ext = (struct sadb_ext*)(((char*)pfkey_msg) + sizeof(struct sadb_msg));
+	pfkey_ext = (struct sadb_ext*)(((char*)(*pfkey_msg)) + sizeof(struct sadb_msg));
 
-	for(ext = 1; ext < SADB_EXT_MAX; ext++) {
+	for(ext = 1; ext <= SADB_EXT_MAX; ext++) {
 		/* copy from extension[ext] to buffer */
 		if(extensions[ext]) {    
 			/* Is this type of extension permitted for this type of message? */
-			if(!(extensions_bitmaps[dir][EXT_BITS_PERM][pfkey_msg->sadb_msg_type] &
+			if(!(extensions_bitmaps[dir][EXT_BITS_PERM][(*pfkey_msg)->sadb_msg_type] &
 			     1<<ext)) {
 				DEBUGGING(
 					"pfkey_msg_build: "
 					"ext type %d not permitted, exts_perm=%08x, 1<<type=%08x\n", 
 					ext, 
-					extensions_bitmaps[dir][EXT_BITS_PERM][pfkey_msg->sadb_msg_type],
+					extensions_bitmaps[dir][EXT_BITS_PERM][(*pfkey_msg)->sadb_msg_type],
 					1<<ext);
 				SENDERR(EINVAL);
 			}
@@ -1018,8 +1136,6 @@ pfkey_msg_build(struct sadb_ext *extensions[], int dir)
 			memcpy(pfkey_ext,
 			       extensions[ext],
 			       (extensions[ext])->sadb_ext_len * IPSEC_PFKEYv2_ALIGN);
-			FREE(extensions[ext]);
-			extensions[ext] = pfkey_ext;
 			((char*)pfkey_ext) += (extensions[ext])->sadb_ext_len * IPSEC_PFKEYv2_ALIGN;
 			/* Mark that we have seen this extension and remember the header location */
 			extensions_seen |= ( 1 << ext );
@@ -1030,22 +1146,22 @@ pfkey_msg_build(struct sadb_ext *extensions[], int dir)
 	DEBUGGING(
 		"pfkey_msg_build: extensions "
 		"permitted=%08x, seen=%08x, required=%08x.\n",
-		extensions_bitmaps[dir][EXT_BITS_PERM][pfkey_msg->sadb_msg_type],
+		extensions_bitmaps[dir][EXT_BITS_PERM][(*pfkey_msg)->sadb_msg_type],
 		extensions_seen,
-		extensions_bitmaps[dir][EXT_BITS_REQ][pfkey_msg->sadb_msg_type]);
+		extensions_bitmaps[dir][EXT_BITS_REQ][(*pfkey_msg)->sadb_msg_type]);
 	
 	if((extensions_seen &
-	    extensions_bitmaps[dir][EXT_BITS_REQ][pfkey_msg->sadb_msg_type]) !=
-	   extensions_bitmaps[dir][EXT_BITS_REQ][pfkey_msg->sadb_msg_type]) {
+	    extensions_bitmaps[dir][EXT_BITS_REQ][(*pfkey_msg)->sadb_msg_type]) !=
+	   extensions_bitmaps[dir][EXT_BITS_REQ][(*pfkey_msg)->sadb_msg_type]) {
 		DEBUGGING(
 			"pfkey_msg_build: required extensions missing:%08x.\n",
-			extensions_bitmaps[dir][EXT_BITS_REQ][pfkey_msg->sadb_msg_type] -
+			extensions_bitmaps[dir][EXT_BITS_REQ][(*pfkey_msg)->sadb_msg_type] -
 			(extensions_seen &
-			 extensions_bitmaps[dir][EXT_BITS_REQ][pfkey_msg->sadb_msg_type]) );
+			 extensions_bitmaps[dir][EXT_BITS_REQ][(*pfkey_msg)->sadb_msg_type]) );
 		SENDERR(EINVAL);
 	}
 	
-	if((error = pfkey_msg_parse(pfkey_msg, NULL, extensions_check, dir))) {
+	if((error = pfkey_msg_parse(*pfkey_msg, NULL, extensions_check, dir))) {
 		DEBUGGING(
 			"pfkey_msg_build: Trouble parsing newly built pfkey message, error=%d.\n",
 			error);
@@ -1059,6 +1175,32 @@ errlab:
 
 /*
  * $Log: pfkey_v2_build.c,v $
+ * Revision 1.13  2000/06/02 22:54:14  rgb
+ * Added Gerhard Gessler's struct sockaddr_storage mods for IPv6 support.
+ *
+ * Revision 1.12  2000/05/10 19:24:01  rgb
+ * Fleshed out sensitivity, proposal and supported extensions.
+ *
+ * Revision 1.11  2000/03/16 14:07:23  rgb
+ * Renamed ALIGN macro to avoid fighting with others in kernel.
+ *
+ * Revision 1.10  2000/01/24 21:14:35  rgb
+ * Added disabled pluto pfkey lib debug flag.
+ *
+ * Revision 1.9  2000/01/21 06:27:32  rgb
+ * Added address cases for eroute flows.
+ * Removed unused code.
+ * Dropped unused argument to pfkey_x_satype_build().
+ * Indented compiler directives for readability.
+ * Added klipsdebug switching capability.
+ * Fixed SADB_EXT_MAX bug not permitting last extension access.
+ *
+ * Revision 1.8  1999/12/29 21:17:41  rgb
+ * Changed pfkey_msg_build() I/F to include a struct sadb_msg**
+ * parameter for cleaner manipulation of extensions[] and to guard
+ * against potential memory leaks.
+ * Changed the I/F to pfkey_msg_free() for the same reason.
+ *
  * Revision 1.7  1999/12/09 23:12:20  rgb
  * Removed unused cruft.
  * Added argument to pfkey_sa_build() to do eroutes.
