@@ -12,14 +12,14 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: pfkey_v2_parse.c,v 1.10 1999/12/10 17:45:24 rgb Exp $
+ * RCSID $Id: pfkey_v2_parse.c,v 1.17 2000/06/02 22:54:14 rgb Exp $
  */
 
 /*
  *		Template from klips/net/ipsec/ipsec/ipsec_parser.c.
  */
 
-char pfkey_v2_parse_c_version[] = "$Id: pfkey_v2_parse.c,v 1.10 1999/12/10 17:45:24 rgb Exp $";
+char pfkey_v2_parse_c_version[] = "$Id: pfkey_v2_parse.c,v 1.17 2000/06/02 22:54:14 rgb Exp $";
 
 /*
  * Some ugly stuff to allow consistent debugging code for use in the
@@ -28,42 +28,44 @@ char pfkey_v2_parse_c_version[] = "$Id: pfkey_v2_parse.c,v 1.10 1999/12/10 17:45
 
 #ifdef __KERNEL__
 
-#include <linux/kernel.h>  /* for printk */
+# include <linux/kernel.h>  /* for printk */
 
-#include <linux/malloc.h> /* kmalloc() */
-#include <linux/errno.h>  /* error codes */
-#include <linux/types.h>  /* size_t */
-#include <linux/interrupt.h> /* mark_bh */
+# include <linux/malloc.h> /* kmalloc() */
+# include <linux/errno.h>  /* error codes */
+# include <linux/types.h>  /* size_t */
+# include <linux/interrupt.h> /* mark_bh */
 
-#include <linux/netdevice.h>   /* struct device, and other headers */
-#include <linux/etherdevice.h> /* eth_type_trans */
-#include <linux/ip.h>          /* struct iphdr */ 
-#if defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE)
-#include <linux/ipv6.h>        /* struct ipv6hdr */
-#endif /* if defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE) */
+# include <linux/netdevice.h>   /* struct device, and other headers */
+# include <linux/etherdevice.h> /* eth_type_trans */
+# include <linux/ip.h>          /* struct iphdr */ 
+# if defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE)
+#  include <linux/ipv6.h>        /* struct ipv6hdr */
+# endif /* if defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE) */
 extern int debug_pfkey;
 
-#define DEBUGGING(format,args...) \
+# define DEBUGGING(format,args...) \
          ((debug_pfkey) ? printk(KERN_INFO "klips_debug:" format , ## args) : 0)
 
 
 #else /* __KERNEL__ */
 
-#include <sys/types.h>
-#include <linux/types.h>
-#include <linux/errno.h>
+# include <sys/types.h>
+# include <linux/types.h>
+# include <linux/errno.h>
 
-#include "../pluto/constants.h" 
-#include "../pluto/defs.h"  /* for PRINTF_LIKE */
-#include "../pluto/log.h"  /* for debugging and DBG_log */
+# include "../pluto/constants.h" 
+# include "../pluto/defs.h"  /* for PRINTF_LIKE */
+# include "../pluto/log.h"  /* for debugging and DBG_log */
 
 extern unsigned int pfkey_lib_debug;  /* bits selecting what to report */
 
-#ifdef PLUTO
-#define DEBUGGING(format,args...)  { DBG_log("pfkey_lib_debug:" format, ## args);  }
-#else
-#define DEBUGGING(format,args...)  if(pfkey_lib_debug) { printf("pfkey_lib_debug:" format, ## args); } else { ; }
-#endif
+/* #define PLUTO */
+
+# ifdef PLUTO
+#  define DEBUGGING(format,args...)  { DBG_log("pfkey_lib_debug:" format, ## args);  }
+# else
+#  define DEBUGGING(format,args...)  if(pfkey_lib_debug) { printf("pfkey_lib_debug:" format, ## args); } else { ; }
+# endif
 
 #endif /* __KERNEL__ */
 
@@ -74,6 +76,56 @@ extern unsigned int pfkey_lib_debug;  /* bits selecting what to report */
 
 #define SENDERR(_x) do { error = -(_x); goto errlab; } while (0)
 
+struct satype_tbl {
+	uint8_t proto;
+	uint8_t satype;
+	char* name;
+} static satype_tbl[] = {
+#ifdef __KERNEL__
+	{ IPPROTO_ESP,	SADB_SATYPE_ESP,	"ESP"  },
+	{ IPPROTO_AH,	SADB_SATYPE_AH,		"AH"   },
+	{ IPPROTO_IPIP,	SADB_X_SATYPE_IPIP,	"IPIP" },
+#ifdef CONFIG_IPCOMP
+	{ IPPROTO_COMP,	SADB_X_SATYPE_COMP,	"COMP" },
+#endif /* CONFIG_IPCOMP */
+#else /* __KERNEL__ */
+	{ SA_ESP,	SADB_SATYPE_ESP,	"ESP"  },
+	{ SA_AH,	SADB_SATYPE_AH,		"AH"   },
+	{ SA_IPIP,	SADB_X_SATYPE_IPIP,	"IPIP" },
+#ifdef CONFIG_IPCOMP
+	{ SA_COMP,	SADB_X_SATYPE_COMP,	"COMP" },
+#endif /* CONFIG_IPCOMP */
+#endif /* __KERNEL__ */
+	{ 0, 0 }
+};
+
+uint8_t satype2proto(uint8_t satype) {
+	int i =0;
+
+	while(satype_tbl[i].satype != satype && satype_tbl[i].satype != 0) {
+		i++;
+	}
+	return satype_tbl[i].proto;
+}
+
+uint8_t proto2satype(uint8_t proto) {
+	int i = 0;
+
+	while(satype_tbl[i].proto != proto && satype_tbl[i].proto != 0) {
+		i++;
+	}
+	return satype_tbl[i].satype;
+}
+
+char* proto2name(uint8_t proto) {
+	int i = 0;
+
+	while(satype_tbl[i].proto != proto && satype_tbl[i].proto != 0) {
+		i++;
+	}
+	return satype_tbl[i].name;
+}
+
 uint8_t sadb_satype2proto[] = {
 #ifdef __KERNEL__
 	0,
@@ -167,7 +219,6 @@ pfkey_sa_parse(struct sadb_ext *pfkey_ext)
 		SENDERR(EINVAL);
 	}
 	
-	
 	if(! ((pfkey_sa->sadb_sa_exttype ==  SADB_EXT_SA) ||
 	      (pfkey_sa->sadb_sa_exttype ==  SADB_X_EXT_SA2)))
 	{
@@ -179,12 +230,14 @@ pfkey_sa_parse(struct sadb_ext *pfkey_ext)
 		SENDERR(EINVAL);
 	}
 	DEBUGGING(
-		"pfkey_sa_parse: successfully found len=%d exttype=%d spi=%08lx replay=%d state=%d flags=%d.\n",
+		"pfkey_sa_parse: successfully found len=%d exttype=%d spi=%08lx replay=%d state=%d auth=%d encrypt=%d flags=%d.\n",
 		pfkey_sa->sadb_sa_len,
 		pfkey_sa->sadb_sa_exttype,
-		ntohl(pfkey_sa->sadb_sa_spi),
+		(long unsigned int)ntohl(pfkey_sa->sadb_sa_spi),
 		pfkey_sa->sadb_sa_replay,
 		pfkey_sa->sadb_sa_state,
+		pfkey_sa->sadb_sa_auth,
+		pfkey_sa->sadb_sa_encrypt,
 		pfkey_sa->sadb_sa_flags);
 
  errlab:
@@ -264,22 +317,52 @@ pfkey_address_parse(struct sadb_ext *pfkey_ext)
 		SENDERR(EINVAL);
 	}
 	
+	switch(pfkey_address->sadb_address_exttype) {	
+	case SADB_EXT_ADDRESS_SRC:
+	case SADB_EXT_ADDRESS_DST:
+	case SADB_EXT_ADDRESS_PROXY:
+	case SADB_X_EXT_ADDRESS_DST2:
+	case SADB_X_EXT_ADDRESS_SRC_FLOW:
+	case SADB_X_EXT_ADDRESS_DST_FLOW:
+	case SADB_X_EXT_ADDRESS_SRC_MASK:
+	case SADB_X_EXT_ADDRESS_DST_MASK:
+		break;
+	default:
+		DEBUGGING( 
+			"pfkey_address_parse: unexpected ext_type=%d.\n", 
+			pfkey_address->sadb_address_exttype); 
+		SENDERR(EINVAL); 
+	}
+	
 	switch(s->sa_family) {
 	case AF_INET:
 		DEBUGGING(
 			"pfkey_address_parse: found address family=%d, AF_INET.\n",
 			s->sa_family);
 		saddr_len = sizeof(struct sockaddr_in);
+#ifdef WITH_SOCKADDR_STORAGE
+		addrtoa(*((struct sockaddr_storage*)s), 0, ipaddr_txt, sizeof(ipaddr_txt));
+#else
 		addrtoa(((struct sockaddr_in*)s)->sin_addr, 0, ipaddr_txt, sizeof(ipaddr_txt));
+#endif /* WITH_SOCKADDR_STORAGE */
 		DEBUGGING(
 			"pfkey_address_parse: found address=%s.\n",
 			ipaddr_txt);
 		break;
-#if defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE)
+#ifdef WITH_SOCKADDR_STORAGE
+#if defined(WITH_IPV6) || defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE)
 	case AF_INET6:
+		DEBUGGING(
+			"pfkey_address_parse: found address family=%d, AF_INET.\n",
+			s->sa_family);
 		saddr_len = sizeof(struct sockaddr_in6);
+		addrtoa(*((struct sockaddr_storage*)s), 0, ipaddr_txt, sizeof(ipaddr_txt));
+		DEBUGGING(
+			"pfkey_address_parse: found address=%s.\n",
+			ipaddr_txt);
 		break;
-#endif /* defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE) */
+#endif /* defined(WITH_IPV6) || defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE) */
+#endif /* WITH_SOCKADDR_STORAGE */
 	default:
 		DEBUGGING(
 			"pfkey_address_parse: s->sa_family=%d not supported.\n",
@@ -303,18 +386,6 @@ pfkey_address_parse(struct sadb_ext *pfkey_ext)
 		SENDERR(EAFNOSUPPORT); /* not supported yet */
 	}
 	
-	
-	if( ! ( (pfkey_address->sadb_address_exttype == SADB_EXT_ADDRESS_SRC) ||
-		(pfkey_address->sadb_address_exttype == SADB_EXT_ADDRESS_DST) ||
-		(pfkey_address->sadb_address_exttype == SADB_EXT_ADDRESS_PROXY) ||
-		(pfkey_address->sadb_address_exttype == SADB_X_EXT_ADDRESS_DST2) ))
-	{
-		DEBUGGING( 
-			"pfkey_address_parse: unexpected ext_type=%d.\n", 
-			pfkey_address->sadb_address_exttype); 
-		SENDERR(EINVAL); 
-	}
-	
 	/* XXX check if port!=0 */
 	
 	DEBUGGING(
@@ -470,18 +541,29 @@ static int
 pfkey_prop_parse(struct sadb_ext *pfkey_ext)
 {
 	int error = 0;
-	int i;
+	int i, num_comb;
 	struct sadb_prop *pfkey_prop = (struct sadb_prop *)pfkey_ext;
+	struct sadb_comb *pfkey_comb = (struct sadb_comb *)((char*)pfkey_ext + sizeof(struct sadb_prop));
 
 	/* sanity checks... */
-	if(pfkey_prop->sadb_prop_len < sizeof(struct sadb_prop) / IPSEC_PFKEYv2_ALIGN) {
+	if((pfkey_prop->sadb_prop_len < sizeof(struct sadb_prop) / IPSEC_PFKEYv2_ALIGN) || 
+	   (((pfkey_prop->sadb_prop_len * IPSEC_PFKEYv2_ALIGN) - sizeof(struct sadb_prop)) % sizeof(struct sadb_comb))) {
 		DEBUGGING(
-			"pfkey_prop_parse: size wrong ext_len=%d, key_ext_len=%d.\n",
+			"pfkey_prop_parse: size wrong ext_len=%d, prop_ext_len=%d comb_ext_len=%d.\n",
 			pfkey_prop->sadb_prop_len,
-			sizeof(struct sadb_prop));
+			sizeof(struct sadb_prop),
+			sizeof(struct sadb_comb));
 		SENDERR(EINVAL);
 	}
 
+	if(pfkey_prop->sadb_prop_replay > 64) {
+		DEBUGGING(
+			"pfkey_prop_parse: replay window size: %d"
+			" -- must be 0 <= size <= 64\n",
+			pfkey_prop->sadb_prop_replay);
+		SENDERR(EINVAL);
+	}
+	
 	for(i=0; i<3; i++) {
 		if(pfkey_prop->sadb_prop_reserved[i]) {
 			DEBUGGING(
@@ -491,10 +573,150 @@ pfkey_prop_parse(struct sadb_ext *pfkey_ext)
 		}
 	}
 
-	DEBUGGING(
-		"pfkey_prop_parse: Sorry, I can't parse exttype=%d yet.\n",
-		pfkey_ext->sadb_ext_type);
-	SENDERR(EINVAL); /* don't process these yet */
+	num_comb = ((pfkey_prop->sadb_prop_len * IPSEC_PFKEYv2_ALIGN) - sizeof(struct sadb_prop)) / sizeof(struct sadb_comb);
+
+	for(i = 0; i < num_comb; i++) {
+		if(pfkey_comb->sadb_comb_auth > SADB_AALG_MAX) {
+			DEBUGGING(
+				"pfkey_prop_parse: pfkey_comb[%d]->sadb_comb_auth=%d > SADB_AALG_MAX=%d.\n",
+				i,
+				pfkey_comb->sadb_comb_auth,
+				SADB_AALG_MAX);
+			SENDERR(EINVAL);
+		}
+
+		if(pfkey_comb->sadb_comb_auth) {
+			if(!pfkey_comb->sadb_comb_auth_minbits) {
+				DEBUGGING(
+					"pfkey_prop_parse: pfkey_comb[%d]->sadb_comb_auth_minbits=0, fatal.\n",
+					i);
+				SENDERR(EINVAL);
+			}
+			if(!pfkey_comb->sadb_comb_auth_maxbits) {
+				DEBUGGING(
+					"pfkey_prop_parse: pfkey_comb[%d]->sadb_comb_auth_maxbits=0, fatal.\n",
+					i);
+				SENDERR(EINVAL);
+			}
+			if(pfkey_comb->sadb_comb_auth_minbits > pfkey_comb->sadb_comb_auth_maxbits) {
+				DEBUGGING(
+					"pfkey_prop_parse: pfkey_comb[%d]->sadb_comb_auth_minbits=%d > maxbits=%d, fatal.\n",
+					i,
+					pfkey_comb->sadb_comb_auth_minbits,
+					pfkey_comb->sadb_comb_auth_maxbits);
+				SENDERR(EINVAL);
+			}
+		} else {
+			if(pfkey_comb->sadb_comb_auth_minbits) {
+				DEBUGGING(
+					"pfkey_prop_parse: pfkey_comb[%d]->sadb_comb_auth_minbits=%d != 0, fatal.\n",
+					i,
+					pfkey_comb->sadb_comb_auth_minbits);
+				SENDERR(EINVAL);
+			}
+			if(pfkey_comb->sadb_comb_auth_maxbits) {
+				DEBUGGING(
+					"pfkey_prop_parse: pfkey_comb[%d]->sadb_comb_auth_maxbits=%d != 0, fatal.\n",
+					i,
+					pfkey_comb->sadb_comb_auth_maxbits);
+				SENDERR(EINVAL);
+			}
+		}
+
+		if(pfkey_comb->sadb_comb_encrypt > SADB_EALG_MAX) {
+			DEBUGGING(
+				"pfkey_comb_parse: pfkey_comb[%d]->sadb_comb_encrypt=%d > SADB_EALG_MAX=%d.\n",
+				i,
+				pfkey_comb->sadb_comb_encrypt,
+				SADB_EALG_MAX);
+			SENDERR(EINVAL);
+		}
+
+		if(pfkey_comb->sadb_comb_encrypt) {
+			if(!pfkey_comb->sadb_comb_encrypt_minbits) {
+				DEBUGGING(
+					"pfkey_prop_parse: pfkey_comb[%d]->sadb_comb_encrypt_minbits=0, fatal.\n",
+					i);
+				SENDERR(EINVAL);
+			}
+			if(!pfkey_comb->sadb_comb_encrypt_maxbits) {
+				DEBUGGING(
+					"pfkey_prop_parse: pfkey_comb[%d]->sadb_comb_encrypt_maxbits=0, fatal.\n",
+					i);
+				SENDERR(EINVAL);
+			}
+			if(pfkey_comb->sadb_comb_encrypt_minbits > pfkey_comb->sadb_comb_encrypt_maxbits) {
+				DEBUGGING(
+					"pfkey_prop_parse: pfkey_comb[%d]->sadb_comb_encrypt_minbits=%d > maxbits=%d, fatal.\n",
+					i,
+					pfkey_comb->sadb_comb_encrypt_minbits,
+					pfkey_comb->sadb_comb_encrypt_maxbits);
+				SENDERR(EINVAL);
+			}
+		} else {
+			if(pfkey_comb->sadb_comb_encrypt_minbits) {
+				DEBUGGING(
+					"pfkey_prop_parse: pfkey_comb[%d]->sadb_comb_encrypt_minbits=%d != 0, fatal.\n",
+					i,
+					pfkey_comb->sadb_comb_encrypt_minbits);
+				SENDERR(EINVAL);
+			}
+			if(pfkey_comb->sadb_comb_encrypt_maxbits) {
+				DEBUGGING(
+					"pfkey_prop_parse: pfkey_comb[%d]->sadb_comb_encrypt_maxbits=%d != 0, fatal.\n",
+					i,
+					pfkey_comb->sadb_comb_encrypt_maxbits);
+				SENDERR(EINVAL);
+			}
+		}
+
+		/* XXX do sanity check on flags */
+
+		if(pfkey_comb->sadb_comb_hard_allocations && pfkey_comb->sadb_comb_soft_allocations > pfkey_comb->sadb_comb_hard_allocations) {
+			DEBUGGING(
+				"pfkey_prop_parse: pfkey_comb[%d]->sadb_comb_soft_allocations=%d > hard_allocations=%d, fatal.\n",
+				i,
+				pfkey_comb->sadb_comb_soft_allocations,
+				pfkey_comb->sadb_comb_hard_allocations);
+			SENDERR(EINVAL);
+		}
+
+		if(pfkey_comb->sadb_comb_hard_bytes && pfkey_comb->sadb_comb_soft_bytes > pfkey_comb->sadb_comb_hard_bytes) {
+			DEBUGGING(
+				"pfkey_prop_parse: pfkey_comb[%d]->sadb_comb_soft_bytes=%Ld > hard_bytes=%Ld, fatal.\n",
+				i,
+				pfkey_comb->sadb_comb_soft_bytes,
+				pfkey_comb->sadb_comb_hard_bytes);
+			SENDERR(EINVAL);
+		}
+
+		if(pfkey_comb->sadb_comb_hard_addtime && pfkey_comb->sadb_comb_soft_addtime > pfkey_comb->sadb_comb_hard_addtime) {
+			DEBUGGING(
+				"pfkey_prop_parse: pfkey_comb[%d]->sadb_comb_soft_addtime=%Ld > hard_addtime=%Ld, fatal.\n",
+				i,
+				pfkey_comb->sadb_comb_soft_addtime,
+				pfkey_comb->sadb_comb_hard_addtime);
+			SENDERR(EINVAL);
+		}
+
+		if(pfkey_comb->sadb_comb_hard_usetime && pfkey_comb->sadb_comb_soft_usetime > pfkey_comb->sadb_comb_hard_usetime) {
+			DEBUGGING(
+				"pfkey_prop_parse: pfkey_comb[%d]->sadb_comb_soft_usetime=%Ld > hard_usetime=%Ld, fatal.\n",
+				i,
+				pfkey_comb->sadb_comb_soft_usetime,
+				pfkey_comb->sadb_comb_hard_usetime);
+			SENDERR(EINVAL);
+		}
+
+		if(pfkey_comb->sadb_comb_reserved) {
+			DEBUGGING(
+				"pfkey_prop_parse: comb[%d].res=%d, must be zero.\n",
+				i,
+				pfkey_comb->sadb_comb_reserved);
+			SENDERR(EINVAL);
+		}
+		pfkey_comb++;
+	}
 
 errlab:
 	return error;
@@ -504,17 +726,21 @@ static int
 pfkey_supported_parse(struct sadb_ext *pfkey_ext)
 {
 	int error = 0;
-	unsigned int i;
+	unsigned int i, num_alg;
 	struct sadb_supported *pfkey_supported = (struct sadb_supported *)pfkey_ext;
-	struct sadb_alg *pfkey_alg;
+	struct sadb_alg *pfkey_alg = (struct sadb_alg*)((char*)pfkey_ext + sizeof(struct sadb_supported));
 
 	/* sanity checks... */
-	if(pfkey_supported->sadb_supported_len <
-	   sizeof(struct sadb_supported) / IPSEC_PFKEYv2_ALIGN) {
+	if((pfkey_supported->sadb_supported_len <
+	   sizeof(struct sadb_supported) / IPSEC_PFKEYv2_ALIGN) ||
+	   (((pfkey_supported->sadb_supported_len * IPSEC_PFKEYv2_ALIGN) -
+	     sizeof(struct sadb_supported)) % sizeof(struct sadb_alg))) {
+
 		DEBUGGING(
-			"pfkey_supported_parse: size wrong ext_len=%d, key_ext_len=%d.\n",
+			"pfkey_supported_parse: size wrong ext_len=%d, supported_ext_len=%d alg_ext_len=%d.\n",
 			pfkey_supported->sadb_supported_len,
-			sizeof(struct sadb_supported));
+			sizeof(struct sadb_supported),
+			sizeof(struct sadb_alg));
 		SENDERR(EINVAL);
 	}
 
@@ -525,28 +751,55 @@ pfkey_supported_parse(struct sadb_ext *pfkey_ext)
 		SENDERR(EINVAL);
 	}
 
-	pfkey_alg = (struct sadb_alg*)((char*)pfkey_ext + sizeof(struct sadb_supported));
-	for(i = 0;
-	    i < pfkey_supported->sadb_supported_len -
-		    (sizeof(struct sadb_supported) / IPSEC_PFKEYv2_ALIGN);
-	    i++) {
+	num_alg = ((pfkey_supported->sadb_supported_len * IPSEC_PFKEYv2_ALIGN) - sizeof(struct sadb_supported)) / sizeof(struct sadb_alg);
+
+	for(i = 0; i < num_alg; i++) {
 		/* process algo description */
 		if(!pfkey_alg->sadb_alg_reserved) {
 			DEBUGGING(
-				"pfkey_supported_parse: alg#%d, alg_id=%d, res=%d, must be zero.\n",
+				"pfkey_supported_parse: alg[%d], alg_id=%d, res=%d, must be zero.\n",
 				i,
 				pfkey_alg->sadb_alg_id,
 				pfkey_alg->sadb_alg_reserved);
 			SENDERR(EINVAL);
 		}
-		pfkey_alg += sizeof(struct sadb_alg);
+
+		/* XXX can alg_id auth/enc be determined from info given?
+		   Yes, but OpenBSD's method does not iteroperate with rfc2367.
+		   rgb, 2000-04-06 */
+
+		switch(pfkey_supported->sadb_supported_exttype) {
+		case SADB_EXT_SUPPORTED_AUTH:
+			if(pfkey_alg->sadb_alg_id > SADB_AALG_MAX) {
+				DEBUGGING(
+					"pfkey_supported_parse: alg[%d], alg_id=%d > SADB_AALG_MAX=%d, fatal.\n",
+					i,
+					pfkey_alg->sadb_alg_id,
+					SADB_AALG_MAX);
+				SENDERR(EINVAL);
+			}
+			break;
+		case SADB_EXT_SUPPORTED_ENCRYPT:
+			if(pfkey_alg->sadb_alg_id > SADB_EALG_MAX) {
+				DEBUGGING(
+					"pfkey_supported_parse: alg[%d], alg_id=%d > SADB_EALG_MAX=%d, fatal.\n",
+					i,
+					pfkey_alg->sadb_alg_id,
+					SADB_EALG_MAX);
+				SENDERR(EINVAL);
+			}
+			break;
+		default:
+			DEBUGGING(
+				"pfkey_supported_parse: alg[%d], alg_id=%d > SADB_EALG_MAX=%d, fatal.\n",
+				i,
+				pfkey_alg->sadb_alg_id,
+				SADB_EALG_MAX);
+			SENDERR(EINVAL);
+		}
+		pfkey_alg++;
 	}
 	
-	DEBUGGING(
-		"pfkey_supported_parse: Sorry, I can't parse exttype=%d yet.\n",
-		pfkey_ext->sadb_ext_type);
-	SENDERR(EINVAL); /* don't process these yet */
-	
  errlab:
 	return error;
 }
@@ -657,7 +910,7 @@ pfkey_x_satype_parse(struct sadb_ext *pfkey_ext)
 		SENDERR(EINVAL);
 	}
 
-	if(!(sadb_satype2proto[pfkey_x_satype->sadb_x_satype_satype])) {
+	if(!(satype2proto(pfkey_x_satype->sadb_x_satype_satype))) {
 		DEBUGGING(
 			"pfkey_x_satype_parse: proto lookup from satype=%d failed.\n",
 			pfkey_x_satype->sadb_x_satype_satype);
@@ -677,6 +930,38 @@ errlab:
 	return error;
 }
 
+static int
+pfkey_x_ext_debug_parse(struct sadb_ext *pfkey_ext)
+{
+	int error = 0;
+	int i;
+	struct sadb_x_debug *pfkey_x_debug = (struct sadb_x_debug *)pfkey_ext;
+
+	DEBUGGING(
+		"pfkey_x_debug_parse:\n");
+	/* sanity checks... */
+	if(pfkey_x_debug->sadb_x_debug_len !=
+	   sizeof(struct sadb_x_debug) / IPSEC_PFKEYv2_ALIGN) {
+		DEBUGGING(
+			"pfkey_x_debug_parse: size wrong ext_len=%d, key_ext_len=%d.\n",
+			pfkey_x_debug->sadb_x_debug_len,
+			sizeof(struct sadb_x_debug));
+		SENDERR(EINVAL);
+	}
+	
+	for(i = 0; i < 4; i++) {
+		if(pfkey_x_debug->sadb_x_debug_reserved[i]) {
+			DEBUGGING(
+				" pfkey_x_debug_parse: reserved[%d]=%d must be set to zero.\n",
+				i, pfkey_x_debug->sadb_x_debug_reserved[i]);
+			SENDERR(EINVAL);
+		}
+	}
+	
+errlab:
+	return error;
+}
+
 int (*ext_default_parsers[SADB_EXT_MAX +1])(struct sadb_ext*) =
 {
 	NULL, /* pfkey_msg_parse, */
@@ -699,7 +984,12 @@ int (*ext_default_parsers[SADB_EXT_MAX +1])(struct sadb_ext*) =
 	pfkey_x_kmprivate_parse,
 	pfkey_x_satype_parse,
 	pfkey_sa_parse,
-	pfkey_address_parse
+	pfkey_address_parse,
+	pfkey_address_parse,
+	pfkey_address_parse,
+	pfkey_address_parse,
+	pfkey_address_parse,
+	pfkey_x_ext_debug_parse
 };
 
 int
@@ -804,7 +1094,7 @@ pfkey_msg_parse(struct sadb_msg *pfkey_msg,
 		extensions_bitmaps[dir][EXT_BITS_PERM][pfkey_msg->sadb_msg_type],
 		extensions_bitmaps[dir][EXT_BITS_REQ][pfkey_msg->sadb_msg_type]);
 	
-	extensions_seen =  1 ;
+	extensions_seen = 1;
 	
 	while( (remain * IPSEC_PFKEYv2_ALIGN) >= sizeof(struct sadb_ext) ) {
 		/* Is there enough message left to support another extension header? */
@@ -947,7 +1237,7 @@ pfkey_msg_parse(struct sadb_msg *pfkey_msg,
 			     ((struct sadb_sa*)extensions[SADB_EXT_SA])->sadb_sa_encrypt !=
 			     SADB_EALG_NONE)) {
 				DEBUGGING(
-					"pfkey_msg_build: "
+					"pfkey_msg_parse: "
 					"encrypt alg=%d is zero, must be non-zero for ESP=%d SAs.\n",
 					((struct sadb_sa*)extensions[SADB_EXT_SA])->sadb_sa_encrypt,
 					((struct sadb_msg*)extensions[SADB_EXT_RESERVED])->sadb_msg_satype);
@@ -972,6 +1262,31 @@ errlab:
 
 /*
  * $Log: pfkey_v2_parse.c,v $
+ * Revision 1.17  2000/06/02 22:54:14  rgb
+ * Added Gerhard Gessler's struct sockaddr_storage mods for IPv6 support.
+ *
+ * Revision 1.16  2000/05/10 19:25:11  rgb
+ * Fleshed out proposal and supported extensions.
+ *
+ * Revision 1.15  2000/01/24 21:15:31  rgb
+ * Added disabled pluto pfkey lib debug flag.
+ * Added algo debugging reporting.
+ *
+ * Revision 1.14  2000/01/22 23:24:29  rgb
+ * Added new functions proto2satype() and satype2proto() and lookup
+ * table satype_tbl.  Also added proto2name() since it was easy.
+ *
+ * Revision 1.13  2000/01/21 09:43:59  rgb
+ * Cast ntohl(spi) as (unsigned long int) to shut up compiler.
+ *
+ * Revision 1.12  2000/01/21 06:28:19  rgb
+ * Added address cases for eroute flows.
+ * Indented compiler directives for readability.
+ * Added klipsdebug switching capability.
+ *
+ * Revision 1.11  1999/12/29 21:14:59  rgb
+ * Fixed debug text cut and paste typo.
+ *
  * Revision 1.10  1999/12/10 17:45:24  rgb
  * Added address debugging.
  *
