@@ -11,7 +11,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: connections.c,v 1.48 1999/12/14 20:31:01 dhr Exp $
+ * RCSID $Id: connections.c,v 1.66 2000/06/21 18:24:31 dhr Exp $
  */
 
 #include <string.h>
@@ -41,45 +41,151 @@
 
 static struct connection *connections = NULL;
 
+/* host_pair: a nexus of information about a pair of hosts.
+ * A host is an IP address, UDP port pair.  This is a debatable choice:
+ * - should port be considered (no choice of port in standard)?
+ * - should ID be considered (hard because not always known)?
+ * - should IP address matter on our end (we don't know our end)?
+ * Since connections are not all oriented, the host identifiers
+ * (struct in_addr) are placed in a canonical order.
+ */
+
+struct host_pair {
+    struct {
+	struct in_addr addr;	/* network order */
+	u_int16_t port;	/* host order */
+    } a, b;
+    bool initial_connection_sent;
+    struct connection *connections;	/* connections with this pair */
+    struct host_pair *next;
+};
+
+static struct host_pair *host_pairs = NULL;
+
+/* check to see that Ids of peers match */
+bool
+same_peer_ids(const struct connection *c, const struct connection *d
+, const struct id *his_id)
+{
+    return id_same(&c->this.id, &d->this.id)
+	&& id_same(his_id == NULL? &c->that.id : his_id, &d->that.id);
+}
+
+/* Arbitrary canonicalization of host pair.
+ * Because it is abitrary, we don't need to compare in host order.
+ */
+static void
+canonical_host_pair(struct host_pair *hp
+, struct in_addr xaddr, u_int16_t xport
+, struct in_addr yaddr, u_int16_t yport)
+{
+    if (xaddr.s_addr < yaddr.s_addr
+    || (xaddr.s_addr == yaddr.s_addr && xport <= yport))
+    {
+	hp->a.addr = xaddr;
+	hp->a.port = xport;
+	hp->b.addr = yaddr;
+	hp->b.port = yport;
+    }
+    else
+    {
+	hp->a.addr = yaddr;
+	hp->a.port = yport;
+	hp->b.addr = xaddr;
+	hp->b.port = xport;
+    }
+}
+
+static struct host_pair *
+find_host_pair(struct in_addr xaddr, u_int16_t xport
+, struct in_addr yaddr, u_int16_t yport)
+{
+    struct host_pair hp;
+    struct host_pair *p, *prev;
+
+    canonical_host_pair(&hp, xaddr, xport, yaddr, yport);
+
+    for (prev = NULL, p = host_pairs; p != NULL; prev = p, p = p->next)
+    {
+	if (same_ip(p->a.addr, hp.a.addr) && p->a.port == hp.a.port
+	&& same_ip(p->b.addr, hp.b.addr) && p->b.port == hp.b.port)
+	{
+	    if (prev != NULL)
+	    {
+		prev->next = p->next;	/* remove p from list */
+		p->next = host_pairs;	/* and stick it on front */
+		host_pairs = p;
+	    }
+	    break;
+	}
+    }
+    return p;
+}
+
+static struct connection *
+find_host_pair_connections(struct in_addr xaddr, u_int16_t xport
+, struct in_addr yaddr, u_int16_t yport)
+{
+    struct host_pair *hp = find_host_pair(xaddr, xport, yaddr, yport);
+
+    return hp == NULL? NULL : hp->connections;
+}
+
+static void
+connect_to_host_pair(struct connection *c)
+{
+    struct host_pair *hp = find_host_pair(c->this.host_addr, c->this.host_port
+	, c->that.host_addr, c->that.host_port);
+
+    if (hp == NULL)
+    {
+	hp = alloc_thing(struct host_pair, "host_pair");
+	canonical_host_pair(hp, c->this.host_addr, c->this.host_port
+	    , c->that.host_addr, c->that.host_port);
+	hp->initial_connection_sent = FALSE;
+	hp->connections = NULL;
+	hp->next = host_pairs;
+	host_pairs = hp;
+    }
+    c->host_pair = hp;
+    c->hp_next = hp->connections;
+    hp->connections = c;
+}
+
 /* find a connection by name.
  * move the winner (if any) to the front.
- * If none is found, and wft is not NULL_FD, a diagnostic is logged to whack.
+ * If none is found, and wfd is not NULL_FD, a diagnostic is logged to whack.
  */
 struct connection *
-con_by_name(const char *nm, int wfd)
+con_by_name(const char *nm, bool strict)
 {
     struct connection *p, *prev;
 
-    prev = NULL;
-    for (p = connections; ; p = p->next)
+    for (prev = NULL, p = connections; ; prev = p, p = p->next)
     {
 	if (p == NULL)
 	{
-	    if (wfd != NULL_FD)
-		whack_log(wfd, RC_UNKNOWN_NAME, "no connection with that name");
+	    if (strict)
+		whack_log(RC_UNKNOWN_NAME, "no connection with that name");
 	    break;
 	}
 	if (strcmp(p->name, nm) == 0)
 	{
 	    if (prev != NULL)
 	    {
-		struct connection *nxt = p->next;
-
-		p->next = connections;
+		prev->next = p->next;	/* remove p from list */
+		p->next = connections;	/* and stick it on front */
 		connections = p;
-		prev->next = nxt;
 	    }
 	    break;
 	}
-	prev = p;
     }
     return p;
 }
 
-static void
+void
 unorient_connection(struct connection *c)
 {
-#ifdef ROAD_WARRIOR_FUDGE
     if (c->rw_state == rwcs_instance)
     {
 	/* This does everything we need.
@@ -88,7 +194,7 @@ unorient_connection(struct connection *c)
 	 */
 	delete_connection(c);
     }
-#endif /* ROAD_WARRIOR_FUDGE */
+    else
     {
 	delete_states_by_connection(c);
 	unroute_connection(c);
@@ -98,13 +204,25 @@ unorient_connection(struct connection *c)
 
 /* Delete a connection */
 
+#define list_rm(etype, enext, e, ehead) { \
+	etype **ep; \
+	for (ep = &(ehead); *ep != (e); ep = &(*ep)->enext) \
+	    passert(*ep != NULL);    /* we must not come up empty-handed */ \
+	*ep = (e)->enext; \
+    }
+
+
 void
 delete_connection(struct connection *c)
 {
-    struct connection *old_cur_connection = cur_connection;
+    struct connection *old_cur_connection
+	= cur_connection == c? NULL : cur_connection;
+#ifdef DEBUG
+    unsigned int old_cur_debugging = cur_debugging;
+#endif
+
+    SET_CUR_CONNECTION(c);
 
-    cur_connection = c;
-#ifdef ROAD_WARRIOR_FUDGE
     /* Must be careful to avoid circularity:
      * we mark c as going away so it won't get deleted recursively.
      */
@@ -114,28 +232,46 @@ delete_connection(struct connection *c)
 	char that_host[ADDRTOA_BUF];
 
 	addrtoa(c->that.host_addr, 0, that_host, sizeof(that_host));
-	log("deleting connection with Road Warrior %s", that_host);
+	log("deleting connection \"%s\" with Road Warrior %s"
+	    , c->name, that_host);
 	c->rw_state = rwcs_going_away;
     }
     else
-#endif /* ROAD_WARRIOR_FUDGE */
     {
 	log("deleting connection");
     }
     unorient_connection(c);	/* won't delete c */
 
     /* find and delete c from connections list */
-    {
-	struct connection **p;
+    list_rm(struct connection, next, c, connections);
+    cur_connection = old_cur_connection;
 
-	for (p = &connections; *p != c; p = &(*p)->next)
-	    passert(*p != NULL);    /* we must not come up empty-handed */
-	*p = c->next;
+    /* find and delete c from the host pair list */
+    {
+	struct host_pair *hp = c->host_pair;
+
+	list_rm(struct connection, hp_next, c, hp->connections);
+	c->host_pair = NULL;	/* redundant, but safe */
+
+	/* if there are no more connections with this host_pair
+	 * and we haven't even made an initial contact, let's delete
+	 * this guy in case we were created by an attempted DOS attack.
+	 */
+	if (hp->connections == NULL && !hp->initial_connection_sent)
+	{
+	    list_rm(struct host_pair, next, hp, host_pairs);
+	    pfree(hp);
+	}
     }
-    cur_connection = old_cur_connection == c? NULL : old_cur_connection;
+
+#ifdef DEBUG
+    cur_debugging = old_cur_debugging;
+#endif
     pfreeany(c->name);
     free_id_content(&c->this.id);
+    pfreeany(c->this.updown);
     free_id_content(&c->that.id);
+    pfreeany(c->that.updown);
     pfree(c);
 }
 
@@ -163,8 +299,18 @@ release_interface(struct iface *i)
 static void
 default_end(struct end *e, struct in_addr dflt_nexthop)
 {
-    if (e->host_nexthop.s_addr == NO_IP)
+    /* default ID to IP (but only if not NO_IP -- WildCard) */
+    if (e->id.kind == ID_NONE && !is_NO_IP(e->host_addr))
+    {
+	e->id.kind = ID_IPV4_ADDR;
+	e->id.ip_addr = e->host_addr;
+    }
+
+    /* default nexthop to other side */
+    if (is_NO_IP(e->host_nexthop))
 	e->host_nexthop = dflt_nexthop;
+
+    /* default client to subnet containing only self */
     if (!e->has_client)
     {
 	e->client_net = e->host_addr;
@@ -172,8 +318,10 @@ default_end(struct end *e, struct in_addr dflt_nexthop)
     }
 }
 
-/* format the topology of an end, leaving out defaults */
-static size_t
+/* format the topology of an end, leaving out defaults
+ * Note: if that==NULL, skip nexthop
+ */
+size_t
 format_end(char *buf, size_t buf_len, struct end *this, struct end *that, bool is_left)
 {
     char client[SUBNETTOA_BUF];
@@ -200,10 +348,10 @@ format_end(char *buf, size_t buf_len, struct end *this, struct end *that, bool i
 	snprintf(host_port, sizeof(host_port), ":%u"
 	    , this->host_port);
 
-    /* id */
+    /* id, if different from host */
     host_id[0] = '\0';
-    if (this->id.kind != ID_IPV4_ADDR
-    || this->id.ipv4.s_addr != this->host_addr.s_addr)
+    if (!(this->id.kind == ID_NONE
+    || (this->id.kind == ID_IPV4_ADDR && same_ip(this->id.ip_addr, this->host_addr))))
     {
 	int len = idtoa(&this->id, host_id+1, sizeof(host_id)-2);
 
@@ -214,7 +362,7 @@ format_end(char *buf, size_t buf_len, struct end *this, struct end *that, bool i
     /* [---hop] */
     hop[0] = '\0';
     hop_sep = "";
-    if (this->host_nexthop.s_addr != that->host_addr.s_addr)
+    if (that != NULL && !same_ip(this->host_nexthop, that->host_addr))
     {
 	addrtoa(this->host_nexthop, 0, hop, sizeof(hop));
 	hop_sep = "---";
@@ -236,32 +384,29 @@ format_end(char *buf, size_t buf_len, struct end *this, struct end *that, bool i
 static void
 unshare_connection_strings(struct connection *c)
 {
-    c->name = clone_bytes(c->name, strlen(c->name) + 1, "connection name");
+    c->name = clone_str(c->name, "connection name");
 
     clone_id_content(&c->this.id);
+    c->this.updown = clone_str(c->this.updown, "updown");
     clone_id_content(&c->that.id);
+    c->that.updown = clone_str(c->that.updown, "updown");
 }
 
 static void
-extract_end(struct end *dst, const struct whack_end *src, const char *which, int whack_fd)
+extract_end(struct end *dst, const struct whack_end *src, const char *which)
 {
     /* decode id, if any */
     if (src->id == NULL)
     {
-	/* default id to be simply the IP address */
-	dst->id = empty_id;
-	dst->id.kind = ID_IPV4_ADDR;
-	dst->id.ipv4 = src->host_addr;
+	dst->id.kind = ID_NONE;
     }
     else
     {
-	const char *ugh = atoid(src->id, &dst->id);
+	complaint_t ugh = atoid(src->id, &dst->id);
 
 	if (ugh != NULL)
 	{
-	    log("bad %s --id: %s (ignored)", which, ugh);
-	    whack_log(whack_fd, RC_BADID
-		, "bad %s --id: %s (ignored)", which, ugh);
+	    loglog(RC_BADID, "bad %s --id: %s (ignored)", which, ugh);
 	    dst->id = empty_id;	/* ignore bad one */
 	}
     }
@@ -273,23 +418,57 @@ extract_end(struct end *dst, const struct whack_end *src, const char *which, int
     dst->client_mask = src->client_mask;
 
     dst->has_client = src->has_client;
-    dst->firewall = src->firewall;
+    dst->updown = src->updown;
     dst->host_port = src->host_port;
 }
 
-void
-add_connection(const struct whack_message *wm, int whack_fd)
+static bool
+rw_check(const struct whack_end *this, const struct whack_end *that
+, const struct whack_message *wm)
 {
-    if (con_by_name(wm->name, NULL_FD) != NULL)
+    if (is_NO_IP(this->host_addr))
     {
-	log("attempt to redefine connection \"%s\"", wm->name);
-	whack_log(whack_fd, RC_DUPNAME, "attempt to redefine connection \"%s\"", wm->name);
+	loglog(RC_ORIENT, "connection must specify host IP address for our side");
+	return FALSE;
     }
-    else if (wm->left.host_addr.s_addr == NO_IP
-    && wm->right.host_addr.s_addr == NO_IP)
+    else
     {
-	log("connection must specify host IP address for our side");
-	whack_log(whack_fd, RC_ORIENT, "connection must specify host IP address for our side");
+	/* check that all RW IKE policies agree because we must implement
+	 * them before the correct connection is known.
+	 * We cannot enforce this for other non-RW connections because
+	 * differentiation is possible when a command specifies which
+	 * to initiate.
+	 */
+	const struct connection *c = find_host_pair_connections(this->host_addr
+	    , this->host_port, mask0.sin_addr, that->host_port);
+
+	for (; c != NULL; c = c->hp_next)
+	{
+	    if ((c->policy ^ wm->policy) & (POLICY_PSK | POLICY_RSASIG))
+	    {
+		loglog(RC_CLASH
+		    , "authentication method disagrees with \"%s\", which is also for an unspecified peer"
+		    , c->name);
+		return FALSE;
+	    }
+	}
+    }
+    return TRUE;	/* happy */
+}
+
+void
+add_connection(const struct whack_message *wm)
+{
+    if (con_by_name(wm->name, FALSE) != NULL)
+    {
+	loglog(RC_DUPNAME, "attempt to redefine connection \"%s\"", wm->name);
+    }
+    else if ((is_NO_IP(wm->left.host_addr)
+	&& !rw_check(&wm->right, &wm->left, wm))
+    || (is_NO_IP(wm->right.host_addr)
+	&& !rw_check(&wm->left, &wm->right, wm)))
+    {
+	/* game over -- give up */
     }
     else
     {
@@ -305,14 +484,14 @@ add_connection(const struct whack_message *wm, int whack_fd)
 	c->sa_rekey_fuzz = wm->sa_rekey_fuzz;
 	c->sa_keying_tries = wm->sa_keying_tries;
 
-	extract_end(&c->this, &wm->left, "left", whack_fd);
-	extract_end(&c->that, &wm->right, "right", whack_fd);
+	extract_end(&c->this, &wm->left, "left");
+	extract_end(&c->that, &wm->right, "right");
 
 	default_end(&c->this, c->that.host_addr);
 	default_end(&c->that, c->this.host_addr);
 
 	/* force any wildcard host IP address to that end */
-	if (wm->left.host_addr.s_addr == NO_IP)
+	if (is_NO_IP(wm->left.host_addr))
 	{
 	    struct end t = c->this;
 
@@ -329,13 +508,16 @@ add_connection(const struct whack_message *wm, int whack_fd)
 	c->newest_ipsec_sa = SOS_NOBODY;
 	c->eroute_owner = SOS_NOBODY;
 
-#ifdef ROAD_WARRIOR_FUDGE
 	c->rw_state = rwcs_permanent;
-	c->parent = NULL;
+
+#ifdef DEBUG
+	c->extra_debugging = wm->debugging;
 #endif
 
 	unshare_connection_strings(c);
 
+	connect_to_host_pair(c);
+
 	/* log all about this connection */
 	log("added connection description \"%s\"", c->name);
 	DBG(DBG_CONTROL,
@@ -359,18 +541,17 @@ add_connection(const struct whack_message *wm, int whack_fd)
     }
 }
 
-#ifdef ROAD_WARRIOR_FUDGE
 struct connection *
-rw_connection(struct connection *c, struct in_addr him)
+rw_connection(const struct connection *c, struct in_addr him)
 {
     struct connection *d = clone_thing(*c, "temporary connection");
 
-    unshare_connection_strings(c);
+    passert(c->rw_state == rwcs_permanent);
+    unshare_connection_strings(d);
 
     DBG(DBG_CONTROL, DBG_log("instantiating \"%s\" for %s"
-	, c->name, inet_ntoa(him)));
+	, d->name, inet_ntoa(him)));
     d->rw_state = rwcs_instance;
-    d->parent = c;
 
     passert(oriented(*d));
     passert(HasWildcardIP(*d));
@@ -391,9 +572,10 @@ rw_connection(struct connection *c, struct in_addr him)
     d->newest_ipsec_sa = SOS_NOBODY;
     d->eroute_owner = SOS_NOBODY;
 
+    connect_to_host_pair(d);
+
     return d;
 }
-#endif /* ROAD_WARRIOR_FUDGE */
 
 bool
 orient(struct connection *c, bool must)
@@ -410,16 +592,17 @@ orient(struct connection *c, bool must)
 	    for (;;)
 	    {
 		/* check if this interface matches this end */
-		if (c->this.host_addr.s_addr == p->addr.s_addr
+		if (same_ip(c->this.host_addr, p->addr)
 		&& (!no_klips || c->this.host_port == pluto_port))
 		{
 		    if (oriented(*c))
 		    {
 			if (c->interface == p)
-			    log("both sides of \"%s\" are our interfaces!"
+			    loglog(RC_LOG_SERIOUS
+				, "both sides of \"%s\" are our interfaces!"
 				, c->name);
 			else
-			    log("two interfaces match \"%s\"", c->name);
+			    loglog(RC_LOG_SERIOUS, "two interfaces match \"%s\"", c->name);
 			c->interface = NULL;	/* withdraw orientation */
 			return FALSE;
 		    }
@@ -427,7 +610,7 @@ orient(struct connection *c, bool must)
 		}
 
 		/* done with this interface if it doesn't match that end */
-		if (!(c->that.host_addr.s_addr == p->addr.s_addr
+		if (!(same_ip(c->that.host_addr, p->addr)
 		&& (!no_klips || c->that.host_port == pluto_port)))
 		    break;
 
@@ -443,7 +626,7 @@ orient(struct connection *c, bool must)
 	if (!oriented(*c))
 	{
 	    if (must)
-		log("cannot find interface for connection \"%s\"", c->name);
+		loglog(RC_LOG_SERIOUS, "cannot find interface for connection \"%s\"", c->name);
 	    return FALSE;
 	}
     }
@@ -453,71 +636,55 @@ orient(struct connection *c, bool must)
 void
 initiate_connection(const char *name, int whackfd)
 {
-    struct connection *c = con_by_name(name, whackfd);
+    struct connection *c = con_by_name(name, TRUE);
 
     if (c != NULL)
     {
-	cur_connection = c;
+	SET_CUR_CONNECTION(c);
 	if (!orient(c, TRUE))
 	{
-	    log("could not orient connection");
-	    whack_log(whackfd, RC_ORIENT, "could not orient connection");
+	    loglog(RC_ORIENT, "could not orient connection");
 	}
-#ifdef ROAD_WARRIOR_FUDGE
 	else if (HasWildcardIP(*c))
 	{
-	    log("cannot initiate connection without peer IP");
-	    whack_log(whackfd
-		, RC_NOPEERIP, "cannot initiate connection without peer IP");
+	    loglog(RC_NOPEERIP, "cannot initiate connection without peer IP");
 	}
-#endif /* ROAD_WARRIOR_FUDGE */
 	else
 	{
 	    /* We will only request an IPsec SA if policy isn't empty
 	     * (ignoring Main Mode items).
 	     * This is a fudge, but not yet important.
+	     * If we are to proceed asynchronously, whackfd will be NULL_FD.
 	     */
-	    ipsecdoi_initiate(whackfd == NULL_FD? NULL_FD : dup(whackfd)
+	    ipsecdoi_initiate(whackfd
 		, c
 		, (c->policy>>POLICY_IPSEC_SHIFT) != 0
 		, c->policy
 		, 1);
 	}
-	cur_connection = NULL;
+	UNSET_CUR_CONNECTION();
     }
 }
 
 void
-terminate_connection(const char *nm, int whackfd)
+terminate_connection(const char *nm)
 {
-#ifdef ROAD_WARRIOR_FUDGE
-    /* Loop because more than one may match in ROAD_WARRIOR_FUDGE.
+    /* Loop because more than one may match (master and instances)
      * But at least one is required (enforced by con_by_name).
      */
     struct connection *c, *n;
 
-    for (c = con_by_name(nm, whackfd); c != NULL; c = n)
+    for (c = con_by_name(nm, TRUE); c != NULL; c = n)
     {
 	n = c->next;	/* grab this before c might disappear */
 	if (strcmp(c->name, nm) == 0)
 	{
-	    cur_connection = c;
+	    SET_CUR_CONNECTION(c);
 	    log("terminating SAs using connection");
 	    delete_states_by_connection(c);
-	    cur_connection = NULL;
+	    UNSET_CUR_CONNECTION();
 	}
     }
-#else /* !ROAD_WARRIOR_FUDGE */
-    struct connection *c = con_by_name(nm, whackfd);
-
-    if (c != NULL)
-    {
-	cur_connection = c;
-	log("terminating SAs using connection");
-	delete_states_by_connection(c);
-	cur_connection = NULL;
-    }
-#endif /* !ROAD_WARRIOR_FUDGE */
 }
 
 /* check nexthop safety
@@ -533,7 +700,7 @@ check_nexthop(const struct connection *c)
 
     if (inside_client(c->this.host_nexthop, c->that))
     {
-	log("cannot perform routing for connection \"%s\""
+	loglog(RC_LOG_SERIOUS, "cannot perform routing for connection \"%s\""
 	    " because nexthop is within peer's client network",
 	    c->name);
 	return FALSE;
@@ -545,7 +712,7 @@ check_nexthop(const struct connection *c)
 	{
 	    if (inside_client(c->this.host_nexthop, d->that))
 	    {
-		log("cannot do routing for connection \"%s\"
+		loglog(RC_LOG_SERIOUS, "cannot do routing for connection \"%s\"
 		    " because nexthop is contained in"
 		    " existing routing for connection \"%s\"",
 		    c->name, d->name);
@@ -553,7 +720,7 @@ check_nexthop(const struct connection *c)
 	    }
 	    if (inside_client(d->this.host_nexthop, c->that))
 	    {
-		log("cannot do routing for connection \"%s\"
+		loglog(RC_LOG_SERIOUS, "cannot do routing for connection \"%s\"
 		    " because it contains nexthop of"
 		    " existing routing for connection \"%s\"",
 		    c->name, d->name);
@@ -600,26 +767,14 @@ route_owner(struct connection *c, bool eroute)
 }
 
 struct connection *
-find_host_connection(struct in_addr me, struct in_addr him)
+find_host_connection(struct in_addr me, u_int16_t my_port
+, struct in_addr him, u_int16_t his_port)
 {
-    struct connection *d;
+    struct connection *c = find_host_pair_connections(me, my_port, him, his_port);
 
-    for (d = connections; d != NULL; d = d->next)
-    {
-	if (d->this.host_addr.s_addr == me.s_addr
-	|| d->that.host_addr.s_addr == me.s_addr)
-	{
-	    if (!oriented(*d))
-		(void)orient(d, FALSE);
-
-	    /* check this.host last -- it is almost certainly us */
-	    if (d->that.host_addr.s_addr == him.s_addr
-	    && d->this.host_addr.s_addr == me.s_addr
-	    && (!no_klips || d->this.host_port == pluto_port))
-		return d;
-	}
-    }
-    return NULL;
+    if (c != NULL && !oriented(*c))
+	orient(c, FALSE);
+    return c;
 }
 
 /* given an up-until-now satisfactory connection, find the best
@@ -627,16 +782,17 @@ find_host_connection(struct in_addr me, struct in_addr him)
  * We can only accept connections with compatible authentication.
  */
 struct connection *
-refine_host_connection(struct state *st, struct id *peer_id, bool initiator)
+refine_host_connection(const struct state *st, const struct id *peer_id
+, bool initiator)
 {
     struct connection *c = st->st_connection;
     u_int16_t auth = st->st_oakley.auth;
-    struct connection *best = NULL;
     struct connection *d;
     lset_t auth_policy;
     const chunk_t *psk;
     const struct RSA_public_key *his_RSA_pub;
     const struct RSA_private_key *my_RSA_pri;
+    bool rw;
 
     if (id_same(&c->that.id, peer_id))
 	return c;
@@ -656,9 +812,7 @@ refine_host_connection(struct state *st, struct id *peer_id, bool initiator)
 	    return NULL;	/* cannot determine my RSA private key! */
 	if (initiator)
 	{
-	    struct id his_addr;	/* for building infrerred ID_IPV4_ADDR */
-
-	    his_RSA_pub = get_RSA_public_key(default_id_to_ip(&his_addr, &c->that));
+	    his_RSA_pub = get_RSA_public_key(&c->that.id);
 	    if (his_RSA_pub == NULL)
 		return NULL;	/* cannot determine his RSA public key! */
 	}
@@ -667,27 +821,18 @@ refine_host_connection(struct state *st, struct id *peer_id, bool initiator)
 	passert(FALSE);
     }
     /* The current connection won't do: search for one that will.
-     * Made more complicated by the fact that the connection might
-     * not be oriented and it might have 0.0.0.0 for peer.
+     * First search for one with the same pair of hosts.
+     * If that fails, search for one with a Road Warrior peer.
      */
-    for (d = connections; d != NULL; d = d->next)
+    d = c->host_pair->connections;
+    for (rw = FALSE; ; rw = TRUE)
     {
-	if (c->this.host_addr.s_addr == d->this.host_addr.s_addr
-	|| c->this.host_addr.s_addr == d->that.host_addr.s_addr)
+	for (; d != NULL; d = d->hp_next)
 	{
-	    /* one end is us -- try to orient & investigate further */
 	    if (!oriented(*d))
 		(void)orient(d, FALSE);
 
-	    if (c->this.host_addr.s_addr == d->this.host_addr.s_addr
-	    && id_same(&c->this.id, &d->this.id)
-	    && id_same(&c->that.id, peer_id)
-	    && (!no_klips || d->this.host_port == pluto_port)
-	    && (c->that.host_addr.s_addr == d->that.host_addr.s_addr
-#ifdef ROAD_WARRIOR_FUDGE
-	       || HasWildcardIP(*c)
-#endif /* ROAD_WARRIOR_FUDGE */
-	    ))
+	    if (same_peer_ids(c, d, peer_id))
 	    {
 		/* check whether authentication would work the same */
 
@@ -704,38 +849,47 @@ refine_host_connection(struct state *st, struct id *peer_id, bool initiator)
 		case OAKLEY_RSA_SIG:
 		    if (my_RSA_pri != get_RSA_private_key(d))
 			continue;	/* different private key */
-		    if (initiator)
-		    {
-			struct id his_addr;	/* for building infrerred ID_IPV4_ADDR */
-
-			if (his_RSA_pub != get_RSA_public_key(default_id_to_ip(&his_addr, &d->that)))
-			    continue;	/* different public key */
-		    }
+		    if (initiator
+		    && his_RSA_pub != get_RSA_public_key(&d->that.id))
+			continue;	/* different public key */
 		    break;
 		default:
 		    passert(FALSE);
 		}
 
-		/* passed all the tests */
-
-#ifdef ROAD_WARRIOR_FUDGE
-		if (HasWildcardIP(*c))
-		{
-		    best = d;	/* RW match -- best so far */
-		}
-		else
-#endif /* ROAD_WARRIOR_FUDGE */
-		{
-		    return d;	/* exact match -- we have a winner */
-		}
+		return d;	/* passed all the tests */
 	    }
 	}
+	if (rw)
+	    break;	/* been around twice already */
+
+	/* for second time around: figure out RW host pair's connections */
+	d = find_host_pair_connections(c->this.host_addr, c->this.host_port
+	    , mask0.sin_addr, c->that.host_port);
     }
-    return best;
+    return NULL;
 }
 
-/* given a connection suitable for ISAKMP (i.e. the hosts match)
+/* Given a connection suitable for ISAKMP (i.e. the hosts match),
  * find a one suitable for IPSEC (i.e. with matching clients).
+ * - c is the starting point
+ * - hp is a connection with the correct pair of hosts.  This cannot
+ *   be the same as c iff we are searching for a wildcarded connection.
+ *
+ * If we don't find an exact match (not even our current connection),
+ * we try for one that still needs instantiation.
+ * This requires inverse instantiation: abstraction.
+ * (1) The peer is abstracted to 0.0.0.0.
+ * (2) If the peer's client subnet just contains the peer
+ *     (peer/32), constrain the peer's client to be 0.0.0.0/32.
+ * (3) If the Phase 1 ID of the peer is simply its IP address,
+ *     and we cannot find a suitable wild-carded connection
+ *     with that ID, allow the peer's ID to be none
+ * Interestingly, we allow a wild card match even if the old
+ * connection was not an instantiation.
+ * Note: whatever ID we happened to choose for our end is
+ * required to stick.  After all, this is the one we used
+ * during authentication.
  */
 struct connection *
 find_client_connection(
@@ -743,18 +897,17 @@ find_client_connection(
     struct in_addr our_net, struct in_addr our_mask,
     struct in_addr peer_net, struct in_addr peer_mask)
 {
-    struct connection
-	*d,
-	*unrouted = NULL;
+    struct connection *hp = c->host_pair->connections;
+    struct id peer_id = c->that.id;
+    struct connection *unrouted = NULL;
+    bool rw;
 
     /* give priority to current connection
      * but even greater priority to a routed connection
      */
 
-    if (c->this.client_net.s_addr == our_net.s_addr
-    && c->this.client_mask.s_addr == our_mask.s_addr
-    && c->that.client_net.s_addr == peer_net.s_addr
-    && c->that.client_mask.s_addr == peer_mask.s_addr)
+    if (client_is(c->this, our_net, our_mask)
+    && client_is(c->that, peer_net, peer_mask))
     {
 	passert(oriented(*c));
 	if (c->routed)
@@ -762,24 +915,18 @@ find_client_connection(
 	unrouted = c;
     }
 
-    for (d = connections; d != NULL; d = d->next)
+    for (rw = FALSE; ; rw = TRUE)
     {
-	if (d->this.host_addr.s_addr == c->this.host_addr.s_addr
-	|| d->that.host_addr.s_addr == c->this.host_addr.s_addr)
+	struct connection *d;
+
+	for (d = hp; d != NULL; d = d->hp_next)
 	{
 	    if (!oriented(*d))
 		(void)orient(d, FALSE);
 
-	    /* check this.host last -- it is almost certainly us */
-	    if (d->that.host_addr.s_addr == c->that.host_addr.s_addr
-	    && d->this.client_net.s_addr == our_net.s_addr
-	    && d->this.client_mask.s_addr == our_mask.s_addr
-	    && d->that.client_net.s_addr == peer_net.s_addr
-	    && d->that.client_mask.s_addr == peer_mask.s_addr
-	    && d->this.host_addr.s_addr == c->this.host_addr.s_addr
-	    && id_same(&d->this.id, &c->this.id)
-	    && id_same(&d->that.id, &c->that.id)
-	    && (!no_klips || d->this.host_port == pluto_port))
+	    if (same_peer_ids(c, d, &peer_id)
+	    && client_is(d->this, our_net, our_mask)
+	    && client_is(d->that, peer_net, peer_mask))
 	    {
 		if (d->routed)
 		    return d;
@@ -787,12 +934,40 @@ find_client_connection(
 		    unrouted = d;
 	    }
 	}
+	if (unrouted != NULL)
+	    break;	/* good enough to preclude more abstract search */
+
+	/* this search failed; try to set up another */
+
+	if (!rw)
+	{
+	    /* try for a road warrior with identical mobile ID */
+	    hp = find_host_connection(c->this.host_addr
+		, c->this.host_port, mask0.sin_addr, c->that.host_port);
+
+	    /* abstract peer_net, if subnet only contains peer.
+	     * peer_mask need not be adjusted.
+	     */
+	    if (same_subnet(peer_net, peer_mask
+	    , c->that.host_addr, mask32.sin_addr))
+		peer_net = mask0.sin_addr;
+	}
+	else if (peer_id.kind == ID_IPV4_ADDR
+	&& same_ip(peer_id.ip_addr, c->that.host_addr))
+	{
+	    /* try again for road warrior, but without ID */
+	    peer_id.kind = ID_NONE;
+	}
+	else
+	{
+	    break;
+	}
     }
     return unrouted;
 }
 
 void
-show_connections_status(int wfd)
+show_connections_status(void)
 {
     struct connection *c;
 
@@ -811,18 +986,18 @@ show_connections_status(int wfd)
 	    if (ll + rl < 70)
 	    {
 		/* display on one line */
-		whack_log(wfd, RC_COMMENT, "\"%s\": %s...%s", c->name, lhs, rhs);
+		whack_log(RC_COMMENT, "\"%s\": %s...%s", c->name, lhs, rhs);
 	    }
 	    else
 	    {
 		/* split over two lines */
-		whack_log(wfd, RC_COMMENT, "\"%s\": %s...", c->name, lhs);
-		whack_log(wfd, RC_COMMENT, "\"%s\": ...%s", c->name, rhs);
+		whack_log(RC_COMMENT, "\"%s\": %s...", c->name, lhs);
+		whack_log(RC_COMMENT, "\"%s\": ...%s", c->name, rhs);
 	    }
 	}
 
-	whack_log(wfd, RC_COMMENT
-	    , "\"%s\": ike_life: %lus; ipsec_life: %lus; rekey_margin: %lus;"
+	whack_log(RC_COMMENT
+	    , "\"%s\":  ike_life: %lus; ipsec_life: %lus; rekey_margin: %lus;"
 	    " rekey_fuzz: %lu%%; keyingtries: %lu"
 	    , c->name
 	    , (unsigned long) c->sa_ike_life_seconds
@@ -832,15 +1007,15 @@ show_connections_status(int wfd)
 	    , (unsigned long) c->sa_keying_tries);
 
 
-	whack_log(wfd, RC_COMMENT
-	    , "\"%s\": policy: %s; interface: %s%s"
+	whack_log(RC_COMMENT
+	    , "\"%s\":  policy: %s; interface: %s%s"
 	    , c->name
 	    , bitnamesof(sa_policy_bit_names, c->policy)
 	    , ifn
 	    , routed);
 
-	whack_log(wfd, RC_COMMENT
-	    , "\"%s\": newest ISAKMP SA: #%ld; newest IPsec SA: #%ld; eroute owner: #%lu"
+	whack_log(RC_COMMENT
+	    , "\"%s\":  newest ISAKMP SA: #%ld; newest IPsec SA: #%ld; eroute owner: #%lu"
 	    , c->name
 	    , c->newest_isakmp_sa
 	    , c->newest_ipsec_sa
