@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: demux.c,v 1.76 1999/12/14 20:31:02 dhr Exp $
+ * RCSID $Id: demux.c,v 1.84 2000/06/21 18:24:32 dhr Exp $
  */
 
 /* Ordering Constraints on Payloads
@@ -317,6 +317,7 @@ static const struct state_microcode state_microcode_table[] = {
     /* STATE_QUICK_R0:
      * HDR*, HASH(1), SA, Ni [, KE ] [, IDci, IDcr ] -->
      * HDR*, HASH(2), SA, Nr [, KE ] [, IDci, IDcr ]
+     * Installs inbound IPsec SAs.
      * ??? it is legal to have multiple SAs, but we don't support it yet.
      */
     { STATE_QUICK_R0, SMF_ALL_AUTH | SMF_ENCRYPTED
@@ -326,7 +327,7 @@ static const struct state_microcode state_microcode_table[] = {
     /* STATE_QUICK_I1:
      * HDR*, HASH(2), SA, Nr [, KE ] [, IDci, IDcr ] -->
      * HDR*, HASH(3)
-     * Installs IPsec SAs.
+     * Installs inbound and outbound IPsec SAs, routing, etc.
      * ??? it is legal to have multiple SAs, but we don't support it yet.
      */
     { STATE_QUICK_I1, SMF_ALL_AUTH | SMF_INITIATOR | SMF_ENCRYPTED
@@ -334,7 +335,7 @@ static const struct state_microcode state_microcode_table[] = {
     , EVENT_SA_REPLACE, quick_inR1_outI2 },
 
     /* STATE_QUICK_R1: HDR*, HASH(3) --> done
-     * Installs IPsec SAs.
+     * Installs outbound IPsec SAs, routing, etc.
      */
     { STATE_QUICK_R1, SMF_ALL_AUTH | SMF_ENCRYPTED
     , P(HASH), LEMPTY, PT(NONE)
@@ -414,7 +415,7 @@ send_packet(struct state *st, const char *where)
 static stf_status
 unexpected(struct msg_digest *md)
 {
-    log("unexpected message received in state %s"
+    loglog(RC_LOG_SERIOUS, "unexpected message received in state %s"
 	, enum_name(&state_names, md->st->st_state));
     return STF_IGNORE;
 }
@@ -422,7 +423,7 @@ unexpected(struct msg_digest *md)
 static stf_status
 informational(struct msg_digest *md UNUSED)
 {
-    log("received and ignored informational message");
+    loglog(RC_LOG_SERIOUS, "received and ignored informational message");
     return STF_IGNORE;
 }
 
@@ -435,9 +436,8 @@ free_md(struct msg_digest *md)
     passert(md->packet_pbs.start != NULL);
     pfree(md->packet_pbs.start);
     md->packet_pbs.start = NULL;
-    whack_log_fd = NULL_FD;
     cur_state = NULL;
-    cur_connection = NULL;
+    UNSET_CUR_CONNECTION();
     cur_from = NULL;
 }
 
@@ -525,15 +525,6 @@ comm_handle(const struct iface *ifp)
     DBG(DBG_RAW,
 	DBG_dump("", md.packet_pbs.start, pbs_room(&md.packet_pbs)));
 
-    if ((pbs_room(&md.packet_pbs) & 03) != 0)
-    {
-	log("IKE packet length must be a multiple of 4"
-	    " but this packet has %d bytes"
-	    , (int) pbs_room(&md.packet_pbs));
-	free_md(&md);
-	return;
-    }
-
     if (!in_struct(&md.hdr, &isakmp_hdr_desc, &md.packet_pbs, &md.message_pbs))
     {
 	/* XXX specific failures (special notification?):
@@ -552,15 +543,6 @@ comm_handle(const struct iface *ifp)
 	return;
     }
 
-    if ((pbs_room(&md.message_pbs) & 03) != 0)
-    {
-	log("IKE message size must be a multiple of 4"
-	    "but this message has %u bytes"
-	    , (unsigned) pbs_room(&md.packet_pbs));
-	free_md(&md);
-	return;
-    }
-
     if (md.hdr.isa_flags & ISAKMP_FLAG_COMMIT)
     {
 	/* XXX we should handle this, whatever it means */
@@ -614,7 +596,7 @@ comm_handle(const struct iface *ifp)
 	{
 	    /* not an initial message */
 
-	    cur_state = md.st = find_state(md.hdr.isa_icookie, md.hdr.isa_rcookie
+	    md.st = find_state(md.hdr.isa_icookie, md.hdr.isa_rcookie
 		, md.sin.sin_addr, md.hdr.isa_msgid);
 
 	    if (md.st == NULL)
@@ -622,7 +604,7 @@ comm_handle(const struct iface *ifp)
 		/* perhaps this is a first message from the responder
 		 * and contains a responder cookie that we've not yet seen.
 		 */
-		cur_state = md.st = find_state(md.hdr.isa_icookie, zero_cookie
+		md.st = find_state(md.hdr.isa_icookie, zero_cookie
 		    , md.sin.sin_addr, md.hdr.isa_msgid);
 
 		if (md.st == NULL)
@@ -633,8 +615,11 @@ comm_handle(const struct iface *ifp)
 		    return;
 		}
 	    }
+	    cur_state = md.st;
+#ifdef DEBUG
+	    extra_debugging(md.st->st_connection);
+#endif
 	    md.from_state = md.st->st_state;
-	    whack_log_fd = md.st->st_whack_sock;
 	}
 	break;
 
@@ -646,7 +631,10 @@ comm_handle(const struct iface *ifp)
     case ISAKMP_XCHG_INFO:	/* an informational exchange */
 	cur_state = md.st = find_state(md.hdr.isa_icookie, md.hdr.isa_rcookie
 	    , md.sin.sin_addr, 0);
-
+#ifdef DEBUG
+	if (md.st != NULL)
+	    extra_debugging(md.st->st_connection);
+#endif
 	if (md.hdr.isa_flags & ISAKMP_FLAG_ENCRYPTION)
 	{
 	    if (md.st == NULL)
@@ -657,11 +645,9 @@ comm_handle(const struct iface *ifp)
 		return;
 	    }
 
-	    whack_log_fd = md.st->st_whack_sock;
-
 	    if (!IS_ISAKMP_SA_ESTABLISHED(md.st->st_state))
 	    {
-		log("encrypted Informational Exchange message is invalid"
+		loglog(RC_LOG_SERIOUS, "encrypted Informational Exchange message is invalid"
 		    " because it is for incomplete ISAKMP SA");
 		/* XXX Could send notification back */
 		free_md(&md);
@@ -670,16 +656,16 @@ comm_handle(const struct iface *ifp)
 
 	    if (md.hdr.isa_msgid == 0)
 	    {
-		log("Informational Exchange message is invalid because"
+		loglog(RC_LOG_SERIOUS, "Informational Exchange message is invalid because"
 		    " it has a Message ID of 0");
 		/* XXX Could send notification back */
 		free_md(&md);
 		return;
 	    }
 
-	    if (!reserve_msgid(md.st->st_connection->that.host_addr, md.hdr.isa_msgid))
+	    if (!reserve_msgid(md.st, md.hdr.isa_msgid))
 	    {
-		log("Informational Exchange message is invalid because"
+		loglog(RC_LOG_SERIOUS, "Informational Exchange message is invalid because"
 		    " it has a previously used Message ID (0x%08lx)"
 		    , (unsigned long)md.hdr.isa_msgid);
 		/* XXX Could send notification back */
@@ -696,7 +682,7 @@ comm_handle(const struct iface *ifp)
 	{
 	    if (md.st != NULL && IS_ISAKMP_SA_ESTABLISHED(md.st->st_state))
 	    {
-		log("Informational Exchange message for"
+		loglog(RC_LOG_SERIOUS, "Informational Exchange message for"
 		    " an established ISAKMP SA must be encrypted");
 		/* XXX Could send notification back */
 		free_md(&md);
@@ -746,7 +732,7 @@ comm_handle(const struct iface *ifp)
 	    cur_state = md.st = find_state(md.hdr.isa_icookie, md.hdr.isa_rcookie
 		, md.sin.sin_addr, 0);
 
-	    if (md.st == (struct state *) NULL)
+	    if (md.st == NULL)
 	    {
 		log("Quick Mode message is for a non-existent (expired?)"
 		    " ISAKMP SA");
@@ -754,10 +740,12 @@ comm_handle(const struct iface *ifp)
 		free_md(&md);
 		return;
 	    }
-
+#ifdef DEBUG
+	    extra_debugging(md.st->st_connection);
+#endif
 	    if (!IS_ISAKMP_SA_ESTABLISHED(md.st->st_state))
 	    {
-		log("Quick Mode message is unacceptable because"
+		loglog(RC_LOG_SERIOUS, "Quick Mode message is unacceptable because"
 		    " it is for an incomplete ISAKMP SA");
 		/* XXX Could send notification back */
 		free_md(&md);
@@ -765,9 +753,9 @@ comm_handle(const struct iface *ifp)
 	    }
 
 	    /* only accept this new Quick Mode exchange if it has a unique message ID */
-	    if (!reserve_msgid(md.sin.sin_addr, md.hdr.isa_msgid))
+	    if (!reserve_msgid(md.st, md.hdr.isa_msgid))
 	    {
-		log("Quick Mode I1 message is unacceptable because"
+		loglog(RC_LOG_SERIOUS, "Quick Mode I1 message is unacceptable because"
 		    " it uses a previously used Message ID 0x%08lx"
 		    " (perhaps this is a duplicated packet)"
 		    , (unsigned long) md.hdr.isa_msgid);
@@ -784,15 +772,17 @@ comm_handle(const struct iface *ifp)
 	}
 	else
 	{
+#ifdef DEBUG
+	    extra_debugging(md.st->st_connection);
+#endif
 	    md.from_state = md.st->st_state;
 	}
 
-	whack_log_fd = md.st->st_whack_sock;
-
 	break;
 
 #ifdef NOTYET
     case ISAKMP_XCHG_NGRP:
+    case ISAKMP_XCHG_ACK_INFO:
 #endif
 
     default:
@@ -839,22 +829,20 @@ comm_handle(const struct iface *ifp)
 	    if (md.st->st_retransmit < MAXIMUM_RETRANSMISSIONS)
 	    {
 		md.st->st_retransmit++;
-		log("retransmitting in response to duplicate packet; already %s"
-		    , enum_name(&state_names, md.st->st_state));
-		whack_log(md.st->st_whack_sock, RC_RETRANSMISSION
-		    , "%s: retransmission in response to duplicate"
+		loglog(RC_RETRANSMISSION
+		    , "retransmitting in response to duplicate packet; already %s"
 		    , enum_name(&state_names, md.st->st_state));
 		send_packet(md.st, "retransmit in response to duplicate");
 	    }
 	    else
 	    {
-		log("discarding duplicate packet -- exhausted retransmission; already %s"
+		loglog(RC_LOG_SERIOUS, "discarding duplicate packet -- exhausted retransmission; already %s"
 		    , enum_name(&state_names, md.st->st_state));
 	    }
 	}
 	else
 	{
-	    log("discarding duplicate packet; already %s"
+	    loglog(RC_LOG_SERIOUS, "discarding duplicate packet; already %s"
 		, enum_name(&state_names, md.st->st_state));
 	}
 	free_md(&md);
@@ -875,7 +863,7 @@ comm_handle(const struct iface *ifp)
 	}
 	if (md.st->st_skeyid_e.ptr == (u_char *) NULL)
 	{
-	    log("discarding encrypted message"
+	    loglog(RC_LOG_SERIOUS, "discarding encrypted message"
 		" because we haven't yet negotiated keying materiel");
 	    /* XXX Could send notification back */
 	    free_md(&md);
@@ -909,7 +897,7 @@ comm_handle(const struct iface *ifp)
 
 	    if (pbs_left(&md.message_pbs) % e->blocksize != 0)
 	    {
-		log("malformed message: not a multiple of encryption blocksize");
+		loglog(RC_LOG_SERIOUS, "malformed message: not a multiple of encryption blocksize");
 		/* XXX Could send notification back */
 		free_md(&md);
 		return;
@@ -945,7 +933,7 @@ comm_handle(const struct iface *ifp)
 
 	if (smc->flags & SMF_INPUT_ENCRYPTED)
 	{
-	    log("packet rejected: should have been encrypted");
+	    loglog(RC_LOG_SERIOUS, "packet rejected: should have been encrypted");
 	    /* XXX Could send notification back */
 	    free_md(&md);
 	    return;
@@ -972,7 +960,7 @@ comm_handle(const struct iface *ifp)
 
 	    if (pd == &md.digest[PAYLIMIT])
 	    {
-		log("more than %d payloads in message; ignored", PAYLIMIT);
+		loglog(RC_LOG_SERIOUS, "more than %d payloads in message; ignored", PAYLIMIT);
 		free_md(&md);
 		return;
 	    }
@@ -987,7 +975,7 @@ comm_handle(const struct iface *ifp)
 			? &isakmp_identification_desc : &isakmp_ipsec_identification_desc;
 		    break;
 		default:
-		    log("%smessage ignored because it contains an unknown or"
+		    loglog(RC_LOG_SERIOUS, "%smessage ignored because it contains an unknown or"
 			" unexpected payload type (%s) at the outermost level"
 			, excuse, enum_show(&payload_names, np));
 		    free_md(&md);
@@ -1001,7 +989,7 @@ comm_handle(const struct iface *ifp)
 		if (0 == (s & (needed | smc->opt_payloads
 		| LELEM(ISAKMP_NEXT_N) | LELEM(ISAKMP_NEXT_D))))
 		{
-		    log("%smessage ignored because it contains an
+		    loglog(RC_LOG_SERIOUS, "%smessage ignored because it contains an
 			"" unexpected payload type (%s) at the outermost level"
 			, excuse, enum_show(&payload_names, np));
 		    free_md(&md);
@@ -1012,7 +1000,7 @@ comm_handle(const struct iface *ifp)
 
 	    if (!in_struct(&pd->payload, sd, &md.message_pbs, &pd->pbs))
 	    {
-		log("%smalformed payload in packet", excuse);
+		loglog(RC_LOG_SERIOUS, "%smalformed payload in packet", excuse);
 		free_md(&md);
 		return;
 	    }
@@ -1050,7 +1038,7 @@ comm_handle(const struct iface *ifp)
 
 	if (needed != 0)
 	{
-	    log("message for %s is missing payloads %s"
+	    loglog(RC_LOG_SERIOUS, "message for %s is missing payloads %s"
 		, enum_show(&state_names, md.from_state)
 		, bitnamesof(payload_name, needed));
 	    free_md(&md);
@@ -1068,7 +1056,7 @@ comm_handle(const struct iface *ifp)
 	if (md.chain[ISAKMP_NEXT_SA] != NULL
 	&& md.hdr.isa_np != ISAKMP_NEXT_SA)
 	{
-	    log("malformed Phase 1 message: does not start with an SA payload");
+	    loglog(RC_LOG_SERIOUS, "malformed Phase 1 message: does not start with an SA payload");
 	    free_md(&md);
 	    return;
 	}
@@ -1092,7 +1080,7 @@ comm_handle(const struct iface *ifp)
 
 	if (md.hdr.isa_np != ISAKMP_NEXT_HASH)
 	{
-	    log("malformed Quick Mode message: does not start with a HASH payload");
+	    loglog(RC_LOG_SERIOUS, "malformed Quick Mode message: does not start with a HASH payload");
 	    free_md(&md);
 	    return;
 	}
@@ -1106,7 +1094,7 @@ comm_handle(const struct iface *ifp)
 	    {
 		if (p != &md.digest[i])
 		{
-		    log("malformed Quick Mode message: SA payload is in wrong position");
+		    loglog(RC_LOG_SERIOUS, "malformed Quick Mode message: SA payload is in wrong position");
 		    free_md(&md);
 		    return;
 		}
@@ -1125,7 +1113,7 @@ comm_handle(const struct iface *ifp)
 	    {
 		if (id->next == NULL || id->next->next != NULL)
 		{
-		    log("malformed Quick Mode message:"
+		    loglog(RC_LOG_SERIOUS, "malformed Quick Mode message:"
 			" if any ID payload is present,"
 			" there must be exactly two");
 		    free_md(&md);
@@ -1133,7 +1121,7 @@ comm_handle(const struct iface *ifp)
 		}
 		if (id+1 != id->next)
 		{
-		    log("malformed Quick Mode message:"
+		    loglog(RC_LOG_SERIOUS, "malformed Quick Mode message:"
 			" the ID payloads are not adjacent");
 		    free_md(&md);
 		    return;
@@ -1151,26 +1139,26 @@ comm_handle(const struct iface *ifp)
 
 	for (p = md.chain[ISAKMP_NEXT_N]; p != NULL; p = p->next)
 	{
-	    log("ignoring informational payload, type %s"
+	    loglog(RC_LOG_SERIOUS, "ignoring informational payload, type %s"
 		, enum_show(&ipsec_notification_names, p->payload.notification.isan_type));
 	    DBG_cond_dump(DBG_PARSING, "info:", p->pbs.cur, pbs_left(&p->pbs));
 	}
 
 	for (p = md.chain[ISAKMP_NEXT_D]; p != NULL; p = p->next)
 	{
-	    log("ignoring Delete SA payload");
+	    loglog(RC_LOG_SERIOUS, "ignoring Delete SA payload");
 	    DBG_cond_dump(DBG_PARSING, "del:", p->pbs.cur, pbs_left(&p->pbs));
 	}
 
 	for (p = md.chain[ISAKMP_NEXT_VID]; p != NULL; p = p->next)
 	{
-	    log("ignoring Vendor ID payload");
+	    loglog(RC_LOG_SERIOUS, "ignoring Vendor ID payload");
 	    DBG_cond_dump(DBG_PARSING, "VID:", p->pbs.cur, pbs_left(&p->pbs));
 	}
 
 	for (p = md.chain[ISAKMP_NEXT_CERT]; p != NULL; p = p->next)
 	{
-	    log("ignoring Certificate payload");
+	    loglog(RC_LOG_SERIOUS, "ignoring Certificate payload");
 	    DBG_cond_dump(DBG_PARSING, "CERT:", p->pbs.cur, pbs_left(&p->pbs));
 	}
     }
@@ -1264,7 +1252,7 @@ comm_handle(const struct iface *ifp)
 		    switch (kind)
 		    {
 		    case EVENT_RETRANSMIT:	/* Retransmit packet */
-			delay = EVENT_RETRANSMIT_DELAY;
+			delay = EVENT_RETRANSMIT_DELAY_0;
 			break;
 
 		    case EVENT_SA_REPLACE:	/* SA replacement event */
@@ -1332,47 +1320,42 @@ comm_handle(const struct iface *ifp)
 		    event_schedule(kind, delay, md.st);
 		}
 
-		/* tell whack what just happened */
 		if (IS_ISAKMP_SA_ESTABLISHED(md.st->st_state)
 		|| IS_IPSEC_SA_ESTABLISHED(md.st->st_state))
 		{
-		    /* Tell Whack about our success,
-		     * and close socket to let Whack finish.
-		     * Note: if the pending_quick directed us to follow
-		     * with Phase 2, the socket fd will have been
-		     * dup'ed by main_inR3().
-		     */
-		    log("%s", state_story[md.st->st_state - STATE_MAIN_R0]);
-		    whack_log(md.st->st_whack_sock, RC_SUCCESS
-			, "%s: SA established"
-			, enum_name(&state_names, md.st->st_state));
+		    /* log our success */
+		    loglog(RC_SUCCESS, "%s: %s"
+			, enum_name(&state_names, md.st->st_state)
+			, state_story[md.st->st_state - STATE_MAIN_R0]);
 		}
 		else
 		{
-		    whack_log(md.st->st_whack_sock, RC_NEW_STATE + md.st->st_state
+		    /* tell whack our progress */
+		    whack_log(RC_NEW_STATE + md.st->st_state
 			, "%s: from %s; %s"
 			, enum_name(&state_names, md.st->st_state)
 			, enum_name(&state_names, md.from_state)
 			, state_story[md.st->st_state - STATE_MAIN_R0]);
 		}
+
 		if (result == STF_UNPEND_QUICK && md.st->st_pending_quick)
 		{
 		    struct connection *c = md.st->st_connection;
 
 		    DBG(DBG_CONTROL, DBG_log("Doing Quick Mode with %s \"%s\""
 			 , inet_ntoa(c->that.host_addr), c->name));
-		    (void) quick_outI1(md.st->st_whack_sock == NULL_FD
-			    ? NULL_FD : dup(md.st->st_whack_sock)
+		    (void) quick_outI1(dup_any(md.st->st_whack_sock)
 			, md.st, c, md.st->st_policy, 1);
 		    md.st->st_pending_quick = FALSE;
 		}
+
 		if (IS_ISAKMP_SA_ESTABLISHED(md.st->st_state)
 		|| IS_IPSEC_SA_ESTABLISHED(md.st->st_state))
 		    release_whack(md.st);
 		break;
 
 	    case STF_INTERNAL_ERROR:
-		whack_log(md.st->st_whack_sock, RC_INTERNALERR + md.note
+		whack_log(RC_INTERNALERR + md.note
 		    , "%s: internal error"
 		    , enum_name(&state_names, md.st->st_state));
 
@@ -1386,7 +1369,7 @@ comm_handle(const struct iface *ifp)
 		/* we've got a distateful DH shared secret --
 		 * let's renegotiate.
 		 */
-		log("dropping and reinitiating exchange to avoid Pluto 1.0 bug"
+		loglog(RC_LOG_SERIOUS, "dropping and reinitiating exchange to avoid Pluto 1.0 bug"
 		    " handling DH shared secret with leading zero byte");
 		ipsecdoi_replace(md.st, md.st->st_try);
 		delete_event(md.st);
@@ -1395,7 +1378,7 @@ comm_handle(const struct iface *ifp)
 		break;
 
 	    case STF_DROP_DOOMED_EXCHANGE:
-		log("dropping exchange to avoid Pluto 1.0 bug"
+		loglog(RC_LOG_SERIOUS, "dropping exchange to avoid Pluto 1.0 bug"
 		    " handling DH shared secret with leading zero byte");
 		delete_event(md.st);
 		delete_state(md.st);
@@ -1412,7 +1395,7 @@ comm_handle(const struct iface *ifp)
 		 * As it is, we act as if this message never happened:
 		 * whatever retrying was in place, remains in place.
 		 */
-		whack_log(md.st->st_whack_sock, RC_NOTIFICATION + md.note
+		whack_log(RC_NOTIFICATION + md.note
 		    , "%s: %s", enum_name(&state_names, md.st->st_state)
 		    , enum_name(&ipsec_notification_names, md.note));
 
