@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: ipsec_doi.c,v 1.82 1999/12/14 20:31:04 dhr Exp $
+ * RCSID $Id: ipsec_doi.c,v 1.105 2000/06/21 18:24:32 dhr Exp $
  */
 
 #include <stdio.h>
@@ -77,7 +77,7 @@ build_and_ship_KE(struct state *st, chunk_t *g
 	if (g->ptr[0] == 0)
 	{
 	    /* generate a new secret to avoid this situation */
-	    log("regenerating DH private secret to avoid Pluto 1.0 bug"
+	    loglog(RC_LOG_SERIOUS, "regenerating DH private secret to avoid Pluto 1.0 bug"
 		" handling public value with leading zero");
 	    mpz_clear(&st->st_sec);
 	    st->st_sec_in_use = FALSE;
@@ -112,7 +112,7 @@ compute_dh_shared(struct state *st, const chunk_t g
     mpz_clear(&mp_shared);
 #ifdef DODGE_DH_MISSING_ZERO_BUG
     if (st->st_shared.ptr[0] == 0)
-	log("shared DH secret has leading zero -- triggers Pluto 1.0 bug");
+	loglog(RC_LOG_SERIOUS, "shared DH secret has leading zero -- triggers Pluto 1.0 bug");
 #endif
     DBG_cond_dump_chunk(DBG_CRYPT, "DH shared secret:\n", st->st_shared);
 }
@@ -134,7 +134,7 @@ accept_KE(chunk_t *dest, const char *val_name, const struct oakley_group_desc *g
 {
     if (pbs_left(pbs) != gr->bytes)
     {
-	log("KE has %u byte DH public value; %u required"
+	loglog(RC_LOG_SERIOUS, "KE has %u byte DH public value; %u required"
 	    , (unsigned) pbs_left(pbs), (unsigned) gr->bytes);
 	/* XXX Could send notification back */
 #ifdef DODGE_DH_MISSING_ZERO_BUG
@@ -162,7 +162,7 @@ accept_PFS_KE(struct msg_digest *md, chunk_t *dest, const char *val_name, const
     {
 	if (st->st_pfs_group != NULL)
 	{
-	    log("missing KE payload in %s message", msg_name);
+	    loglog(RC_LOG_SERIOUS, "missing KE payload in %s message", msg_name);
 	    return INVALID_KEY_INFORMATION;
 	}
     }
@@ -170,13 +170,13 @@ accept_PFS_KE(struct msg_digest *md, chunk_t *dest, const char *val_name, const
     {
 	if (st->st_pfs_group == NULL)
 	{
-	    log("%s message KE payload requires a GROUP_DESCRIPTION attribute in SA"
+	    loglog(RC_LOG_SERIOUS, "%s message KE payload requires a GROUP_DESCRIPTION attribute in SA"
 		, msg_name);
 	    return INVALID_KEY_INFORMATION;
 	}
 	if (ke_pd->next != NULL)
 	{
-	    log("%s message contains several KE payloads; we accept at most one", msg_name);
+	    loglog(RC_LOG_SERIOUS, "%s message contains several KE payloads; we accept at most one", msg_name);
 	    return INVALID_KEY_INFORMATION;	/* ??? */
 	}
 	return accept_KE(dest, val_name, st->st_pfs_group, &ke_pd->pbs);
@@ -292,8 +292,10 @@ main_outI1(
 
     /* set up new state */
     cur_state = st = new_state();
-
     st->st_connection = c;
+#ifdef DEBUG
+    extra_debugging(c);
+#endif
     st->st_pending_quick = pending_quick;
     st->st_policy = policy;
     st->st_whack_sock = whack_sock;
@@ -307,7 +309,7 @@ main_outI1(
     /* put a very short fuse on this state object
      * in case things don't work out.
      */
-    event_schedule(EVENT_SA_EXPIRE, 0, st);
+    event_schedule(EVENT_SO_DISCARD, 0, st);
 
     log("initiating Main Mode");
 
@@ -340,20 +342,18 @@ main_outI1(
 	if (auth_policy == LEMPTY)
 	{
 	    /* unspecified: figure out what we can manage */
-	    struct id his_addr;	/* for building infrerred ID_IPV4_ADDR */
-
 	    if (get_preshared_secret(c) != NULL)
 		auth_policy |= POLICY_PSK;
 
 	    if (get_RSA_private_key(c) != NULL
-	    && get_RSA_public_key(default_id_to_ip(&his_addr, &c->that)) != NULL)
+	    && get_RSA_public_key(&c->that.id) != NULL)
 		auth_policy |= POLICY_RSASIG;
 	    /* Not clear what we should do if neither is possible.
 	     * Perhaps we should not have entered negotiations at all.
 	     */
 	    if (auth_policy == LEMPTY)
 	    {
-		log("we don't know how to authenticate this connection");
+		loglog(RC_LOG_SERIOUS, "we don't know how to authenticate this connection");
 		cur_state = NULL;
 		return STF_INTERNAL_ERROR;
 	    }
@@ -384,9 +384,9 @@ main_outI1(
 
     /* Set up a retransmission event, half a minute henceforth */
     delete_event(st);
-    event_schedule(EVENT_RETRANSMIT, EVENT_RETRANSMIT_DELAY, st);
+    event_schedule(EVENT_RETRANSMIT, EVENT_RETRANSMIT_DELAY_0, st);
 
-    whack_log(st->st_whack_sock, RC_NEW_STATE + STATE_MAIN_I1
+    whack_log(RC_NEW_STATE + STATE_MAIN_I1
 	, "%s: initiate", enum_name(&state_names, st->st_state));
     cur_state = NULL;
     return STF_NO_REPLY;
@@ -408,7 +408,7 @@ ipsecdoi_initiate(
      * XXX issue an event to wait for it to finish and then try
      * XXX to establish it.
      */
-    struct state *st = find_phase1_state(c->that.host_addr);
+    struct state *st = find_isakmp_sa(c);
 
     if (st == NULL)
     {
@@ -440,7 +440,7 @@ ipsecdoi_initiate(
 void
 ipsecdoi_replace(struct state *st, unsigned long try)
 {
-    int whack_sock = st->st_whack_sock == NULL_FD? NULL_FD : dup(st->st_whack_sock);
+    int whack_sock = dup_any(st->st_whack_sock);
     lset_t policy = st->st_policy;
 
     if (IS_PHASE1(st->st_state))
@@ -482,7 +482,7 @@ skeyid_preshared(struct state *st)
 
     if (pss == NULL)
     {
-	log("preshared secret disappeared!");
+	loglog(RC_LOG_SERIOUS, "preshared secret disappeared!");
 	return FALSE;
     }
     else
@@ -656,6 +656,13 @@ main_mode_hash_body(struct state *st, bool hashi, bool hashus
 , union hash_ctx *ctx
 , void (*hash_update)(union hash_ctx *, const u_char *input, unsigned int len))
 {
+#if 0	/* if desperate to debug hashing */
+#   define hash_update(ctx, input, len) { \
+	DBG_dump("hash input", input, len); \
+	(hash_update)(ctx, input, len); \
+	}
+#endif
+
 #   define hash_update_chunk(ctx, ch) hash_update((ctx), (ch).ptr, (ch).len)
     if (hashi)
     {
@@ -711,6 +718,7 @@ main_mode_hash_body(struct state *st, bool hashi, bool hashus
 	hash_update_chunk(ctx, id_b);
     }
 #   undef hash_update_chunk
+#   undef hash_update
 }
 
 static size_t
@@ -822,22 +830,21 @@ RSA_check_signature(struct state *st
     u_char *hash_in_s = &s[sig_len - hash_len];
 
     /* find the public key for peer id */
+    k = get_RSA_public_key(&st->st_connection->that.id);
+    if (k == NULL)
     {
-	struct id his_addr;
+	char buf[200];
 
-	k = get_RSA_public_key(default_id_to_ip(&his_addr, &st->st_connection->that));
-	if (k == NULL)
-	{
-	    log("no RSA public key known");
-	    /* ??? is this the best code there is? */
-	    return INVALID_KEY_INFORMATION;
-	}
+	(void) idtoa(&st->st_connection->that.id, buf, sizeof(buf));
+	loglog(RC_LOG_SERIOUS, "no RSA public key known for '%s'", buf);
+	/* ??? is this the best code there is? */
+	return INVALID_KEY_INFORMATION;
     }
 
     /* decrypt the signature -- reversing RSA_sign_hash */
     if (sig_len != k->k)
     {
-	log("SIG length does not match public key length");
+	loglog(RC_LOG_SERIOUS, "SIG length does not match public key length");
 	return INVALID_KEY_INFORMATION;
     }
 
@@ -859,7 +866,7 @@ RSA_check_signature(struct state *st
      * PKCS#1 v1.5 8.1 encryption-block formatting
      */
     {
-	const char *ugh = NULL;
+	complaint_t ugh = NULL;
 
 	if (s[0] != 0x00)
 	    ugh = "no leading 00";
@@ -900,7 +907,7 @@ RSA_check_signature(struct state *st
 	     * an observer cannot tell what kind of failure happened.
 	     * I don't know what this means in practice.
 	     */
-	    log("SIG did not decrypt into good ECB: %s. Bad key?", ugh);
+	    loglog(RC_LOG_SERIOUS, "SIG did not decrypt into good ECB: %s. Bad key?", ugh);
 	    return INVALID_KEY_INFORMATION;
 	}
     }
@@ -910,7 +917,7 @@ RSA_check_signature(struct state *st
     {
 	DBG_cond_dump(DBG_CRYPT, "decrypted SIG", s, sig_len);
 	DBG_cond_dump(DBG_CRYPT, "computed HASH", hash_val, hash_len);
-	log("received SIG does not match computed HASH (but key is probably correct)");
+	loglog(RC_LOG_SERIOUS, "received SIG does not match computed HASH (but key is probably correct)");
 	/* XXX Could send notification back */
 	return INVALID_HASH_INFORMATION;
     }
@@ -937,7 +944,7 @@ check_main_authenticator(struct msg_digest *md, bool hashi)
 	    {
 		DBG_cond_dump(DBG_CRYPT, "received HASH:"
 		    , hash_pbs->cur, pbs_left(hash_pbs));
-		log("received Hash Payload does not match computed value");
+		loglog(RC_LOG_SERIOUS, "received Hash Payload does not match computed value");
 		/* XXX Could send notification back */
 		return INVALID_HASH_INFORMATION;
 	    }
@@ -967,7 +974,7 @@ check_main_authenticator(struct msg_digest *md, bool hashi)
 	|| memcmp(hash_pbs->cur, hash_val, hash_len) != 0) \
 	{ \
 	    DBG_cond_dump(DBG_CRYPT, "received " hash_name ":", hash_pbs->cur, pbs_left(hash_pbs)); \
-	    log("received " hash_name " does not match computed value in " msg_name); \
+	    loglog(RC_LOG_SERIOUS, "received " hash_name " does not match computed value in " msg_name); \
 	    /* XXX Could send notification back */ \
 	    return STF_FAIL + INVALID_HASH_INFORMATION; \
 	} \
@@ -981,7 +988,7 @@ accept_nonce(struct msg_digest *md, chunk_t *dest, const char *name)
 
     if (len < MINIMUM_NONCE_SIZE || MAXIMUM_NONCE_SIZE < len)
     {
-	log("%s length not between %d and %d"
+	loglog(RC_LOG_SERIOUS, "%s length not between %d and %d"
 	    , name , MINIMUM_NONCE_SIZE, MAXIMUM_NONCE_SIZE);
 	return PAYLOAD_MALFORMED;	/* ??? */
     }
@@ -1119,12 +1126,12 @@ init_phase2_iv(struct state *st, const msgid_t *msgid)
 stf_status
 quick_outI1(
 	int whack_sock,
-	struct state *phase1_st,
+	struct state *isakmp_sa,
 	struct connection *c,
 	lset_t policy,
 	unsigned long try)
 {
-    struct state *st = duplicate_state(phase1_st);
+    struct state *st = duplicate_state(isakmp_sa);
     u_char space[8192];	/* NOTE: we assume 8192 is big enough to build the packet */
     pb_stream reply;	/* not really a reply */
     pb_stream rbody;
@@ -1136,13 +1143,16 @@ quick_outI1(
     cur_state = st;
     st->st_whack_sock = whack_sock;
     st->st_connection = c;
+#ifdef DEBUG
+    extra_debugging(c);
+#endif
     st->st_policy = policy;
     st->st_try = try;
 
     st->st_myuserprotoid = st->st_peeruserprotoid = 0;
     st->st_myuserport = st->st_peeruserport = 0;
 
-    st->st_msgid = generate_msgid(c->that.host_addr);
+    st->st_msgid = generate_msgid(isakmp_sa);
     st->st_state = STATE_QUICK_I1;
 
     insert_state(st);	/* needs cookies, connection, and msgid */
@@ -1178,7 +1188,7 @@ quick_outI1(
      * since no negotiation is possible, we pick one that is
      * very likely supported.
      */
-    st->st_pfs_group = policy & POLICY_PFS? phase1_st->st_oakley.group : NULL;
+    st->st_pfs_group = policy & POLICY_PFS? isakmp_sa->st_oakley.group : NULL;
 
     if (!out_sa(&rbody
     , &ipsec_sadb[(st->st_policy & (POLICY_ENCRYPT | POLICY_AUTHENTICATE | POLICY_TUNNEL)) >> POLICY_IPSEC_SHIFT]
@@ -1252,9 +1262,9 @@ quick_outI1(
 
     /* encrypt message, except for fixed part of header */
 
-    init_phase2_iv(phase1_st, &st->st_msgid);
-    st->st_new_iv_len = phase1_st->st_new_iv_len;
-    memcpy(st->st_new_iv, phase1_st->st_new_iv, st->st_new_iv_len);
+    init_phase2_iv(isakmp_sa, &st->st_msgid);
+    st->st_new_iv_len = isakmp_sa->st_new_iv_len;
+    memcpy(st->st_new_iv, isakmp_sa->st_new_iv, st->st_new_iv_len);
 
     if (!encrypt_message(&rbody, st))
 	return STF_INTERNAL_ERROR;
@@ -1267,15 +1277,17 @@ quick_outI1(
 
     send_packet(st, "quick_outI1");
 
-    event_schedule(EVENT_RETRANSMIT, EVENT_RETRANSMIT_DELAY, st);
+    event_schedule(EVENT_RETRANSMIT, EVENT_RETRANSMIT_DELAY_0, st);
 
-    whack_log(st->st_whack_sock, RC_NEW_STATE + STATE_QUICK_I1
+    whack_log(RC_NEW_STATE + STATE_QUICK_I1
 	, "%s: initiate", enum_name(&state_names, st->st_state));
     cur_state = NULL;
     return STF_NO_REPLY;
 }
 
-/* Decode the ID payload of Phase 1 (main_inI3_outR3 and main_inR3) */
+/* Decode the ID payload of Phase 1 (main_inI3_outR3 and main_inR3)
+ * Note: we may change connections as a result.
+ */
 static bool
 decode_peer_id(struct msg_digest *md, bool initiator)
 {
@@ -1303,7 +1315,7 @@ decode_peer_id(struct msg_digest *md, bool initiator)
 	if (!(id->isaid_doi_specific_a == 0 && id->isaid_doi_specific_b == 0)
 	&& !(id->isaid_doi_specific_a == IPPROTO_UDP && id->isaid_doi_specific_b == IKE_UDP_PORT))
 	{
-	    log("protocol/port in Phase 1 ID Payload must be 0/0 or %d/%d"
+	    loglog(RC_LOG_SERIOUS, "protocol/port in Phase 1 ID Payload must be 0/0 or %d/%d"
 		" but are %d/%d"
 		, IPPROTO_UDP, IKE_UDP_PORT
 		, id->isaid_doi_specific_a, id->isaid_doi_specific_b);
@@ -1311,38 +1323,47 @@ decode_peer_id(struct msg_digest *md, bool initiator)
 	}
 	if (pbs_left(id_pbs) != sizeof(struct in_addr))
 	{
-	    log("size of ID_IPV4_ADDR identification should be %u"
+	    loglog(RC_LOG_SERIOUS, "size of ID_IPV4_ADDR identification should be %u"
 		" but is %d in ID payload"
 		, (unsigned) sizeof(struct in_addr)
 		, (unsigned) pbs_left(id_pbs));
 	    /* XXX Could send notification back */
 	    return FALSE;
 	}
-	memcpy(&peer.ipv4, id_pbs->cur, sizeof(peer.ipv4));
+	memcpy(&peer.ip_addr, id_pbs->cur, sizeof(peer.ip_addr));
 	break;
 
     case ID_USER_FQDN:
 	if (memchr(id_pbs->cur, '@', pbs_left(id_pbs)) == NULL)
 	{
-	    log("peer's ID_USER_FQDN contains no @");
+	    loglog(RC_LOG_SERIOUS, "peer's ID_USER_FQDN contains no @");
 	    return FALSE;
 	}
 	/* FALLTHROUGH */
     case ID_FQDN:
 	if (!(id->isaid_doi_specific_a == 0 && id->isaid_doi_specific_b == 0))
 	{
-	    log("protocol/port in Phase 1 ID Payload must be 0/0"
+	    loglog(RC_LOG_SERIOUS, "protocol/port in Phase 1 ID Payload must be 0/0"
 		" but are %d/%d"
 		, id->isaid_doi_specific_a, id->isaid_doi_specific_b);
 	    return FALSE;
 	}
-	/* ??? ought to do some sanity check, but what? */
+
+	if (memchr(id_pbs->cur, '\0', pbs_left(id_pbs)) != NULL)
+	{
+	    loglog(RC_LOG_SERIOUS, "Phase 1 ID Payload of type %s contains a NUL"
+		, enum_show(&ident_names, peer.kind));
+	    return FALSE;
+	}
+
+	/* ??? ought to do some more sanity check, but what? */
+
 	setchunk(peer.name, id_pbs->cur, pbs_left(id_pbs));
 	break;
 
     default:
 	/* XXX Could send notification back */
-	log("Unacceptable identity type (%s) in Phase 1 ID Payload"
+	loglog(RC_LOG_SERIOUS, "Unacceptable identity type (%s) in Phase 1 ID Payload"
 	    , enum_show(&ident_names, peer.kind));
 	return FALSE;
     }
@@ -1353,10 +1374,10 @@ decode_peer_id(struct msg_digest *md, bool initiator)
 
     DBG(DBG_PARSING,
 	{
-	    char buf[100];
+	    char buf[200];
 
 	    idtoa(&peer, buf, sizeof(buf));
-	    DBG_log("Peer's ID is %s: %s",
+	    DBG_log("Peer's ID is %s: '%s'",
 		enum_show(&ident_names, id->isaid_idtype),
 		buf);
 	});
@@ -1371,10 +1392,10 @@ decode_peer_id(struct msg_digest *md, bool initiator)
 	r = refine_host_connection(st, &peer, initiator);
 	if (r == NULL)
 	{
-	    char buf[100];
+	    char buf[200];
 
 	    idtoa(&peer, buf, sizeof(buf));
-	    log("no suitable connection for peer %s", buf);
+	    loglog(RC_LOG_SERIOUS, "no suitable connection for peer '%s'", buf);
 	    return FALSE;
 	}
 	else if (r != c)
@@ -1383,17 +1404,14 @@ decode_peer_id(struct msg_digest *md, bool initiator)
 
 	    DBG(DBG_CONTROL
 		, DBG_log("switched from \"%s\" to \"%s\"", c->name, r->name));
-#ifdef ROAD_WARRIOR_FUDGE
 	    if (HasWildcardIP(*r))
 	    {
 		/* instantiate it */
 		r = rw_connection(r, c->that.host_addr);
 	    }
-#endif /* ROAD_WARRIOR_FUDGE */
-	    cur_connection = st->st_connection = r;	/* kill reference to c */
-#ifdef ROAD_WARRIOR_FUDGE
+	    st->st_connection = r;	/* kill reference to c */
+	    SET_CUR_CONNECTION(r);
 	    rw_connection_discard(c);
-#endif /* ROAD_WARRIOR_FUDGE */
 	}
     }
 
@@ -1416,7 +1434,7 @@ decode_net_id(
 	case ID_IPV4_ADDR:
 	    if (pbs_left(id_pbs) != sizeof(*net))
 	    {
-		log("%s ID payload ID_IPV4_ADDR wrong length in Quick I1"
+		loglog(RC_LOG_SERIOUS, "%s ID payload ID_IPV4_ADDR wrong length in Quick I1"
 		    , which);
 		/* XXX Could send notification back */
 		return FALSE;
@@ -1430,7 +1448,7 @@ decode_net_id(
 	case ID_IPV4_ADDR_SUBNET:
 	    if (pbs_left(id_pbs) != sizeof(*net) + sizeof(*mask))
 	    {
-		log("%s ID payload ID_IPV4_ADDR_SUBNET wrong length in Quick I1"
+		loglog(RC_LOG_SERIOUS, "%s ID payload ID_IPV4_ADDR_SUBNET wrong length in Quick I1"
 		    , which);
 		/* XXX Could send notification back */
 		return FALSE;
@@ -1447,10 +1465,9 @@ decode_net_id(
 	    break;
 
 	case ID_IPV4_ADDR_RANGE:
-	    /* ??? NOT YET TESTED */
 	    if (pbs_left(id_pbs) != sizeof(*net) + sizeof(*mask))
 	    {
-		log("%s ID payload ID_IPV4_ADDR_RANGE wrong length in Quick I1"
+		loglog(RC_LOG_SERIOUS, "%s ID payload ID_IPV4_ADDR_RANGE wrong length in Quick I1"
 		    , which);
 		/* XXX Could send notification back */
 		return FALSE;
@@ -1469,8 +1486,11 @@ decode_net_id(
 
 		if (start > end)
 		{
-		    log("%s ID payload in Quick I1, ID_IPV4_ADDR_RANGE improper: start is greater than end"
-			, which);
+		    char buf[SUBNETTOA_BUF];	/* length is overkill, but what is right? */
+		    char *lwbs = strncpy(buf, inet_ntoa(*net), sizeof(buf));
+
+		    loglog(RC_LOG_SERIOUS, "%s ID payload in Quick I1, ID_IPV4_ADDR_RANGE %s-%s improper: start is greater than end"
+			, which, lwbs, inet_ntoa(*mask));
 		    return FALSE;
 		}
 		/* (a) iff imask is one less than a power of two (i.e. good),
@@ -1481,8 +1501,13 @@ decode_net_id(
 		 */
 		if (((imask+1) & imask) != 0 || (start & imask) != 0)
 		{
-		    log("%s ID payload in Quick I1, ID_IPV4_ADDR_RANGE unacceptable: not a subnet"
-			, which);
+		    char buf[SUBNETTOA_BUF];	/* length is overkill, but what is right? */
+		    char *lwbs = strncpy(buf, inet_ntoa(*net), sizeof(buf));
+
+		    loglog(RC_LOG_SERIOUS, "%s ID payload in Quick I1,"
+			" ID_IPV4_ADDR_RANGE %s-%s unacceptable:"
+			" not equivalent to a subnet"
+			, which, lwbs, inet_ntoa(*mask));
 		    return FALSE;
 		}
 		mask->s_addr = htonl(~imask);
@@ -1499,7 +1524,7 @@ decode_net_id(
 
 	default:
 	    /* XXX support more */
-	    log("unsupported ID type %s"
+	    loglog(RC_LOG_SERIOUS, "unsupported ID type %s"
 		, enum_show(&ident_names, id->isaiid_idtype));
 	    /* XXX Could send notification back */
 	    return FALSE;
@@ -1521,11 +1546,10 @@ check_net_id(
     if (!decode_net_id(id, id_pbs, &net_temp, &mask_temp, which))
 	return FALSE;
 
-    if (net->s_addr != net_temp.s_addr
-    || mask->s_addr != mask_temp.s_addr
+    if (!same_subnet(*net, *mask, net_temp, mask_temp)
     || *protoid != id->isaiid_protoid || *port != id->isaiid_port)
     {
-	log("%s ID returned doesn't match my proposal", which);
+	loglog(RC_LOG_SERIOUS, "%s ID returned doesn't match my proposal", which);
 	return FALSE;
     }
     return TRUE;
@@ -1692,25 +1716,24 @@ main_inI1_outR1(struct msg_digest *md)
 {
     struct payload_digest *const sa_pd = md->chain[ISAKMP_NEXT_SA];
     struct state *st;
-    struct connection *c = find_host_connection(md->iface->addr, md->sin.sin_addr);
+    struct connection *c = find_host_connection(md->iface->addr, pluto_port
+	, md->sin.sin_addr, ntohs(md->sin.sin_port));
 
     pb_stream r_sa_pbs;
 
     if (c == NULL)
     {
-#ifdef ROAD_WARRIOR_FUDGE
 	/* see if a wildcarded connection can be found */
-	c = find_host_connection(md->iface->addr, mask0.sin_addr);
-	passert(c->parent == NULL);
+	c = find_host_connection(md->iface->addr, pluto_port
+	    , mask0.sin_addr, ntohs(md->sin.sin_port));
 	if (c != NULL)
 	{
 	    /* create a temporary connection that is a copy of this one */
 	    c = rw_connection(c, md->sin.sin_addr);
 	}
 	else
-#endif /* ROAD_WARRIOR_FUDGE */
 	{
-	    log("initial Main Mode message from %s"
+	    loglog(RC_LOG_SERIOUS, "initial Main Mode message from %s"
 		" but no connection has been authorized"
 		, inet_ntoa(md->sin.sin_addr));
 	    /* XXX notification is in order! */
@@ -1721,6 +1744,9 @@ main_inI1_outR1(struct msg_digest *md)
     /* Set up state */
     cur_state = md->st = st = new_state();	/* (caller will reset cur_state) */
     st->st_connection = c;
+#ifdef DEBUG
+    extra_debugging(c);
+#endif
     st->st_try = 0;	/* not our job to try again from start */
     st->st_policy = c->policy;	/* only as accurate as connection */
 
@@ -1732,20 +1758,18 @@ main_inI1_outR1(struct msg_digest *md)
     /* put a very short fuse on this state object
      * in case things don't work out.
      */
-    event_schedule(EVENT_SA_EXPIRE, 0, st);
+    event_schedule(EVENT_SO_DISCARD, 0, st);
 
     st->st_doi = ISAKMP_DOI_IPSEC;
     st->st_situation = SIT_IDENTITY_ONLY; /* We only support this */
     st->st_state = STATE_MAIN_R1;
 
-#ifdef ROAD_WARRIOR_FUDGE
     if (c->rw_state == rwcs_instance)
     {
 	log("responding to Main Mode from Road Warrior %s"
 	    , inet_ntoa(c->that.host_addr));
     }
     else
-#endif /* ROAD_WARRIOR_FUDGE */
     {
 	log("responding to Main Mode");
     }
@@ -1976,7 +2000,10 @@ main_inR2_outI3(struct msg_digest *md)
 		, sig_val, hash_val, hash_len);
 
 	    if (sig_len == 0)
+	    {
+		loglog(RC_LOG_SERIOUS, "unable to locate my private key for RSA Signature");
 		return STF_FAIL + AUTHENTICATION_FAILED;
+	    }
 
 	    if (!out_generic_raw(ISAKMP_NEXT_NONE, &isakmp_signature_desc
 	    , &md->rbody, sig_val, sig_len, "SIG_I"))
@@ -2054,7 +2081,10 @@ main_inI3_outR3(struct msg_digest *md)
 		, sig_val, hash_val, hash_len);
 
 	    if (sig_len == 0)
+	    {
+		loglog(RC_LOG_SERIOUS, "unable to locate my private key for RSA Signature");
 		return STF_FAIL + AUTHENTICATION_FAILED;
+	    }
 
 	    if (!out_generic_raw(ISAKMP_NEXT_NONE, &isakmp_signature_desc
 	    , &md->rbody, sig_val, sig_len, "SIG_R"))
@@ -2111,6 +2141,10 @@ main_inR3(struct msg_digest *md)
  * HDR*, HASH(1), SA, Ni [, KE ] [, IDci, IDcr ] -->
  * HDR*, HASH(2), SA, Nr [, KE ] [, IDci, IDcr ]
  * (see draft-ietf-ipsec-isakmp-oakley-07.txt 5.5)
+ * Installs inbound IPsec SAs.
+ * Although this seems early, we know enough to do so, and
+ * this way we know that it is soon enough to catch all
+ * packets that other side could send using this IPsec SA.
  */
 stf_status
 quick_inI1_outR1(struct msg_digest *md)
@@ -2120,7 +2154,7 @@ quick_inI1_outR1(struct msg_digest *md)
      */
 
     struct state *st = duplicate_state(md->st);
-    struct connection *const c = st->st_connection;
+    struct connection *c = st->st_connection;
     struct payload_digest *const id_pd = md->chain[ISAKMP_NEXT_ID];
     struct in_addr our_net, our_mask, peer_net, peer_mask;
 
@@ -2139,6 +2173,9 @@ quick_inI1_outR1(struct msg_digest *md)
     memcpy(st->st_new_iv, md->st->st_new_iv, st->st_new_iv_len);
 
     cur_state = st;	/* (caller will reset) */
+#ifdef DEBUG
+    extra_debugging(c);
+#endif
     md->st = st;	/* feed back new state */
 
     st->st_policy = c->policy;	/* somebody has got to do it */
@@ -2146,7 +2183,7 @@ quick_inI1_outR1(struct msg_digest *md)
     /* put a very short fuse on this state object
      * in case things don't work out.
      */
-    event_schedule(EVENT_SA_EXPIRE, 0, st);
+    event_schedule(EVENT_SO_DISCARD, 0, st);
 
 
     /* HASH(1) in */
@@ -2179,7 +2216,7 @@ quick_inI1_outR1(struct msg_digest *md)
 
     if ((st->st_policy & POLICY_PFS) && st->st_pfs_group == NULL)
     {
-	log("we require PFS but Quick I1 SA specifies no GROUP_DESCRIPTION");
+	loglog(RC_LOG_SERIOUS, "we require PFS but Quick I1 SA specifies no GROUP_DESCRIPTION");
 	return STF_FAIL + NO_PROPOSAL_CHOSEN;	/* ??? */
     }
 
@@ -2224,52 +2261,58 @@ quick_inI1_outR1(struct msg_digest *md)
 
     /* Now that we have identities of client subnets, we must look for
      * a suitable connection (our current one only matches for hosts).
-     * After this, use p instead of c.
      */
     {
 	struct connection *p = find_client_connection(c
 	    , our_net, our_mask, peer_net, peer_mask);
 
-#ifdef ROAD_WARRIOR_FUDGE
-	if (p == NULL && c->parent != NULL)
-	{
-	    /* We didn't find an exact match, so try for one that still
-	     * needs instantiation.  If the peer's client subnet just
-	     * contains the peer, search for 0.0.0.0/32 as client.
-	     */
-	    p = find_client_connection(c->parent
-		, our_net, our_mask
-		, peer_net.s_addr == c->that.host_addr.s_addr
-		  && peer_mask.s_addr == mask32.sin_addr.s_addr
-		    ? mask0.sin_addr : peer_net
-		, peer_mask);
-	    if (p != NULL)
-	    {
-		/* we found one -- instantiate it */
-		p = rw_connection(p, c->that.host_addr);
-		cur_connection = st->st_connection = p;	/* kill reference to c */
-		rw_connection_discard(c);
-	    }
-	}
-#endif /*  ROAD_WARRIOR_FUDGE */
-
 	if (p == NULL)
 	{
-	    char
-		me[ADDRTOA_BUF], mine[SUBNETTOA_BUF],
-		he[ADDRTOA_BUF], his[SUBNETTOA_BUF];
+	    /* This message occurs in very puzzling circumstances
+	     * so we must add as much information and beauty as we can.
+	     */
+	    struct end
+		me = c->this,
+		he = c->that;
+	    char buf[200];
+	    size_t l;
 
-	    addrtoa(c->this.host_addr, 0, me, sizeof(me));
-	    subnettoa(our_net, our_mask, 0, mine, sizeof(mine));
-	    addrtoa(c->that.host_addr, 0, he, sizeof(he));
-	    subnettoa(peer_net, peer_mask, 0, his, sizeof(mine));
+	    me.client_net = our_net;
+	    me.client_mask = our_mask;
+	    he.client_net = peer_net;
+	    he.client_mask = peer_mask;
+
+	    l = format_end(buf, sizeof(buf), &me, NULL, TRUE);
+	    l += snprintf(buf + l, sizeof(buf) - l, "...");
+	    (void)format_end(buf + l, sizeof(buf) - l, &he, NULL, FALSE);
 	    log("cannot respond to IPsec SA request"
-		" because no connection is known for %s===%s...%s===%s",
-		mine, me, he, his);
+		" because no connection is known for %s"
+		, buf);
 	    return STF_FAIL + INVALID_ID_INFORMATION;
 	}
-	st->st_connection = p;	/* ??? more plumbing needed? */
-	DBG(DBG_CONTROL, DBG_log("using connection \"%s\"", p->name));
+	else if (p != c)
+	{
+	    /* use p, not c */
+
+	    if (HasWildcardIP(*p))
+		p = rw_connection(p, c->that.host_addr);    /* instantiate */
+#ifdef DEBUG
+	    /* temporarily bump up cur_debugging to get "using..." message
+	     * printed if we'd want it with new connection.
+	     */
+	    {
+		unsigned int old_cur_debugging = cur_debugging;
+
+		cur_debugging |= p->extra_debugging;
+		DBG(DBG_CONTROL, DBG_log("using connection \"%s\"", p->name));
+		cur_debugging = old_cur_debugging;
+	    }
+#endif
+	    st->st_connection = p;
+	    SET_CUR_CONNECTION(p);
+	    rw_connection_discard(c);
+	    c = p;
+	}
     }
 
     log("responding to Quick Mode");
@@ -2318,6 +2361,17 @@ quick_inI1_outR1(struct msg_digest *md)
     /* Compute reply HASH(2) and insert in output */
     (void)quick_mode_hash12(r_hashval, r_hash_start, md->rbody.cur, st, TRUE);
 
+    /* Derive new keying material */
+    compute_keymats(st);
+
+    /* Tell the kernel to establish the new inbound SA
+     * (unless the commit bit is set -- which we don't support).
+     * We do this before any state updating so that
+     * failure won't look like success.
+     */
+    if (!install_inbound_ipsec_sa(st))
+	return STF_INTERNAL_ERROR;	/* ??? we may be partly committed */
+
     /* encrypt message, except for fixed part of header */
 
     if (!encrypt_message(&md->rbody, st))
@@ -2333,6 +2387,7 @@ quick_inI1_outR1(struct msg_digest *md)
  * HDR*, HASH(2), SA, Nr [, KE ] [, IDci, IDcr ] -->
  * HDR*, HASH(3)
  * (see draft-ietf-ipsec-isakmp-oakley-07.txt 5.5)
+ * Installs inbound and outbound IPsec SAs, routing, etc.
  */
 stf_status
 quick_inR1_outI2(struct msg_digest *md)
@@ -2400,12 +2455,9 @@ quick_inR1_outI2(struct msg_digest *md)
 	     */
 	    const struct connection *c = st->st_connection;
 
-	    if (c->this.client_net.s_addr != c->this.host_addr.s_addr
-	    || c->this.client_mask.s_addr != mask32.sin_addr.s_addr
-	    || c->that.client_net.s_addr != c->that.host_addr.s_addr
-	    || c->that.client_mask.s_addr != mask32.sin_addr.s_addr)
+	    if (!self_client(c->this) || !self_client(c->that))
 	    {
-		log("IDci, IDcr payloads missing in message"
+		loglog(RC_LOG_SERIOUS, "IDci, IDcr payloads missing in message"
 		    " but default does not match proposal");
 		return STF_FAIL + INVALID_ID_INFORMATION;
 	    }
@@ -2433,8 +2485,8 @@ quick_inR1_outI2(struct msg_digest *md)
     /* Derive new keying material */
     compute_keymats(st);
 
-    /* XXX Unless the commit bit is set, we need to send a message
-     * XXX to the kernel to establish the new SA.
+    /* Tell the kernel to establish the inbound, outbound, and routing part
+     * of the new SA (unless the commit bit is set -- which we don't support).
      * We do this before any state updating so that
      * failure won't look like success.
      */
@@ -2456,6 +2508,7 @@ quick_inR1_outI2(struct msg_digest *md)
 /* Handle last message of Quick Mode.
  * HDR*, HASH(3) -> done
  * (see draft-ietf-ipsec-isakmp-oakley-07.txt 5.5)
+ * Installs outbound IPsec SAs, routing, etc.
  */
 stf_status
 quick_inI2(struct msg_digest *md)
@@ -2466,11 +2519,9 @@ quick_inI2(struct msg_digest *md)
     CHECK_QUICK_HASH(md, quick_mode_hash3(hash_val, st)
 	, "HASH(3)", "Quick I2");
 
-    /* Derive new keying material */
-    compute_keymats(st);
-
-    /* XXX Unless the commit bit is set, we need to send a message
-     * XXX to the kernel to establish the new SA.
+    /* Tell the kernel to establish the outbound and routing part of the new SA
+     * (the previous state established inbound)
+     * (unless the commit bit is set -- which we don't support).
      * We do this before any state updating so that
      * failure won't look like success.
      */
