@@ -11,7 +11,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: ipsec_doi.h,v 1.21 1999/12/13 00:40:53 dhr Exp $
+ * RCSID $Id: ipsec_doi.h,v 1.22 2000/06/21 02:27:28 dhr Exp $
  */
 
 extern void ipsecdoi_initiate(int whack_sock, struct connection *c
@@ -21,7 +21,7 @@ extern void ipsecdoi_replace(struct state *st, unsigned long try);
 
 extern void init_phase2_iv(struct state *st, const msgid_t *msgid);
 
-extern stf_status quick_outI1(int whack_sock, struct state *phase1_st
+extern stf_status quick_outI1(int whack_sock, struct state *isakmp_sa
     , struct connection *c, lset_t policy, unsigned long try);
 
 extern state_transition_fn
