@@ -11,7 +11,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: kernel.h,v 1.10 1999/12/12 22:27:25 dhr Exp $
+ * RCSID $Id: kernel.h,v 1.13 2000/01/17 19:50:22 dhr Exp $
  */
 
 extern bool no_klips;	/* don't actually use KLIPS */
@@ -19,11 +19,12 @@ extern bool no_klips;	/* don't actually use KLIPS */
 extern void pfkey_handle(void);
 
 struct connection;	/* forward declaration of tag */
-extern bool route_connection(struct connection *c);
+extern bool route_connection(struct connection *c, bool doit);
 extern void unroute_connection(struct connection *c);
 
 struct state;	/* forward declaration of tag */
-extern ipsec_spi_t get_ipsec_spi(void);
+extern ipsec_spi_t get_ipsec_spi(ipsec_spi_t avoid);
 
-extern bool install_ipsec_sa(struct state *st, bool initiator);
-extern bool delete_ipsec_sa(struct state *st);
+extern bool install_inbound_ipsec_sa(struct state *st);
+extern bool install_ipsec_sa(struct state *st, bool inbound_also);
+extern bool delete_ipsec_sa(struct state *st, bool inbound_only);
