@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: packet.c,v 1.18 1999/12/13 00:40:54 dhr Exp $
+ * RCSID $Id: packet.c,v 1.22 2000/06/21 18:24:34 dhr Exp $
  */
 
 #include <stdio.h>
@@ -24,6 +24,7 @@
 #include "defs.h"
 #include "log.h"
 #include "packet.h"
+#include "whack.h"	/* for RC_LOG_SERIOUS */
 
 /* ISAKMP Header: for all messages
  * layout from draft-ietf-ipsec-isakmp-09.txt section 3.1
@@ -539,6 +540,7 @@ DBG_print_struct(const char *label, const void *struct_ptr
 	case ft_len:	/* length of this struct and any following crud */
 	case ft_lv:	/* length/value field of attribute */
 	case ft_enum:	/* value from an enumeration */
+	case ft_loose_enum:	/* value from an enumeration with only some names known */
 	case ft_af_enum:	/* Attribute Format + value from an enumeration */
 	case ft_set:	/* bits representing set */
 	    switch (i) {
@@ -568,6 +570,7 @@ DBG_print_struct(const char *label, const void *struct_ptr
 		    immediate = TRUE;
 		/* FALL THROUGH */
 	    case ft_enum:	/* value from an enumeration */
+	    case ft_loose_enum:	/* value from an enumeration with only some names known */
 		DBG_log("   %s: %s", fp->name, enum_show(fp->desc, n));
 		break;
 	    case ft_set:	/* bits representing set */
@@ -653,7 +656,7 @@ in_struct(void *struct_ptr, struct_desc *sd,
     ugh[0] = '\0';
 
     if (ins->roof - cur < (ptrdiff_t)sd->size) {
-	snprintf(ugh, sizeof(ugh), "not enough room in packet for %s", sd->name);
+	snprintf(ugh, sizeof(ugh), "not enough room in input packet for %s", sd->name);
     } else {
 	u_int8_t *roof = cur + sd->size;    /* may be changed by a length field */
 	u_int8_t *outp = struct_ptr;
@@ -687,6 +690,7 @@ in_struct(void *struct_ptr, struct_desc *sd,
 	    case ft_len:	/* length of this struct and any following crud */
 	    case ft_lv:		/* length/value field of attribute */
 	    case ft_enum:	/* value from an enumeration */
+	    case ft_loose_enum:	/* value from an enumeration with only some names known */
 	    case ft_af_enum:	/* Attribute Format + value from an enumeration */
 	    case ft_set:	/* bits representing set */
 	    {
@@ -725,6 +729,8 @@ in_struct(void *struct_ptr, struct_desc *sd,
 			    "%s of %s has an unknown value: %lu",
 			    fp->name, sd->name, (unsigned long)n);
 		    }
+		    /* FALL THROUGH */
+		case ft_loose_enum:	/* value from an enumeration with only some names known */
 		    break;
 		case ft_set:	/* bits representing set */
 		    if (!testset(fp->desc, n)) {
@@ -780,7 +786,7 @@ in_struct(void *struct_ptr, struct_desc *sd,
     }
 
     /* some failure got us here: report it */
-    log(ugh);
+    loglog(RC_LOG_SERIOUS, ugh);
     return FALSE;
 }
 
@@ -789,16 +795,26 @@ in_raw(void *bytes, size_t len, pb_stream *ins, const char *name)
 {
     if (pbs_left(ins) < len)
     {
-	log("not enough bytes left to get %s from %s", name, ins->name);
+	loglog(RC_LOG_SERIOUS, "not enough bytes left to get %s from %s", name, ins->name);
 	return FALSE;
     }
     else
     {
-	memcpy(bytes, ins->cur, len);
-	DBG(DBG_PARSING,
-	    DBG_log("parsing %u raw bytes of %s into %s",
-		(unsigned) len, name, ins->name);
-	    DBG_dump(name, bytes, len));
+	if (bytes == NULL)
+	{
+	    DBG(DBG_PARSING,
+		DBG_log("skipping %u raw bytes of %s (%s)",
+		    (unsigned) len, ins->name, name);
+		DBG_dump(name, ins->cur, len));
+	}
+	else
+	{
+	    memcpy(bytes, ins->cur, len);
+	    DBG(DBG_PARSING,
+		DBG_log("parsing %u raw bytes of %s into %s",
+		    (unsigned) len, ins->name, name);
+		DBG_dump(name, bytes, len));
+	}
 	ins->cur += len;
 	return TRUE;
     }
@@ -832,7 +848,7 @@ out_struct(const void *struct_ptr, struct_desc *sd,
     ugh[0] = '\0';
     if (outs->roof - cur < (ptrdiff_t)sd->size) {
 	snprintf(ugh, sizeof(ugh)
-	    , "not enough room left in packet to place %s", sd->name);
+	    , "not enough room left in output packet to place %s", sd->name);
     } else {
 	bool immediate = FALSE;
 	pb_stream obj;
@@ -862,6 +878,7 @@ out_struct(const void *struct_ptr, struct_desc *sd,
 	    case ft_len:	/* length of this struct and any following crud */
 	    case ft_lv:		/* length/value field of attribute */
 	    case ft_enum:	/* value from an enumeration */
+	    case ft_loose_enum:	/* value from an enumeration with only some names known */
 	    case ft_af_enum:	/* Attribute Format + value from an enumeration */
 	    case ft_set:	/* bits representing set */
 	    {
@@ -905,6 +922,8 @@ out_struct(const void *struct_ptr, struct_desc *sd,
 			    "%s of %s has an unknown value: %lu",
 			    fp->name, sd->name, (unsigned long)n);
 		    }
+		    /* FALL THROUGH */
+		case ft_loose_enum:	/* value from an enumeration with only some names known */
 		    break;
 		case ft_set:	/* bits representing set */
 		    if (!testset(fp->desc, n)) {
@@ -962,7 +981,7 @@ out_struct(const void *struct_ptr, struct_desc *sd,
     }
 
     /* some failure got us here: report it */
-    log(ugh);	/* ??? serious, but errno not relevant */
+    loglog(RC_LOG_SERIOUS, ugh);	/* ??? serious, but errno not relevant */
     return FALSE;
 }
 
@@ -995,7 +1014,7 @@ out_raw(const void *bytes, size_t len, pb_stream *outs, const char *name)
 {
     if (pbs_left(outs) < len)
     {
-	log("not enough room left to place %lu bytes of %s in %s",
+	loglog(RC_LOG_SERIOUS, "not enough room left to place %lu bytes of %s in %s",
 	    (unsigned long) len, name, outs->name);
 	return FALSE;
     }
@@ -1016,7 +1035,7 @@ out_zero(size_t len, pb_stream *outs, const char *name)
 {
     if (pbs_left(outs) < len)
     {
-	log("not enough room left to place %s in %s", name, outs->name);
+	loglog(RC_LOG_SERIOUS, "not enough room left to place %s in %s", name, outs->name);
 	return FALSE;
     }
     else
