@@ -11,7 +11,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: preshared.c,v 1.24 1999/12/14 20:31:06 dhr Exp $
+ * RCSID $Id: preshared.c,v 1.33 2000/06/21 18:24:34 dhr Exp $
  */
 
 #include <stdio.h>
@@ -39,6 +39,7 @@
 #include "state.h"
 #include "preshared.h"
 #include "log.h"
+#include "whack.h"	/* for RC_LOG_SERIOUS */
 
 struct fld {
     const char *name;
@@ -72,7 +73,7 @@ RSA_show_key_fields(struct RSA_private_key *k, int fieldcnt)
 	passert(sz <= sizeof(buf));
 	mpz_get_str(buf, 16, n);
 
-	log("%s: %s", p->name, buf);
+	DBG_log(" %s: %s", p->name, buf);
     }
 }
 
@@ -98,20 +99,22 @@ static const char *
 RSA_private_key_sanity(struct RSA_private_key *k)
 {
     /* note that the *last* error found is reported */
-    const char *ugh = NULL;
+    complaint_t ugh = NULL;
     mpz_t t, u;
 
 #ifdef DEBUG	/* debugging info that compromises security */
     DBG(DBG_PRIVATE, RSA_show_private_key(k));
 #endif
 
-    /* PKCS#1 1.5 section 6 requires modulus to have at least 12 octets */
+    /* PKCS#1 1.5 section 6 requires modulus to have at least 12 octets.
+     * We actually require more (for security).
+     */
     if (k->pub.k < RSA_MIN_OCTETS)
-	return "modulus too small: must be at least 12 octets";
+	return RSA_MIN_OCTETS_UGH;
 
     /* we picked a max modulus size to simplify buffer allocation */
     if (k->pub.k > RSA_MAX_OCTETS)
-	return "modulus too large for Pluto";
+	return RSA_MAX_OCTETS_UGH;
 
     mpz_init(t);
     mpz_init(u);
@@ -184,7 +187,7 @@ struct secret *secrets = NULL;
  * Failure is indicated by a NULL.
  */
 static const struct secret *
-get_secret(struct connection *c, enum PrivateKeyKind kind)
+get_secret(struct connection *c, enum PrivateKeyKind kind, bool asym)
 {
     enum {
 	match_default = 01,
@@ -195,19 +198,19 @@ get_secret(struct connection *c, enum PrivateKeyKind kind)
     struct secret *best = NULL;
     struct secret *s;
     struct id
-	my_addr,
-	*my_id = default_id_to_ip(&my_addr, &c->this),
-	his_addr,
-	*his_id = default_id_to_ip(&his_addr, &c->that);
+	*my_id = &c->this.id,
+	rw_id,
+	*his_id = &c->that.id;
 
-#ifdef ROAD_WARRIOR_FUDGE
-    if (c->rw_state == rwcs_instance && c->that.id.kind == ID_NONE)
+    if (c->rw_state == rwcs_instance
+    && c->that.id.kind == ID_IPV4_ADDR
+    && same_ip(c->that.id.ip_addr, c->that.host_addr))
     {
 	/* roadwarrior: replace him with 0.0.0.0 */
-	passert(his_id == &his_addr);
-	his_addr.ipv4 = mask0.sin_addr;
+	rw_id.kind = ID_IPV4_ADDR;
+	rw_id.ip_addr.s_addr = NO_IP;
+	his_id = &rw_id;
     }
-#endif /* ROAD_WARRIOR_FUDGE */
 
     for (s = secrets; s != NULL; s = s->next)
     {
@@ -247,6 +250,14 @@ get_secret(struct connection *c, enum PrivateKeyKind kind)
 
 	    switch (match)
 	    {
+	    case match_me:
+		/* if this is an asymmetric (eg. public key) system,
+		 * allow this-side-only match to count, even if
+		 * there are other ids in the list.
+		 */
+		if(!asym)
+		    break;
+		/* FALLTHROUGH */
 	    case match_default:	/* default all */
 	    case match_me | match_default:	/* default peer */
 	    case match_me | match_him:	/* explicit */
@@ -260,7 +271,7 @@ get_secret(struct connection *c, enum PrivateKeyKind kind)
 		    && s->u.preshared_secret.len == best->u.preshared_secret.len
 		    && memcmp(s->u.preshared_secret.ptr, best->u.preshared_secret.ptr, s->u.preshared_secret.len) == 0))
 		    {
-			log("multiple ipsec.secrets entries match endpoints:"
+			loglog(RC_LOG_SERIOUS, "multiple ipsec.secrets entries match endpoints:"
 			    " first secret used");
 			best = s;	/* list is backwards: take latest in list */
 		    }
@@ -284,7 +295,7 @@ get_secret(struct connection *c, enum PrivateKeyKind kind)
 const chunk_t *
 get_preshared_secret(struct connection *c)
 {
-    const struct secret *s = get_secret(c, PPK_PSK);
+    const struct secret *s = get_secret(c, PPK_PSK, FALSE);
 
 #ifdef DEBUG
     DBG(DBG_PRIVATE,
@@ -303,7 +314,7 @@ get_preshared_secret(struct connection *c)
 const struct RSA_private_key *
 get_RSA_private_key(struct connection *c)
 {
-    const struct secret *s = get_secret(c, PPK_RSA);
+    const struct secret *s = get_secret(c, PPK_RSA, TRUE);
 
     return s == NULL? NULL : &s->u.RSA_private_key;
 }
@@ -430,7 +441,7 @@ shift(void)
 		p = strchr(p+1, *p);
 		if (p == NULL)
 		{
-		    log("\"%s\" line %d: unterminated string"
+		    loglog(RC_LOG_SERIOUS, "\"%s\" line %d: unterminated string"
 			, sfp->filename, sfp->lino);
 		    p = tok + strlen(tok);
 		}
@@ -499,7 +510,7 @@ flushline(const char *m)
     else
     {
 	if (m != NULL)
-	    log("\"%s\" line %d: %s", sfp->filename, sfp->lino, m);
+	    loglog(RC_LOG_SERIOUS, "\"%s\" line %d: %s", sfp->filename, sfp->lino, m);
 	do ; while (shift());
 	return FALSE;
     }
@@ -509,8 +520,8 @@ static void
 process_key(struct secret *s)
 {
     char buf[2048];
-    const char *ugh = NULL;
-    const char *ugh_extra = "";
+    complaint_t ugh = NULL;
+    complaint_t ugh_extra = "";
     char ugh_space[100];	/* perhaps enough space */
 
     if (*tok == '"' || *tok == '\'')
@@ -608,7 +619,7 @@ process_key(struct secret *s)
 			passert(sz <= sizeof(buf));
 			mpz_get_str(buf, 16, n);
 
-			log("%s: %s", p->name, buf);
+			loglog(RC_LOG_SERIOUS, "%s: %s", p->name, buf);
 		    }
 #endif
 		}
@@ -639,7 +650,7 @@ process_key(struct secret *s)
 
     if (ugh != NULL)
     {
-	log("\"%s\" line %d: %s%s"
+	loglog(RC_LOG_SERIOUS, "\"%s\" line %d: %s%s"
 	    , sfp->filename, sfp->lino, ugh, ugh_extra);
     }
     else if (flushline("expected record boundary in key"))
@@ -672,23 +683,29 @@ process_secret_records(void)
 
 	    if (!shift())
 	    {
-		log("\"%s\" line %d: unexpected end of include directive"
+		loglog(RC_LOG_SERIOUS, "\"%s\" line %d: unexpected end of include directive"
 		    , sfp->filename, sfp->lino);
 		continue;   /* abandon this record */
 	    }
 
+	    /* if path is relative and including file's pathname has
+	     * a non-empty dirname, prefix this path with that dirname.
+	     */
 	    if (tok[0] != '/' && end_prefix != NULL)
 	    {
-		size_t pl = end_prefix - sfp->filename;
+		size_t pl = end_prefix - sfp->filename + 1;
 
+		/* "clamp" length to prevent problems now;
+		 * will be rediscovered and reported later.
+		 */
 		if (pl > sizeof(fn))
-		    pl = sizeof(fn);	/* "clamp" length to prevent problems */
+		    pl = sizeof(fn);
 		memcpy(fn, sfp->filename, pl);
 		p += pl;
 	    }
 	    if (sfp->cur - tok >= &fn[sizeof(fn)] - p)
 	    {
-		log("\"%s\" line %d: include pathname too long"
+		loglog(RC_LOG_SERIOUS, "\"%s\" line %d: include pathname too long"
 		    , sfp->filename, sfp->lino);
 		continue;   /* abandon this record */
 	    }
@@ -731,11 +748,11 @@ process_secret_records(void)
 		     * See RFC2407 IPsec Domain of Interpretation 4.6.2
 		     */
 		    struct id id;
-		    const char *ugh = atoid(tok, &id);
+		    complaint_t ugh = atoid(tok, &id);
 
 		    if (ugh != NULL)
 		    {
-			log("\"%s\" line %d: %s \"%s\""
+			loglog(RC_LOG_SERIOUS, "\"%s\" line %d: %s \"%s\""
 			    , sfp->filename, sfp->lino, ugh, tok);
 		    }
 		    else
@@ -744,12 +761,12 @@ process_secret_records(void)
 
 			i->next = s->ids;
 			s->ids = i;
-			/* DBG_log("id type %d: %s %.*s", i->id.kind, inet_ntoa(i->id.ipv4), (int)i->id.name.len, i->id.name.ptr); */
+			/* DBG_log("id type %d: %s %.*s", i->id.kind, inet_ntoa(i->id.ip_addr), (int)i->id.name.len, i->id.name.ptr); */
 		    }
 		    if (!shift())
 		    {
 			/* unexpected Record Boundary or EOF */
-			log("\"%s\" line %d: unexpected end of id list"
+			loglog(RC_LOG_SERIOUS, "\"%s\" line %d: unexpected end of id list"
 			    , sfp->filename, sfp->lino);
 			break;
 		    }
@@ -777,7 +794,7 @@ process_secrets_file(const char *file_pat)
 
     if (pos.depth > 10)
     {
-	log("preshared secrets file \"%s\" nested too deeply", file_pat);
+	loglog(RC_LOG_SERIOUS, "preshared secrets file \"%s\" nested too deeply", file_pat);
 	return;
     }
 
@@ -790,15 +807,15 @@ process_secrets_file(const char *file_pat)
 	    switch (r)
 	    {
 	    case GLOB_NOSPACE:
-		log("out of space processing secrets filename \"%s\"", file_pat);
+		loglog(RC_LOG_SERIOUS, "out of space processing secrets filename \"%s\"", file_pat);
 		break;
 	    case GLOB_ABORTED:
 		break;	/* already logged */
 	    case GLOB_NOMATCH:
-		log("no secrets filename matched \"%s\"", file_pat);
+		loglog(RC_LOG_SERIOUS, "no secrets filename matched \"%s\"", file_pat);
 		break;
 	    default:
-		log("unknown glob error %d", r);
+		loglog(RC_LOG_SERIOUS, "unknown glob error %d", r);
 		break;
 	    }
 	    globfree(&globbuf);
@@ -956,17 +973,17 @@ unpack_RSA_public_key(struct RSA_public_key *rsa, chunk_t *pubkey)
 	    , (pubkey->ptr[1] << BITS_PER_BYTE) + pubkey->ptr[2]);
     }
 
-    if (pubkey->len - (exp.ptr - pubkey->ptr) < exp.len + RSA_MIN_OCTETS)
+    if (pubkey->len - (exp.ptr - pubkey->ptr) < exp.len + RSA_MIN_OCTETS_RFC)
 	return "RSA public key blob too short";
 
     mod.ptr = exp.ptr + exp.len;
     mod.len = &pubkey->ptr[pubkey->len] - mod.ptr;
 
     if (mod.len < RSA_MIN_OCTETS)
-	return "RSA modulus too short";
+	return RSA_MIN_OCTETS_UGH;
 
     if (mod.len > RSA_MAX_OCTETS)
-	return "RSA modulus too long";
+	return RSA_MAX_OCTETS_UGH;
 
     n_to_mpz(&rsa->e, exp.ptr, exp.len);
     n_to_mpz(&rsa->n, mod.ptr, mod.len);
@@ -1027,11 +1044,11 @@ add_public_key(struct id *id, enum pubkey_alg alg, chunk_t *key)
     {
     case PUBKEY_ALG_RSA:
 	{
-	    const char *ugh = unpack_RSA_public_key(&p->u.rsa, key);
+	    complaint_t ugh = unpack_RSA_public_key(&p->u.rsa, key);
 
 	    if (ugh != NULL)
 	    {
-		log("%s", ugh);
+		loglog(RC_LOG_SERIOUS, "%s", ugh);
 		pfree(p);
 		return;
 	    }
