@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: whack.c,v 1.53 1999/12/14 20:31:07 dhr Exp $
+ * RCSID $Id: whack.c,v 1.61 2000/06/18 21:20:35 dhr Exp $
  */
 
 #include <stdio.h>
@@ -60,7 +60,7 @@ help(void)
 	    "   "
 	    " [--nexthop <ip-address>]"
 	    " [--client <subnet>]"
-	    " [--firewall]"
+	    " [--updown <updown>]"
 	    " \\\n   "
 	    " --to"
 	    " (--host <ip-address> | --id <identity>)"
@@ -69,8 +69,10 @@ help(void)
 	    "   "
 	    " [--nexthop <ip-address>]"
 	    " [--client <subnet>]"
-	    " [--firewall]"
+	    " [--updown <updown>]"
 	    " \\\n   "
+	    " [--psk]"
+	    " [--rsasig]"
 	    " [--encrypt]"
 	    " [--authenticate]"
 	    " [--tunnel]"
@@ -98,7 +100,7 @@ help(void)
 	    "\n\n"
 	"pubkey: whack"
 	    " --keyid <id>"
-	    " --pubkeyrsa <key>"
+	    " [--pubkeyrsa <key>]"
 	    "\n\n"
 #ifdef DEBUG
 	"debug: whack"
@@ -152,7 +154,7 @@ diag(const char *mess)
 
 /* conditially calls diag; prints second arg as quoted string */
 static void
-diagq(const char *ugh, const char *this)
+diagq(complaint_t ugh, const char *this)
 {
     if (ugh != NULL)
     {
@@ -177,7 +179,7 @@ enum {
     OPT_IKEPORT,
     OPT_NEXTHOP,
     OPT_CLIENT,
-    OPT_FIREWALL,	/* last per-end */
+    OPT_UPDOWN,	/* last per-end */
 
 #   define OPT_POLICY_FIRST  OPT_PSK
     OPT_PSK,	/* same order as POLICY_* */
@@ -247,7 +249,7 @@ static const struct option long_opts[] = {
     { "ikeport", required_argument, NULL, OPT_IKEPORT + OO + NUMERIC_ARG },
     { "nexthop", required_argument, NULL, OPT_NEXTHOP + OO },
     { "client", required_argument, NULL, OPT_CLIENT + OO },
-    { "firewall", no_argument, NULL, OPT_FIREWALL + OO },
+    { "updown", required_argument, NULL, OPT_UPDOWN + OO },
 
     { "psk", no_argument, NULL, OPT_PSK + OO },
     { "rsasig", no_argument, NULL, OPT_RSASIG + OO },
@@ -322,6 +324,36 @@ pack_str(char **p)
     }
 }
 
+static void
+check_life_time(time_t life, time_t limit, const char *name
+, const struct whack_message *msg)
+{
+    time_t mint = msg->sa_rekey_margin * (100 + msg->sa_rekey_fuzz) / 100;
+
+    if (life > limit)
+    {
+	char buf[200];
+
+	snprintf(buf, sizeof(buf)
+	    , "%s [%lu seconds] must be less than %lu seconds"
+	    , name, (unsigned long)life, (unsigned long)limit);
+	diag(buf);
+    }
+    if (life <= mint)
+    {
+	char buf[200];
+
+	snprintf(buf, sizeof(buf)
+	    , "%s [%lu] must be greater than"
+	    " rekeymargin*(100+rekeyfuzz)/100 [%lu*(100+%lu)/100]"
+	    , name
+	    , (unsigned long)life
+	    , (unsigned long)msg->sa_rekey_margin
+	    , (unsigned long)msg->sa_rekey_fuzz);
+	diag(buf);
+    }
+}
+
 /* This is a hack for initiating ISAKMP exchanges. */
 
 int
@@ -337,7 +369,9 @@ main(int argc, char **argv)
 
     msg.name = NULL;
     msg.left.id = NULL;
+    msg.left.updown = NULL;
     msg.right.id = NULL;
+    msg.right.updown = NULL;
     msg.keyid = NULL;
     msg.keyval.ptr = NULL;
 
@@ -459,8 +493,8 @@ main(int argc, char **argv)
 	    msg.policy |= POLICY_TUNNEL;	/* client => tunnel */
 	    continue;
 
-	case OPT_FIREWALL:
-	    msg.right.firewall = TRUE;
+	case OPT_UPDOWN:	/* --updown <updown> */
+	    msg.right.updown = optarg;
 	    continue;
 
 
@@ -470,9 +504,11 @@ main(int argc, char **argv)
 		diag("connection missing both --host and --id (before --to)");
 	    msg.left = msg.right;
 	    memset(&msg.right, '\0', sizeof(msg.right));
+	    msg.right.id = NULL;
+	    msg.right.updown = NULL;
 	    msg.right.host_port = IKE_UDP_PORT;
 	    opts_seen_before_to = opts_seen;
-	    opts_seen = (opts_seen & ~LRANGE(OPT_HOST,OPT_FIREWALL));
+	    opts_seen = (opts_seen & ~LRANGE(OPT_HOST,OPT_UPDOWN));
 	    continue;
 
 	case OPT_PSK:		/* --psk */
@@ -505,6 +541,7 @@ main(int argc, char **argv)
 	    continue;
 
 	case OPT_KEYID:	/* --keyid <identity> */
+	    msg.whack_key = TRUE;
 	    msg.keyid = optarg;	/* decoded by Pluto */
 	    continue;
 
@@ -611,19 +648,19 @@ main(int argc, char **argv)
 	if ((opts_seen & (LELEM(OPT_HOST) | LELEM(OPT_ID))) == 0)
 	    diag("connection missing both --host and --id (after --to)");
 
-	if (msg.left.host_addr.s_addr == NO_IP
-	&& msg.right.host_addr.s_addr == NO_IP)
+	if (is_NO_IP(msg.left.host_addr)
+	&& is_NO_IP(msg.right.host_addr))
 	    diag("hosts cannot both be 0.0.0.0");
 
 	if ((opts_seen_before_to & LELEM(OPT_NEXTHOP)) == 0)
 	{
-	    if (msg.right.host_addr.s_addr == NO_IP)
+	    if (is_NO_IP(msg.right.host_addr))
 		diag("left nexthop must be specified when right is Road Warrior");
 	    msg.left.host_nexthop = msg.right.host_addr;
 	}
 	if ((opts_seen & LELEM(OPT_NEXTHOP)) == 0)
 	{
-	    if (msg.left.host_addr.s_addr == NO_IP)
+	    if (is_NO_IP(msg.left.host_addr))
 		diag("right nexthop must be specified when left is Road Warrior");
 	    msg.right.host_nexthop = msg.left.host_addr;
 	}
@@ -631,6 +668,7 @@ main(int argc, char **argv)
 	msg.whack_connection = TRUE;
     }
 
+    /* decide whether --name is mandatory or forbidden */
     if (opts_seen & (LELEM(OPT_ROUTE) | LELEM(OPT_UNROUTE)
     | LELEM(OPT_INITIATE) | LELEM(OPT_TERMINATE)
     | LELEM(OPT_DELETE) | LELEM(OPT_TO)))
@@ -638,17 +676,16 @@ main(int argc, char **argv)
 	if ((opts_seen & LELEM(OPT_NAME)) == 0)
 	    diag("missing --name <connection_name>");
     }
-    else
+    else if (!msg.whack_options)
     {
 	if ((opts_seen & LELEM(OPT_NAME)) != 0)
 	    diag("no reason for --name");
     }
 
-    if (opts_seen & (LELEM(OPT_KEYID) | LELEM(OPT_PUBKEYRSA)))
+    if (opts_seen & LELEM(OPT_PUBKEYRSA))
     {
-	if (!LALLIN(opts_seen, LELEM(OPT_KEYID) | LELEM(OPT_PUBKEYRSA)))
-	    diag("--keyid and --keyval require each other");
-	msg.whack_key = TRUE;
+	if ((opts_seen & LELEM(OPT_KEYID)) == 0)
+	    diag("--pubkeyrsa requires --keyid");
     }
 
     if (!(msg.whack_connection || msg.whack_key || msg.whack_delete
@@ -657,9 +694,7 @@ main(int argc, char **argv)
     || msg.whack_listen || msg.whack_unlisten || msg.whack_status
     || msg.whack_options || msg.whack_shutdown))
     {
-	/* nothing specified -- treat as --help */
-	help();
-	return 0;
+	diag("no action specified; try --help for hints");
     }
 
     /* tricky quick and dirty check for wild values */
@@ -668,29 +703,11 @@ main(int argc, char **argv)
      != msg.sa_rekey_fuzz)
 	diag("rekeymargin or rekeyfuzz values are so large that they cause oveflow");
 
-    {
-	time_t mint = msg.sa_rekey_margin
-	    * (100 + msg.sa_rekey_fuzz) / 100;
+    check_life_time (msg.sa_ike_life_seconds, OAKLEY_ISAKMP_SA_LIFETIME_MAXIMUM
+	, "ikelifetime", &msg);
 
-#	define check_life(life, name) \
-	    if (life <= mint) \
-	    { \
-		char buf[200]; \
-		snprintf(buf, sizeof(buf) \
-		    , name " [%lu] must be greater than" \
-		    " rekeymargin*(100+rekeyfuzz)/100" \
-		    " [%lu*(100+%lu)/100]" \
-		    , (unsigned long)life \
-		    , (unsigned long)msg.sa_rekey_margin \
-		    , (unsigned long)msg.sa_rekey_fuzz); \
-		diag(buf); \
-	    }
-
-	check_life(msg.sa_ike_life_seconds, "ikelifetime");
-	check_life(msg.sa_ipsec_life_seconds, "ipseclifetime");
-
-#	undef check_life
-    }
+    check_life_time(msg.sa_ipsec_life_seconds, SA_LIFE_DURATION_MAXIMUM
+	, "ipseclifetime", &msg);
 
     /* pack strings for inclusion in message */
     next_str = msg.string;
@@ -698,8 +715,10 @@ main(int argc, char **argv)
 
     if (!pack_str(&msg.name)	/* string 1 */
     || !pack_str(&msg.left.id)	/* string 2 */
-    || !pack_str(&msg.right.id)	/* string 3 */
-    || !pack_str(&msg.keyid)	/* string 4 */
+    || !pack_str(&msg.left.updown)	/* string 3 */
+    || !pack_str(&msg.right.id)	/* string 4 */
+    || !pack_str(&msg.right.updown)	/* string 5 */
+    || !pack_str(&msg.keyid)	/* string 6 */
     || str_roof - next_str < (ptrdiff_t)msg.keyval.len)    /* chunk (sort of string 5) */
 	diag("too many bytes of strings to fit in message to pluto");
 
@@ -797,6 +816,7 @@ main(int argc, char **argv)
 			    /* be happy */
 			    exit_status = 0;
 			    break;
+			/* case RC_LOG_SERIOUS: */
 			default:
 			    /* pass through */
 			    exit_status = s;
